Package: BayesSUR
Type: Package
Title: Bayesian Seemingly Unrelated Regression Models in
        High-Dimensional Settings
Version: 2.3-3
URL: https://github.com/mbant/BayesSUR
BugReports: https://github.com/mbant/BayesSUR/issues
Authors@R: c(person("Marco", "Banterle", email = "marco.banterle@gmail.com", role = c("aut")),
             person("Zhi", "Zhao", email = "zhi.zhao@medisin.uio.no", role = c("aut", "cre")),
             person("Leonardo", "Bottolo", email = "lb664@cam.ac.uk", role = c("ctb")),
             person("Sylvia", "Richardson", email = "sylvia.richardson@mrc-bsu.cam.ac.uk", role = c("ctb")),
             person("Waldir", "Leoncio", email = "w.l.netto@medisin.uio.no", role = c("ctb")),
             person("Alex", "Lewin", email = "alex.lewin@lshtm.ac.uk", role = c("aut")),
             person("Manuela", "Zucknick", email = "manuela.zucknick@medisin.uio.no", role = c("aut")))
Description: Bayesian seemingly unrelated regression with general variable selection and dense/sparse covariance matrix. The sparse seemingly unrelated regression is described in Bottolo et al. (2021) <doi:10.1111/rssc.12490>, the software paper is in Zhao et al. (2021) <doi:10.18637/jss.v100.i11>, and the model with random effects is described in Zhao et al. (2024) <doi:10.1093/jrsssc/qlad102>.
License: MIT + file LICENSE
Copyright: The C++ files pugixml.cpp, pugixml.hpp and pugiconfig.hpp
        are used with Copyright (C) 2006-2018 from Arseny Kapoulkine
        and Copyright (C) 2003 from Kristen Wegner. The R function
        vertical.image.legend() is used with Copyright (C) 2013 from
        Jenise Swall.
VignetteBuilder: R.rsp, knitr, rmarkdown
Depends: R (>= 3.5.0)
Encoding: UTF-8
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, igraph, Matrix, tikzDevice, stats, utils, grDevices,
        graphics
Suggests: R.rsp, knitr, rmarkdown, BDgraph, data.table, plyr, scrime
LazyData: true
NeedsCompilation: yes
SystemRequirements: C++17
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-17 18:28:12 UTC; zhiz
Author: Marco Banterle [aut],
  Zhi Zhao [aut, cre],
  Leonardo Bottolo [ctb],
  Sylvia Richardson [ctb],
  Waldir Leoncio [ctb],
  Alex Lewin [aut],
  Manuela Zucknick [aut]
Maintainer: Zhi Zhao <zhi.zhao@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-27 06:30:09 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-27 07:38:31 UTC; unix
Archs: BayesSUR.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,BayesSUR)
S3method(fitted,BayesSUR)
S3method(plot,BayesSUR)
S3method(predict,BayesSUR)
S3method(print,BayesSUR)
S3method(summary,BayesSUR)
export(BayesSUR)
export(elpd)
export(getEstimator)
export(plotCPO)
export(plotEstimator)
export(plotGraph)
export(plotMCMCdiag)
export(plotManhattan)
export(plotNetwork)
importFrom(Matrix,Matrix)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.flush)
importFrom(grDevices,dev.hold)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,gray)
importFrom(grDevices,grey)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,gsize)
importFrom(igraph,layout_in_circle)
importFrom(igraph,layout_with_fr)
importFrom(igraph,plot.igraph)
importFrom(parallel,detectCores)
importFrom(stats,density)
importFrom(stats,sd)
importFrom(tikzDevice,tikz)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(BayesSUR)

# BayesSUR

<!-- badges: start -->

[![CRAN](http://www.r-pkg.org/badges/version/BayesSUR)](https://cran.r-project.org/package=BayesSUR)
[![r-universe](https://ocbe-uio.r-universe.dev/badges/BayesSUR)](https://ocbe-uio.r-universe.dev/BayesSUR)
[![License](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/licenses/MIT)
[![DOI](https://img.shields.io/badge/doi-10.32614%2FCRAN.package.BayesSUR-brightgreen)](https://doi.org/10.32614/CRAN.package.BayesSUR)

<!-- badges: end -->


This R package is for high-dimensional multivariate Bayesian variable and covariance selection in linear regression, including methods in [Bottolo et al. (2021)](https://doi.org/10.1111/rssc.12490), [Zhao et al. (2021)](https://doi.org/10.18637/jss.v100.i11) and [Zhao et al. (2024)](https://doi.org/10.1093/jrsssc/qlad102). 
See the package vignettes [`BayesSUR.pdf`](inst/doc/BayesSUR.pdf) for more information and an additional example below for the BayesSUR model with random effects.

## Installation

Install the latest released version from [CRAN](https://CRAN.R-project.org/package=BayesSUR)

```r
install.packages("BayesSUR")
```

Install the latest development version from [GitHub](https://github.com/mbant/BayesSUR)

```r
#install.packages("remotes")
remotes::install_github("mbant/BayesSUR/BayesSUR")
```

## Additional example

The BayesSUR model has been extended to include mandatory variables by assigning Gaussian priors as random effects rather than spike-and-slab priors, named as **SSUR-MRF with random effects** in [Zhao et al. 2023](https://doi.org/10.1093/jrsssc/qlad102).
The R code for the simulated data and real data analyses in [Zhao et al. 2023](https://doi.org/10.1093/jrsssc/qlad102) can be found at the GitHub repository [BayesSUR-RE](https://github.com/zhizuio/BayesSUR-RE).

Here, we show a simulation example to run the BayesSUR mdoel with random effects.

### Simulate data

We design a network as the following figure (a) to construct a complex structure between $20$ response variables and $300$ predictors.
It assumes that the responses are divided into six groups, and the first $120$ predictors are divided into nine groups.

<img src="man/figures/figure2.png" width="90%" />

_**Figure**: True relationships between response variables and predictors.
(a) Network structure between $\mathbf Y$ and $\mathbf X$.
(b) Spare latent indicator variable $\Gamma$ for the associations between $\mathbf Y$ and $\mathbf X$ in the SUR model.
Black blocks indicate nonzero coefficients and white blocks indicate zero coefficients.
(c) Additional structure in the residual covariance matrix between response variables not explained by $\mathbf X\mathbf B$.
Black blocks indicate correlated residuals of the corresponding response variables and white blocks indicate uncorrelated residuals of the corresponding response variables._

<br>

Load the simulation function `sim.ssur()` as follows.

```{r}
sim.ssur <- function(n, s, p, t0 = 0, seed = 123, mv = TRUE,
                     t.df = Inf, random.intercept = 0, intercept = TRUE) {
  # set seed to fix coefficients
  set.seed(7193)
  sd_b <- 1
  mu_b <- 1
  b <- matrix(rnorm((p + ifelse(t0 == 0, 1, 0)) * s, mu_b, sd_b), p + ifelse(t0 == 0, 1, 0), s)

  # design groups and pathways of Gamma matrix
  gamma <- matrix(FALSE, p + ifelse(t0 == 0, 1, 0), s)
  if (t0 == 0) gamma[1, ] <- TRUE
  gamma[2:6 - ifelse(t0 == 0, 0, 1), 1:5] <- TRUE
  gamma[11:21 - ifelse(t0 == 0, 0, 1), 6:12] <- TRUE
  gamma[31:51 - ifelse(t0 == 0, 0, 1), 1:5] <- TRUE
  gamma[31:51 - ifelse(t0 == 0, 0, 1), 13:15] <- TRUE
  gamma[52:61 - ifelse(t0 == 0, 0, 1), 1:12] <- TRUE
  gamma[71:91 - ifelse(t0 == 0, 0, 1), 6:15] <- TRUE
  gamma[111:121 - ifelse(t0 == 0, 0, 1), 1:15] <- TRUE
  gamma[122 - ifelse(t0 == 0, 0, 1), 16:18] <- TRUE
  gamma[123 - ifelse(t0 == 0, 0, 1), 19] <- TRUE
  gamma[124 - ifelse(t0 == 0, 0, 1), 20] <- TRUE

  G_kron <- matrix(0, s * p, s * p)
  G_m <- bdiag(matrix(1, ncol = 5, nrow = 5),
               matrix(1, ncol = 7, nrow = 7),
               matrix(1, ncol = 8, nrow = 8))
  G_p <- bdiag(matrix(1, ncol = 5, nrow = 5), diag(3),
               matrix(1, ncol = 11, nrow = 11), diag(9),
               matrix(1, ncol = 21, nrow = 21),
               matrix(1, ncol = 10, nrow = 10), diag(9),
               matrix(1, ncol = 21, nrow = 21), diag(19),
               matrix(1, ncol = 11, nrow = 11), diag(181))
  G_kron <- kronecker(G_m, G_p)

  combn11 <- combn(rep((1:5 - 1) * p, each = length(1:5)) +
                     rep(1:5, times = length(1:5)), 2)
  combn12 <- combn(rep((1:5 - 1) * p, each = length(30:60)) +
                     rep(30:60, times = length(1:5)), 2)
  combn13 <- combn(rep((1:5 - 1) * p, each = length(110:120)) +
                     rep(110:120, times = length(1:5)), 2)
  combn21 <- combn(rep((6:12 - 1) * p, each = length(10:20)) +
                     rep(10:20, times = length(6:12)), 2)
  combn22 <- combn(rep((6:12 - 1) * p, each = length(51:60)) +
                     rep(51:60, times = length(6:12)), 2)
  combn23 <- combn(rep((6:12 - 1) * p, each = length(70:90)) +
                     rep(70:90, times = length(6:12)), 2)
  combn24 <- combn(rep((6:12 - 1) * p, each = length(110:120)) +
                     rep(110:120, times = length(6:12)), 2)
  combn31 <- combn(rep((13:15 - 1) * p, each = length(30:50)) +
                     rep(30:50, times = length(13:15)), 2)
  combn32 <- combn(rep((13:15 - 1) * p, each = length(70:90)) +
                     rep(70:90, times = length(13:15)), 2)
  combn33 <- combn(rep((13:15 - 1) * p, each = length(110:120)) +
                     rep(110:120, times = length(13:15)), 2)
  combn4 <- combn(rep((16:18 - 1) * p, each = length(121)) +
                    rep(121, times = length(16:18)), 2)
  combn5 <- matrix(rep((19 - 1) * p, each = length(122)) +
                     rep(122, times = length(19)), nrow = 1, ncol = 2)
  combn6 <- matrix(rep((20 - 1) * p, each = length(123)) +
                     rep(123, times = length(20)), nrow = 1, ncol = 2)

  combnAll <- rbind(t(combn11), t(combn12), t(combn13),
                    t(combn21), t(combn22), t(combn23), t(combn24),
                    t(combn31), t(combn32), t(combn33),
                    t(combn4), combn5, combn6)

  set.seed(seed + 7284)
  sd_x <- 1
  x <- matrix(rnorm(n * p, 0, sd_x), n, p)

  if (t0 == 0 & intercept) x <- cbind(rep(1, n), x)
  if (!intercept) {
    gamma <- gamma[-1, ]
    b <- b[-1, ]
  }
  xb <- matrix(NA, n, s)
  if (mv) {
    for (i in 1:s) {
      if (sum(gamma[, i]) >= 1) {
        if (sum(gamma[, i]) == 1) {
          xb[, i] <- x[, gamma[, i]] * b[gamma[, i], i]
        } else {
          xb[, i] <- x[, gamma[, i]] %*% b[gamma[, i], i]
        }
      } else {
        xb[, i] <- sapply(1:s, function(i) rep(1, n) * b[1, i])
      }
    }
  } else {
    if (sum(gamma) >= 1) {
      xb <- x[, gamma] %*% b[gamma, ]
    } else {
      xb <- sapply(1:s, function(i) rep(1, n) * b[1, i])
    }
  }

  corr_param <- 0.9
  M <- matrix(corr_param, s, s)
  diag(M) <- rep(1, s)

  ## wanna make it decomposable
  Prime <- list(c(1:(s * .4), (s * .8):s),
                c((s * .4):(s * .6)),
                c((s * .65):(s * .75)),
                c((s * .8):s))
  G <- matrix(0, s, s)
  for (i in 1:length(Prime)) {
    G[Prime[[i]], Prime[[i]]] <- 1
  }

  # check
  dimnames(G) <- list(1:s, 1:s)
  length(gRbase::mcsMAT(G - diag(s))) > 0

  var <- solve(BDgraph::rgwish(n = 1, adj = G, b = 3, D = M))

  # change seeds to add randomness on error
  set.seed(seed + 8493)
  sd_err <- 0.5
  if (is.infinite(t.df)) {
    err <- matrix(rnorm(n * s, 0, sd_err), n, s) %*% chol(as.matrix(var))
  } else {
    err <- matrix(rt(n * s, t.df), n, s) %*% chol(as.matrix(var))
  }

  if (t0 == 0) {
    b.re <- NA
    z <- NA
    y <- xb + err
    if (random.intercept != 0) {
      y <- y + matrix(rnorm(n * s, 0, sqrt(random.intercept)), n, s)
    }

    z <- sample(1:4, n, replace = T, prob = rep(1 / 4, 4))

    return(list(y = y, x = x, b = b, gamma = gamma, z = model.matrix(~ factor(z) + 0)[, ],
                b.re = b.re, Gy = G, mrfG = combnAll))
  } else {
    # add random effects
    z <- t(rmultinom(n, size = 1, prob = c(.1, .2, .3, .4)))
    z <- sample(1:t0, n, replace = T, prob = rep(1 / t0, t0))
    set.seed(1683)
    b.re <- rnorm(t0, 0, 2)
    y <- matrix(b.re[z], nrow = n, ncol = s) + xb + err

    return(list(
      y = y, x = x, b = b, gamma = gamma, z = model.matrix(~ factor(z) + 0)[, ],
      b.re = b.re, Gy = G, mrfG = combnAll
    ))
  }
}
```

To simulate data with sample size $n=250$, responsible variables $s=20$ and covariates $p=300$, we can specify the corresponding parameters in the function `sim.ssur()` as follows.

```{r}
library("BayesSUR")
library("Matrix")
n <- 250
s <- 20
p <- 300
sim1 <- sim.ssur(n, s, p, seed = 1)
```

To simulate data from $4$ individual groups with group indicator variables following the defaul multinomial distribution $multinomial(0.1,0.2,0.3,0.4)$, we can simply add the argument `t0 = 4` in the function `sim.ssur()` as follows.

```{r}
t0 <- 4
sim2 <- sim.ssur(n, s, p, t0, seed = 1) # learning data
sim2.val <- sim.ssur(n, s, p, t0, seed=101) # validation data
```

### Run BayesSUR model with random effects

According to the guideline of prior specification in [Zhao et al. 2023](https://doi.org/10.1093/jrsssc/qlad102), we first set the following parameters `hyperpar` and then running the BayesSUR model with random effects via `betaPrior = "reGroup"` (default `betaPrior = "independent"` with spike-and-slab priors for all coefficients). 
**For illustration, we run a short MCMC** with `nIter = 300` and `burnin = 100`. 
Note that here the graph used for the Markov random field prior is the true graph from the returned object of the simulation `sim2$mrfG`.

```{r}
hyperpar <- list(mrf_d = -2, mrf_e = 1.6, a_w0 = 100, b_w0 = 500, a_w = 15, b_w = 60)
set.seed(1038)
fit2 <- BayesSUR(
  data = cbind(sim2$y, sim2$z, sim2$x),
  Y = 1:s,
  X_0 = s + 1:t0,
  X = s + t0 + 1:p,
  outFilePath = "sim2_mrf_re",
  hyperpar = hyperpar,
  gammaInit = "0",
  betaPrior = "reGroup",
  nIter = 300, burnin = 100,
  covariancePrior = "HIW",
  standardize = F,
  standardize.response = F,
  gammaPrior = "MRF",
  mrfG = sim2$mrfG,
  output_CPO = T
)
```

```
## BayesSUR -- Bayesian Seemingly Unrelated Regression Modelling
## Reading input files ... ... successfull!
## Clearing and initialising output files 
## Initialising the (SUR) MCMC Chain ...  ...  DONE!
## Drafting the output files with the start of the chain ... DONE! 
## 
## Starting 2 (parallel) chain(s) for 300 iterations:
## Temperature ladder updated, new temperature ratio : 1.1
##  MCMC ends.   --- Saving results and exiting
## Saved to :   sim2_mrf_re/data_SSUR_****_out.txt
## Final w0  : 4.9971
## Final w   : 2.29497
## Final tau : 0.293487    w/ proposal variance: 1.2229
## Final eta : 0.0471505
##   -- Average Omega : 0
## Final temperature ratio : 1.1
## 
## DONE, exiting!
```

Check some summarized information of the results:

```{r}
summary(fit2)
```

```
## Call:
##   BayesSUR(data = cbind(sim2$y, sim2$z, sim2$x), ...)
## 
## CPOs:
##         Min.      1st Qu.       Median      3rd Qu.         Max. 
## 0.0001896996 0.0242732651 0.0348570615 0.0465600279 0.1312571329 
## 
## Number of selected predictors (mPIP > 0.5): 19 of 20x300
## 
## Top 10 predictors on average mPIP across all responses:
##     X.74     X.69     X.77     X.82    X.114    X.116    X.122    X.157    X.265 
## 0.081840 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 
##     X.16 
## 0.047015 
## 
## Top 10 responses on average mPIP across all predictors:
##         X.8        X.10         X.5        X.19        X.11         X.9         X.6 
## 0.012703000 0.010049000 0.007943333 0.006600000 0.005522333 0.004029667 0.003001667 
##        X.12         X.2         X.4 
## 0.002852333 0.002769667 0.002404667 
## 
## Expected log pointwise predictive density (elpd) estimates:
##   elpd.LOO = -16803.49,  elpd.WAIC = -16799.94
## 
## MCMC specification:
##   iterations = 300,  burn-in = 100,  chains = 2
##   gamma local move sampler: bandit
##   gamma initialisation: 0
## 
## Model specification:
##   covariance prior: HIW
##   gamma prior: MRF
## 
## Hyper-parameters:
##   a_w   b_w    nu a_tau b_tau a_eta b_eta mrf_d mrf_e  a_w0  b_w0 
##  15.0  60.0  22.0   0.1  10.0   0.1   1.0  -2.0   1.6 100.0 500.0
```

Show the estimates of regression coefficients, variable selection indicators and residual graph

```{r}
# show estimates
plot(fit2, estimator=c("beta","gamma","Gy"), type="heatmap", name.predictors = "auto")
```
![<font size="2">_Simulation result: Estimates of regression coefficients, variable selection indicators and residual graph._</font>](man/figures/figureS.png)

Compute the model performace with respect to **variable selection**

```{r}
# compute accuracy, sensitivity, specificity of variable selection
gamma <- getEstimator(fit2)
(accuracy <- sum(data.matrix(gamma > 0.5) == sim2$gamma) / prod(dim(gamma)))
```

```
## [1] 0.8725
```

```{r}
(sensitivity <- sum((data.matrix(gamma > 0.5) == 1) & (sim2$gamma == 1)) / sum(sim2$gamma == 1))
```

```
## [1] 0.01558442
```

```{r}
(specificity <- sum((data.matrix(gamma > 0.5) == 0) & (sim2$gamma == 0)) / sum(sim2$gamma == 0))
```

```
## [1] 0.9986616
```

Compute the model performance with respect to **response prediction**

```{r}
# compute RMSE and RMSPE for prediction performance
beta <- getEstimator(fit2, estimator = "beta", Pmax = .5, beta.type = "conditional")
(RMSE <- sqrt(sum((sim2$y - cbind(sim2$z, sim2$x) %*% beta)^2) / prod(dim(sim2$y))))
```

```
## [1] 8.723454
```

```{r}
(RMSPE <- sqrt(sum((sim2.val$y - cbind(sim2.val$z, sim2.val$x) %*% beta)^2) / prod(dim(sim2.val$y))))
```

```
## [1] 8.859939
```

Compute the model performance with respect to **coefficient bias**

```{r}
# compute bias of beta estimates
b <- sim2$b
b[sim2$gamma == 0] <- 0
(beta.l2 <- sqrt(sum((beta[-c(1:4), ] - b)^2) / prod(dim(b))))
```

```
## [1] 0.5039502
```

Compute the model performance with respect to **covariance selection**

```{r}
g.re <- getEstimator(fit2, estimator = "Gy")
(g.accuracy <- sum((g.re > 0.5) == sim2$Gy) / prod(dim(g.re)))
```

```
## [1] 0.585
```

```{r}
(g.sensitivity <- sum(((g.re > 0.5) == sim2$Gy)[sim2$Gy == 1]) / sum(sim2$Gy == 1))
```

```
## [1] 0.2475248
```

```{r}
(g.specificity <- sum(((g.re > 0.5) == sim2$Gy)[sim2$Gy == 0]) / sum(sim2$Gy == 0))
```

```
## [1] 0.9292929
```

## References

> Leonardo Bottolo, Marco Banterle, Sylvia Richardson, Mika Ala-Korpela, Marjo-Riitta Järvelin, Alex Lewin (2021).
> A computationally efficient Bayesian seemingly unrelated regressions model for high-dimensional quantitative trait loci discovery.
> _Journal of the Royal Statistical Society: Series C (Applied Statistics)_, 70(4):886-908. DOI: [10.1111/rssc.12490](https://doi.org/10.1111/rssc.12490).

> Zhi Zhao, Marco Banterle, Leonardo Bottolo, Sylvia Richardson, Alex Lewin, Manuela Zucknick (2021).
> BayesSUR: An R package for high-dimensional multivariate Bayesian variable and covariance selection in linear regression.
> _Journal of Statistical Software_, 100(11):1-32. DOI: [10.18637/jss.v100.i11](https://doi.org/10.18637/jss.v100.i11).

> Zhi Zhao, Marco Banterle, Alex Lewin, Manuela Zucknick (2023).
> Multivariate Bayesian structured variable selection for pharmacogenomic studies.
> _Journal of the Royal Statistical Society: Series C (Applied Statistics)_, 73(2):420-443 qlad102. DOI: [10.1093/jrsssc/qlad102](https://doi.org/10.1093/jrsssc/qlad102).

#' @title get fitted responses
#' @description
#' Return the fitted response values that correspond to the posterior mean 
#' estimates from a \code{BayesSUR} class object.
#' @name fitted.BayesSUR
#' 
#' @param object an object of class \code{BayesSUR}
#' @param beta.type type of estimated beta for the fitted model. Default is 
#' \code{marginal}, giving marginal beta estimation. If 
#' \code{beta.type="conditional"}, it gives beta estimation conditional 
#' on gamma=1
#' @param Pmax valid if \code{beta.type="conditional"}. If 
#' \code{beta.type="conditional"} and \code{Pmax=0.5}, it gives median 
#' probability model betas. Default is 0
#' @param ... other arguments
#'
#' @return Fitted values extracted from an object of class \code{BayesSUR}. If 
#' the \code{BayesSUR} specified data standardization, the fitted values are 
#' base based on standardized data.
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check fitted values
#' fitted.val <- fitted(fit)
#'
#' @export
fitted.BayesSUR <- function(object, Pmax = 0, beta.type = "marginal", ...) {
  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify correct argument 'Pmax' in [0,1]!")
  }
  if ((Pmax > 0) && (beta.type == "marginal")) {
    stop("Pmax > 0 is valid only if the argument beta.type = 'conditional'!")
  }

  beta_hat <- getEstimator(object, estimator = "beta", Pmax = Pmax, 
                           beta.type = beta.type, ...)

  object$output[-1] <- 
    paste(object$output$outFilePath, object$output[-1], sep = "")
  X <- as.matrix(read.table(object$output$X, header = TRUE))

  if ("X0" %in% names(object$output)) {
    X0 <- as.matrix(read.table(object$output$X0))
  } else {
    X0 <- NULL
  }

  y.pred <- cbind(X0, X) %*% beta_hat

  return(y.pred)
}

#' @title indices list of target genes in the GDSC data set
#'
#' @description
#' Indices list of target genes corresponding the \code{example_GDSC} data set. 
#' It has two components representing the gene indices of the MAPK/ERK pathway and 
#' BCR-ABL gene fusion in the \code{example_GDSC} data set.
#'
#' @examples
#' # Load the indices of gene targets from the GDSC sample dataset
#' data("targetGene", package = "BayesSUR")
#' str(targetGene)
#'
#' \dontrun{
#' # ===============
#' # This code below is to do preprocessing of GDSC data and obtain the complete dataset
#' # "targetGene.rda" above. The user needs load the datasets from
#' # https://cellmodelpassports.sanger.ac.uk/downloads.
#' # But downloading and transforming the three used datasets below to *.csv files first.
#' # ===============
#'
#' requireNamespace("plyr", quietly = TRUE)
#' requireNamespace("data.table", quietly = TRUE)
#'
#'
#' features <- data.frame(read.csv("/gdsc_en_input_w5.csv", head = T))
#' names.fea <- strsplit(rownames(features), "")
#' features <- t(features)
#' p <- c(13321, 13747 - 13321, 13818 - 13747)
#' Cell.Line <- rownames(features)
#' features <- data.frame(Cell.Line, features)
#'
#' ic50_00 <- data.frame(read.csv("gdsc_drug_sensitivity_fitted_data_w5.csv", head = T))
#' ic50_0 <- ic50_00[, c(1, 4, 7)]
#' drug.id <- data.frame(read.csv("gdsc_tissue_output_w5.csv", head = T))[, c(1, 3)]
#' drug.id2 <- drug.id[!duplicated(drug.id$drug.id), ]
#' # delete drug.id=1066 since ID1066 and ID156 both correspond drug AZD6482,
#' # and no ID1066 in the "suppl.Data1" by Garnett et al. (2012)
#' drug.id2 <- drug.id2[drug.id2$drug.id != 1066, ]
#' drug.id2$drug.name <- as.character(drug.id2$drug.name)
#' drug.id2$drug.name <- substr(drug.id2$drug.name, 1, nchar(drug.id2$drug.name) - 6)
#' drug.id2$drug.name <- gsub(" ", "-", drug.id2$drug.name)
#'
#' ic50 <- ic50_0
#' # mapping the drug_id to drug names in drug sensitivity data set
#' ic50$drug_id <- plyr::mapvalues(ic50$drug_id, from = drug.id2[, 2], to = drug.id2[, 1])
#' colnames(ic50) <- c("Cell.Line", "compound", "IC50")
#'
#' # transform drug sensitivity overall cell lines to a data matrix
#' y0 <- reshape(ic50, v.names = "IC50", timevar = "compound", 
#'                     idvar = "Cell.Line", direction = "wide")
#' y0$Cell.Line <- gsub("-", ".", y0$Cell.Line)
#'
#' # ===============
#' # select nonmissing pharmacological data
#' # ===============
#' y00 <- y0
#' m0 <- dim(y0)[2] - 1
#' eps <- 0.05
#' # r1.na is better to be not smaller than r2.na
#' r1.na <- 0.3
#' r2.na <- 0.2
#' k <- 1
#' while (sum(is.na(y0[, 2:(1 + m0)])) > 0) {
#'   r1.na <- r1.na - eps / k
#'   r2.na <- r1.na - eps / k
#'   k <- k + 1
#'   ## select drugs with <30% (decreasing with k) missing data overall cell lines
#'   na.y <- apply(y0[, 2:(1 + m0)], 2, function(xx) sum(is.na(xx)) / length(xx))
#'   while (sum(na.y < r1.na) < m0) {
#'     y0 <- y0[, -c(1 + which(na.y >= r1.na))]
#'     m0 <- sum(na.y < r1.na)
#'     na.y <- apply(y0[, 2:(1 + m0)], 2, function(xx) sum(is.na(xx)) / length(xx))
#'   }
#'
#'   ## select cell lines with treatment of at least 80% (increasing with k) drugs
#'   na.y0 <- apply(y0[, 2:(1 + m0)], 1, function(xx) sum(is.na(xx)) / length(xx))
#'   while (sum(na.y0 < r2.na) < (dim(y0)[1])) {
#'     y0 <- y0[na.y0 < r2.na, ]
#'     na.y0 <- apply(y0[, 2:(1 + m0)], 1, function(xx) sum(is.na(xx)) / length(xx))
#'   }
#'   num.na <- sum(is.na(y0[, 2:(1 + m0)]))
#'   message("#{NA}=", num.na, "\n", "r1.na =", r1.na, ", r2.na =", r2.na, "\n")
#' }
#'
#' # ===============
#' # combine drug sensitivity, tissues and molecular features
#' # ===============
#' yx <- merge(y0, features, by = "Cell.Line")
#' names.cell.line <- yx$Cell.Line
#' names.drug <- colnames(yx)[2:(dim(y0)[2])]
#' names.drug <- substr(names.drug, 6, nchar(names.drug))
#' # numbers of gene expression features, copy number festures and muatation features
#' p <- c(13321, 13747 - 13321, 13818 - 13747)
#' num.nonpen <- 13
#' yx <- data.matrix(yx[, -1])
#' y <- yx[, 1:(dim(y0)[2] - 1)]
#' x <- cbind(yx[, dim(y0)[2] - 1 + sum(p) + 1:num.nonpen], yx[, dim(y0)[2] - 1 + 1:sum(p)])
#'
#' # delete genes with only one mutated cell line
#' x <- x[, -c(num.nonpen + p[1] + p[2] + 
#'             which(colSums(x[, num.nonpen + p[1] + p[2] + 1:p[3]]) <= 1))]
#' p[3] <- ncol(x) - num.nonpen - p[1] - p[2]
#'
#' GDSC <- list(
#'   y = y, x = x, p = p, num.nonpen = num.nonpen, names.cell.line = names.cell.line,
#'   names.drug = names.drug
#' )
#'
#'
#' ## ================
#' ## ================
#' ## select a small set of drugs
#' ## ================
#' ## ================
#'
#' name_drugs <- c(
#'   "Methotrexate", "RDEA119", "PD-0325901", "CI-1040", "AZD6244", "Nilotinib",
#'   "Axitinib"
#' )
#'
#' # extract the drugs' pharmacological profiling and tissue dummy
#' YX0 <- cbind(GDSC$y[, colnames(GDSC$y) %in% paste("IC50.", name_drugs, sep = "")]
#' [, c(1, 3, 6, 4, 7, 2, 5)], GDSC$x[, 1:GDSC$num.nonpen])
#' colnames(YX0) <- c(name_drugs, colnames(GDSC$x)[1:GDSC$num.nonpen])
#' # extract the genetic information of CNV & MUT
#' X23 <- GDSC$x[, GDSC$num.nonpen + GDSC$p[1] + 1:(p[2] + p[3])]
#' colnames(X23)[1:p[2]] <- paste(substr(
#'   colnames(X23)[1:p[2]], 1,
#'   nchar(colnames(X23)[1:p[2]]) - 3
#' ), ".CNV", sep = "")
#'
#' # locate all genes with CNV or MUT information
#' name_genes_duplicate <- c(
#'   substr(colnames(X23)[1:p[2]], 1, nchar(colnames(X23)[1:p[2]]) - 4),
#'   substr(colnames(X23)[p[2] + 1:p[3]], 1, nchar(colnames(X23)[p[2] + 1:p[3]]) - 4)
#' )
#' name_genes <- name_genes_duplicate[!duplicated(name_genes_duplicate)]
#'
#' # select the GEX which have the common genes with CNV or MUT
#' X1 <- GDSC$x[, GDSC$num.nonpen + 
#'             which(colnames(GDSC$x)[GDSC$num.nonpen + 1:p[1]] %in% name_genes)]
#'
#' p[1] <- ncol(X1)
#' X1 <- log(X1)
#'
#' # summary the data information
#' example_GDSC <- list(data = cbind(YX0, X1, X23))
#' example_GDSC$blockList <- list(
#'   1:length(name_drugs), length(name_drugs) + 1:GDSC$num.nonpen,
#'   ncol(YX0) + 1:sum(p))
#'
#' # ========================
#' # construct the G matrix: edge potentials in the MRF prior
#' # ========================
#'
#' # edges between drugs: Group1 ("RDEA119","17-AAG","PD-0325901","CI-1040" and "AZD6244")
#' # indexed as (2:5)
#' # http://software.broadinstitute.org/gsea/msigdb/cards/KEGG_MAPK_SIGNALING_PATHWAY
#' pathway_genes <- read.table("MAPK_pathway.txt")[[1]]
#' Idx_Pathway1 <- which(c(colnames(X1), name_genes_duplicate) %in% pathway_genes)
#' Gmrf_Group1Pathway1 <- t(combn(rep(Idx_Pathway1, each = length(2:5)) +
#'   rep((2:5 - 1) * sum(p), times = length(Idx_Pathway1)), 2))
#'
#' # edges between drugs: Group2 ("Nilotinib","Axitinib") indexed as (6:7)
#' # delete gene ABL2
#' Idx_Pathway2 <- which(c(colnames(X1), name_genes_duplicate) %like% "BCR" |
#'   c(colnames(X1), name_genes_duplicate) %like% "ABL")[-c(3, 5)]
#' Gmrf_Group2Pathway2 <- t(combn(rep(Idx_Pathway2, each = length(6:7)) +
#'   rep((6:7 - 1) * sum(p), times = length(Idx_Pathway2)), 2))
#'
#' # edges between the common gene in different data sources
#' Gmrf_CommonGene <- NULL
#' list_CommonGene <- list(0)
#' k <- 1
#' for (i in 1:length(name_genes)) {
#'   Idx_CommonGene <- which(c(colnames(X1), name_genes_duplicate) == name_genes[i])
#'   if (length(Idx_CommonGene) > 1) {
#'     Gmrf_CommonGene <- 
#'     rbind(Gmrf_CommonGene, t(combn(rep(Idx_CommonGene, each = length(name_drugs)) + 
#'     rep((1:length(name_drugs) - 1) * sum(p), times = length(Idx_CommonGene)), 2)))
#'     k <- k + 1
#'   }
#' }
#' Gmrf_duplicate <- rbind(Gmrf_Group1Pathway1, Gmrf_Group2Pathway2, Gmrf_CommonGene)
#' Gmrf <- Gmrf_duplicate[!duplicated(Gmrf_duplicate), ]
#' example_GDSC$mrfG <- Gmrf
#'
#' # create the target gene names of the two groups of drugs
#' targetGenes1 <- matrix(Idx_Pathway1, nrow = 1)
#' colnames(targetGenes1) <- colnames(example_GDSC$data)[seq_along(targetGene$group1)]
#' targetGenes2 <- matrix(Idx_Pathway2, nrow = 1)
#' colnames(targetGenes2) <- colnames(example_GDSC$data)[seq_along(targetGene$group2)]
#'
#' targetGene <- list(group1 = targetGenes1, group2 = targetGenes2)
#'
#' ## Write data file targetGene.rda to the user's directory by save()
#' }
#'
"targetGene"

#' @title plot conditional predictive ordinate
#' @description
#' Plot the conditional predictive ordinate (CPO) for each individual of a 
#' fitted model generated by \code{BayesSUR} which is a \code{BayesSUR} object. 
#' CPO is a handy posterior predictive check because it may be used to identify 
#' outliers, influential observations, and for hypothesis testing across 
#' different non-nested models (Gelfand 1996).
#' @importFrom graphics axis box text par abline
#' @name plotCPO
#' 
#' @param x an object of class \code{BayesSUR}
#' @param outlier.thresh threshold for the CPOs. The default is 0.01.
#' @param outlier.mark mark the outliers with the response names. 
#' The default is \code{FALSE}
#' @param scale.CPO scaled CPOs which is divided by their maximum. 
#' The default is \code{TRUE}
#' @param x.loc a vector of features distance
#' @param axis.label a vector of predictor names which are shown in CPO plot. 
#' The default is \code{NULL} only showing the indices. The value \code{"auto"} 
#' show the predictor names from the original data.
#' @param mark.pos location of the marked text relative to the point
#' @param las graphical parameter of plot.default
#' @param cex.axis graphical parameter of plot.default
#' @param mark.color color of the marked text. The default color is red
#' @param mark.cex font size of the marked text. The default font size is 0.8
#' @param xlab a title for the x axis
#' @param ylab a title for the y axis
#' @param ... other arguments
#' 
#' @details The default threshold for the CPOs to detect the outliers is 0.01 
#' by Congdon (2005). It can be tuned by the argument \code{outlier.thresh}.
#'
#' @references Statisticat, LLC (2013). \emph{Bayesian Inference.} Farmington, CT: Statisticat, LLC.
#' @references Gelfand A. (1996). \emph{Model Determination Using Sampling Based Methods}. In Gilks W., Richardson S., Spiegelhalter D. (eds.), Markov Chain Monte Carlo in Practice, pp. 145–161. Chapman & Hall, Boca Raton, FL.
#' @references Congdon P. (2005). \emph{Bayesian Models for Categorical Data}. John Wiley & Sons, West Sussex, England.
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
#' )
#'
#' ## check output
#' # plot the conditional predictive ordinate (CPO)
#' plotCPO(fit)
#'
#' @export
plotCPO <- function(x, outlier.mark = TRUE, outlier.thresh = 0.01, 
                    scale.CPO = TRUE, x.loc = FALSE, axis.label = NULL, 
                    las = 0, cex.axis = 1, 
                    mark.pos = c(0, -.01), mark.color = 2, mark.cex = 0.8,
                    xlab = "Observations", ylab = NULL, ...) {
  x$output[-1] <- paste(x$output$outFilePath, x$output[-1], sep = "")
  if (is.null(x$output$CPO)) {
    stop("Please specify argument output_CPO in BayesSUR()!")
  }

  CPO <- as.matrix(read.table(x$output$CPO))

  rownames(CPO) <- rownames(as.matrix(read.table(x$output$Y, header = TRUE)))
  colnames(CPO) <- colnames(as.matrix(read.table(x$output$Y, header = TRUE)))
  CPO_idx <- seq_len(nrow(CPO))

  if (is.null(ylab)) {
    ylab <- ifelse(scale.CPO, "scaled CPOs", "CPOs")
  }

  name.observations <- rownames(CPO)

  if (is.null(axis.label)) {
    x.loc <- CPO_idx
    names(x.loc) <- CPO_idx
  } else {
    if (axis.label[1] == "auto") {
      x.loc <- CPO_idx
      names(x.loc) <- name.observations
    } else {
      if (!x.loc[1]) {
        x.loc <- seq_along(axis.label)
      } else {
        if (length(axis.label) != length(x.loc)) {
          stop("The given predictor names are not consistent with the data")
        }
      }
      names(x.loc) <- axis.label
    }
  }

  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  par(xpd = FALSE)
  if (ncol(CPO) > 1) {
    if (scale.CPO) CPO <- CPO / max(CPO)
    plot.default(as.vector(CPO) ~ rep(CPO_idx, times = ncol(CPO)), 
                 xlim = c(1, nrow(CPO)), ylim = c(0, max(CPO)), 
                 xaxt = "n", bty = "n", ylab = ylab, xlab = xlab, 
                 main = "Conditional predictive ordinate", pch = 19)
    axis(1, at = x.loc, labels = names(x.loc), las = las, cex.axis = cex.axis)
    box()

    # mark the names of the specified response variables corresponding to the given responses
    if (outlier.mark) {
      if (min(CPO) > outlier.thresh) {
        message("NOTE: The minimum CPO is larger than the threshold of the (scaled) CPO!\n")
      } else {
        name.responses <- colnames(CPO)
        text(rep(CPO_idx, times = ncol(CPO))[
          which(as.vector(CPO) <= outlier.thresh)] + mark.pos[1], 
          as.vector(CPO[CPO < outlier.thresh]) + mark.pos[2], 
          labels = rep(name.responses, 
                       each = nrow(CPO))[as.vector(CPO) < outlier.thresh], 
          col = mark.color, cex = mark.cex)
        abline(h = outlier.thresh, lty = 2, col = mark.color)
      }
    }
  } else {
    if (scale.CPO) CPO <- CPO / max(CPO)
    plot.default(CPO ~ seq_along(CPO), xaxt = "n", bty = "n", 
                 xlim = c(1, length(CPO)), 
                 ylim = c(min(CPO) + mark.pos[2] * 2, max(CPO)), 
                 ylab = ylab, xlab = xlab, 
                 main = "Conditional predictive ordinate", pch = 19)
    axis(1, at = x.loc, labels = names(x.loc), las = las, cex.axis = cex.axis)
    box()

    # mark the names of the specified response variables corresponding to the given responses
    if (outlier.mark) {
      if (min(CPO) > outlier.thresh) {
        message("NOTE: The minimum CPO is larger than the threshold of the (scaled) CPO!\n")
      } else {
        opar <- par(no.readonly = TRUE)
        on.exit(par(opar))
        par(new = TRUE)
        plot.default(CPO[CPO < outlier.thresh] ~ which(CPO < outlier.thresh), 
                     xaxt = "n", bty = "n", xlim = c(1, length(CPO)), 
                     ylim = c(min(CPO) + mark.pos[2] * 2, max(CPO)), 
                     ylab = "", xlab = "", main = "", 
                     pch = 19, col = mark.color)
        text(which(CPO <= outlier.thresh) + mark.pos[1], 
             CPO[CPO < outlier.thresh] + mark.pos[2], 
             labels = name.observations[CPO < outlier.thresh], 
             col = mark.color, cex = mark.cex)
        abline(h = outlier.thresh, lty = 2, col = mark.color)
      }
    }
  }
}

#' @title plot MCMC diagnostic plots
#' @description
#' Show trace plots and diagnostic density plots of a fitted model object of class \code{BayesSUR}.
#' @importFrom graphics par plot.default legend title matplot
#' @importFrom stats density
#' @importFrom grDevices hcl.colors
#' @name plotMCMCdiag
#' @param x an object of class \code{BayesSUR}
#' @param nbloc number of splits for the last half iterations after substracting 
#' burn-in length
#' @param HIWg diagnostic plot of the response graph. Default is \code{NULL}. 
#' \code{HIW="degree"} prints the diagnostic of the degrees of response nodes. 
#' \code{HIW="edges"} prints the diagnostic of every edge between two responses. 
#' \code{HIW="lik"} prints the diagnostic of the posterior likelihoods of the 
#' hyperparameters related to the response relationships
#' @param header the main title
#' @param ... other arguments for the plots of the log-likelihood and model size
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' plotMCMCdiag(fit)
#'
#' @export
plotMCMCdiag <- function(x, nbloc = 3, HIWg = NULL, header = "", ...) {
  if (!inherits(x, "BayesSUR")) {
    stop("Use only with a \"BayesSUR\" object")
  }

  x$output[-1] <- paste0(x$output$outFilePath, x$output[-1])
  logP <- t(as.matrix(read.table(x$output$logP)))
  model_size <- as.matrix(read.table(x$output$model_size))
  ncol_Y <- ncol(read.table(x$output$gamma))
  nIter <- x$input$nIter

  covariancePrior <- x$input$covariancePrior
  if (covariancePrior == "HIW" && is.null(x$output$Gvisit)) {
    Gvisit <- as.matrix(read.table(x$output$Gvisit))
  }
  
  tick <- x$input$tick
  if (nIter <= 1) {
    stop("The diagosis only shows results from more than one MCMC iteration!")
  }
  if (nIter < 4 * tick) {
    message(paste("NOTE: The diagosis only shows results of two iteration points due 
            to less than", 4*tick, "MCMC iterations!"))
  }

  if (nIter >= 4 * tick) {
    logP <- logP[, ncol(logP) - floor(nIter / tick) - 1 + 1:floor(nIter / tick)]
  } else {
    logP <- logP[, c(1, ncol(logP))]
  }
  if (is.null(HIWg)) {
    Ptau.indx <- ifelse(covariancePrior != "IG", 7, 3)
    Plik.indx <- ifelse(covariancePrior != "IG", 10, 5)
    
    model_size <- model_size
    if (nIter >= 4 * tick) {
      model_size <- rowSums(model_size[nrow(model_size) - floor(nIter / tick) - 
                                         1 + 1:floor(nIter / tick), ])
    } else {
      model_size <- rowSums(model_size[c(1, nrow(model_size)), ])
    }

    dens.all <- density(logP[Ptau.indx, ]) 
    if (nIter >= 4000) {
      dens.first <- density(logP[Ptau.indx, 1:floor(ncol(logP) / 2)])
      dens.last <- 
        density(logP[Ptau.indx, (1 + floor(ncol(logP) / 2)):ncol(logP)])

      ymin <- min(dens.all$y, dens.first$y, dens.last$y)
      ymax <- max(dens.all$y, dens.first$y, dens.last$y)
      xmin <- min(dens.all$x, dens.first$x, dens.last$x)
      xmax <- max(dens.all$x, dens.first$x, dens.last$x)
    } else {
      ymin <- min(dens.all$y)
      ymax <- max(dens.all$y)
      xmin <- min(dens.all$x)
      xmax <- max(dens.all$x)
      nbloc <- 1
    }

    ### nsplit number of split of the sweep
    mid <- floor(floor(ncol(logP) / 2) / nbloc)
    ymax2 <- xmin2 <- xmax2 <- list.dens <- NULL
    for (i in 1:nbloc) {
      dens <- density(logP[Ptau.indx, 
                           (ifelse(nbloc == 1, 0, floor(ncol(logP) / 2)) +
                              1 + mid * (i - 1)):ncol(logP)])
      ymax2 <- max(ymax2, dens$y)
      xmin2 <- min(xmin2, dens$x)
      xmax2 <- max(xmax2, dens$x)
      list.dens <- c(list.dens, list(dens))
    }

    ### plot the figures
    opar <- par(no.readonly = TRUE)
    on.exit(par(opar))
    par(mfrow = c(2, 2))

    if (nbloc > 1) {
      plot.default(logP[Plik.indx, ], xlab = paste0("Iterations (*",tick, ")"), 
                   ylab = "Log likelihood (posterior)", type = "l", lty = 1, ...)
    } else {
      plot.default(logP[Plik.indx, ] ~ c(1, nIter), xlab = "Iterations", 
                   ylab = "Log likelihood (posterior)", type = "l", lty = 1, ...)
    }

    if (nbloc > 1) {
      plot.default(model_size, xlab = paste0("Iterations (*",tick, ")"), 
                   ylab = "Model size", type = "l", lty = 1, ...)
    } else {
      plot.default(model_size ~ c(1, nIter), xlab = "Iterations", 
                   ylab = "Model size", type = "l", lty = 1, ...)
    }

    title0 <- expression(paste("Log Posterior Distribution: log ", 
                               P(gamma ~ group("|", list(Y, .), ""))))
    plot.default(dens.all, main = "", col = "black", xlim = c(xmin, xmax), 
                 ylim = c(ymin, ymax), xlab = title0, ylab = "", 
                 type = "l", lty = 1)
    if (nIter >= 4000) {
      par(new = TRUE)
      plot.default(dens.first, main = "", col = "red", xlim = c(xmin, xmax), 
                   ylim = c(ymin, ymax), xlab = "", ylab = "", 
                   type = "l", lty = 1)
      par(new = TRUE)
      plot.default(dens.last, main = "", col = "green", xlim = c(xmin, xmax), 
                   ylim = c(ymin, ymax), xlab = "", , ylab = "Density", 
                   type = "l", lty = 1)
      if (nbloc > 1) {
        legend("topleft", title = "iteration", 
               legend = paste0(c("ALL", "First half", "Last half"), " = [", 
                              c(1, 1, floor(ncol(logP) / 2) * tick + 1), ":", 
                              c(ncol(logP), floor((ncol(logP)) / 2), 
                                ncol(logP)) * tick, "]"), 
               col = 1:3, lty = 1, text.col = 1:3, cex = 0.8)
      }
    }
    for (i in 1:nbloc) {
      plot.default(list.dens[[i]], col = i, xlim = c(xmin2, xmax2), 
                   ylim = c(ymin, ymax2), xlab = title0, ylab = "", 
                   type = "l", lty = 1, main = "")
      if (nbloc > 1) par(new = TRUE)
    }
    title(ylab = "Density")
    if (nbloc > 1) {
      legend("topleft", title = "moving window", 
             legend = paste0("set ", 1:nbloc, " = [", 
                             (floor((ncol(logP)) / 2) + mid * (nbloc:1 - 1)) * 
                               tick + 1, ":", (ncol(logP)) * tick, "]"), 
             col = 1:nbloc, lty = 1, text.col = 1:nbloc, cex = 0.8)
    }
  } else {
    if (covariancePrior != "HIW") {
      stop("The argument HIWg only works for the model with hyper-inverse 
           Wishart prior on the covariance!")
    }

    if (HIWg == "degree") {
      m <- ncol_Y
      node1 <- node2 <- NULL
      for (i in 1:(m - 1)) {
        node1 <- c(node1, rep(i, m - i))
        node2 <- c(node2, (i + 1):m)
      }
      nodes <- cbind(node1, node2)
      node.degree <- matrix(0, nrow = nrow(Gvisit), ncol = m)
      for (i in 1:m) {
        node.degree[, i] <- 
          rowSums(Gvisit[, which(nodes == i, arr.ind = TRUE)[, 1]])
      }

      matplot(node.degree, type = "l", lty = 1, col = hcl.colors(m), 
              xlab = paste0("Iterations (*",tick, ")"), ylab = "degree", 
              main = "Response degrees", xlim = c(1, nrow(Gvisit) * 1.1))
      legend("topright", legend = 1:m, col = hcl.colors(m), lty = 1, 
             text.col = hcl.colors(m), cex = 1 / m * 4)
    }

    if (substr(HIWg, 1, 4) == "edge") {
      m <- ncol_Y
      node1 <- node2 <- NULL
      for (i in 1:(m - 1)) {
        node1 <- c(node1, rep(i, m - i))
        node2 <- c(node2, (i + 1):m)
      }

      if (HIWg == "edge") {
        matplot(Gvisit, type = "l", lty = 1, col = hcl.colors(ncol(Gvisit)), 
                xlab = paste0("Iterations (*",tick, ")"), ylab = "", 
                main = "Edges selection", xlim = c(1, nrow(Gvisit) * 1.1))
        legend("topright", legend = paste0(node1, "-", node2), 
               col = hcl.colors(ncol(Gvisit)), lty = 1,
               text.col = hcl.colors(ncol(Gvisit)), cex = 1 / m * 2)
      } else {
        plot.default(
          Gvisit[, which(paste0(node1, node2) == substr(HIWg, 5, nchar(HIWg)))], 
          type = "l", lty = 1, xlab = paste0("Iterations (*",tick, ")"), ylab = "", 
          main = paste0("Edge-", substr(HIWg, 5, nchar(HIWg)), " selection"))
      }
    }

    if (HIWg == "lik") {
      Gvisit <- t(logP[1:4, ])
      matplot(Gvisit, type = "l", lty = 1, col = seq_len(ncol(Gvisit)), 
              xlab = paste0("Iterations (*",tick, ")"), ylab = "Log likelihood (posterior)", 
              main = "Likelihoods of graph learning")
      legend("topright", legend = c("tau", "eta", "JT", "SigmaRho"), 
             col = seq_len(ncol(Gvisit)), lty = 1, 
             text.col = seq_len(ncol(Gvisit)), cex = 0.8)
    }
  }
  title(paste0("\n", header), outer = TRUE)
}

#' @title Fitting BayesSUR models
#' @description
#' Main function of the package. Fits a range of models introduced in the 
#' package vignette \code{BayesSUR.pdf}. Returns an object of S3 class 
#' \code{BayesSUR}. There are three options for the prior on the residual 
#' covariance matrix (i.e., independent inverse-Gamma, inverse-Wishart and 
#' hyper-inverse Wishart) and three options for the prior on the latent 
#' indicator variable (i.e., independent Bernoulli, hotspot and Markov random 
#' field). So there are nine models in total. See details for their combinations.
#'
#' @useDynLib BayesSUR
#' @aliases BayesSUR-package
#' @importFrom utils head tail read.table write.table
#' @importFrom Rcpp sourceCpp
#' @importFrom xml2 as_xml_document write_xml
#' @importFrom parallel detectCores
#'
#' @name BayesSUR
#' @param data a numeric matrix with variables on the columns and observations 
#' on the rows, if arguments \code{Y} and \code{X} (and possibly \code{X_0}) 
#' are vectors. Can be \code{NULL} if arguments \code{Y} and \code{X} (and 
#' possibly \code{X_0}) are numeric matrices
#' @param Y,X vectors of indices (with respect to the data matrix) for the 
#' outcomes (\code{Y}) and the predictors to select (\code{X}) respectively; 
#' if the \code{data} argument is \code{NULL}, these needs to be numeric 
#' matrices containing the data instead, with variables on the columns and 
#' observations on the rows
#' @param X_0 vectors of indices (with respect to the data matrix) for the 
#' fixed predictors that are not selected, i.e. always included in the model; 
#' if the data argument is not provided, this needs to be a numeric matrix 
#' containing the data instead, with variables on the columns and observations 
#' on the rows
#' @param betaPrior string indicating the prior for regression coefficients; it 
#' has to be either \code{independent} for independent spike-and-slab priors 
#' (only slab part for \code{X_0} if specified), or \code{reGroup} for weakly 
#' normal priors for mandatory variables (random effects) and spike-and-slab 
#' priors for other variables of Zhao (2023)
#' @param covariancePrior string indicating the prior for the covariance $C$; 
#' it has to be either \code{HIW} for the hyper-inverse-Wishar (which will 
#' result in a sparse covariance matrix), \code{IW} for the inverse-Wishart 
#' prior (dense covariance) or \code{IG} for independent inverse-Gamma on all 
#' the diagonal elements and 0 otherwise. See the details for the model 
#' specification
#' @param gammaPrior string indicating the gamma prior to use, either 
#' \code{hotspot} (default) for the Hotspot prior of Bottolo (2011), \code{MRF} 
#' for the Markov Random Field prior or \code{hierarchical} for a simpler 
#' hierarchical prior. See the details for the model specification
#' @param nIter number of iterations for the MCMC procedure. Default 10000
#' @param burnin number of iterations to discard at the start of the chain. 
#' Default is 5000
#' @param nChains number of parallel tempered chains to run (default 2). The 
#' temperature is adapted during the burnin phase
#' @param outFilePath path to where the output files are to be written
#' @param gammaSampler string indicating the type of sampler for gamma, either 
#' \code{bandit} for the Thompson sampling inspired samper or \code{MC3} for 
#' the usual MC^3 sampler.  See Russo et al.(2018) or Madigan and York (1995) 
#' for details
#' @param gammaInit gamma initialisation to either all-zeros (\code{0}), all 
#' ones (\code{1}), MLE-informed (\code{MLE}) or (default) randomly (\code{R})
#' @param mrfG either a matrix or a path to the file containing (the edge list 
#' of) the G matrix for the MRF prior on gamma (if necessary)
#' @param standardize logical flag for X variable standardization. Default is 
#' \code{standardize=TRUE}. Coefficients are returned on the standardized scale
#' @param standardize.response logical flag for Y standardization. Default is 
#' \code{standardize.response=TRUE}
#' @param hyperpar a list of named hypeparameters to use instead of the default 
#' values. Valid names are mrf_d, mrf_e, a_sigma, b_sigma, a_tau, b_tau, nu, 
#' a_eta, b_eta, a_o, b_o, a_pi, b_pi, a_w and b_w. Their default values are 
#' a_w=2, b_w=5, a_omega=2, b_omega=1, a_o=2, b_o=p-2, a_pi=2, b_pi=1, nu=s+2, 
#' a_tau=0.1, b_tau=10, a_eta=0.1, b_eta=1, a_sigma=1, b_sigma=1, mrf_d=-3 and 
#' mrf_e=0.03. See the vignette for more information
#' @param maxThreads maximum threads used for parallelization. Default is 1. 
#' Reproducibility of results with \code{set.seed()} is only guaranteed if 
#' \code{maxThreads=1}
#' @param tick an integer used for printing the iteration index and some updated 
#' parameters every tick-th iteration. Default is 1000 
#' @param output_gamma allow (\code{TRUE}) or suppress (\code{FALSE}) the 
#' output for  gamma. See the return value below for more information
#' @param output_beta allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for beta. See the return value below for more information
#' @param output_Gy allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for Gy. See the return value below for more information
#' @param output_sigmaRho allow (\code{TRUE}) or suppress (\code{FALSE}) the 
#' output for sigmaRho. See the return value below for more information
#' @param output_pi allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for pi. See the return value below for more information
#' @param output_tail allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for tail (hotspot tail probability). See the return value below for more 
#' information
#' @param output_model_size allow (\code{TRUE}) or suppress (\code{FALSE}) the 
#' output for model_size. See the return value below for more information
#' @param output_model_visit allow (\code{TRUE}) or suppress (\code{FALSE}) the 
#' output for all visited models over the MCMC iterations. Default is 
#' \code{FALSE}. See the return value below for more information
#' @param output_CPO allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for (scaled) conditional predictive ordinates (\code{*_CPO_out.txt}),
#' CPO with joint posterior predictive of the response variables 
#' (\code{*_CPOsumy_out.txt}) and widely applicable information criterion 
#' (\code{*_WAIC_out.txt}). See the return value below for more information
#' @param output_Y allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for responses dataset Y
#' @param output_X allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
#' for predictors dataset X
#' @param tmpFolder the path to a temporary folder where intermediate data 
#' files are stored (will be erased at the end of the chain). It is specified 
#' relative to \code{outFilePath}
#'
#' @details The arguments \code{covariancePrior} and \code{gammaPrior} specify 
#' the model HRR, dSUR or SSUR with different gamma prior. Let 
#' \eqn{\gamma_{jk}} be latent indicator variable of each coefficient and 
#' \eqn{C} be covariance matrix of response variables. The nine models 
#' specified through the arguments \code{covariancePrior} and 
#' \code{gammaPrior} are as follows.
#' \tabular{cccc}{
#'                 \tab \eqn{\gamma_{jk}}~Bernoulli \tab \eqn{\gamma_{jk}}~hotspot \tab \eqn{\gamma}~MRF \cr
#'   \eqn{C}~indep \tab HRR-B                       \tab HRR-H                     \tab HRR-M           \cr
#'   \eqn{C}~IW   \tab dSUR-B                      \tab dSUR-H                    \tab dSUR-M          \cr
#'   \eqn{C}~HIW   \tab SSUR-B                      \tab SSUR-H                    \tab SSUR-M
#' }
#'
#' @return An object of class \code{BayesSUR} is saved as 
#' \code{obj_BayesSUR.RData} in the output file, including the following 
#' components:
#' \itemize{
#' \item status - the running status
#' \item input - a list of all input parameters by the user
#' \item output - a list of the all output filenames:
#' \itemize{
#' \item "\code{*_logP_out.txt}" - contains each row for the \eqn{1000t}-th iteration's log-likelihoods of parameters, i.e., Tau, Eta, JunctionTree, SigmaRho, O, Pi, Gamma, W, Beta and data conditional log-likelihood depending on the models.
#' \item "\code{*_gamma_out.txt}" - posterior mean of the latent indicator matrix.
#' \item "\code{*_pi_out.txt}" - posterior mean of the predictor effects (prospensity) by decomposing the probability of the latent indicator.
#' \item "\code{*_hotspot_tail_p_out.txt}" - posterior mean of the hotspot tail probability. Only available for the hotspot prior on the gamma.
#' \item "\code{*_beta_out.txt}" - posterior mean of the coefficients matrix.
#' \item "\code{*_Gy_out.txt}" - posterior mean of the response graph. Only available for the HIW prior on the covariance.
#' \item "\code{*_sigmaRho_out.txt}" - posterior mean of the transformed parameters. Not available for the IG prior on the covariance.
#' \item "\code{*_model_size_out.txt}" - contains each row for the\eqn{1000t}-th iteration's model sizes of the multiple response variables.
#' \item "\code{*_model_visit_gy_out.txt}" - contains each row for the nonzero indices of the vectorized estimated graph matrix for each iteration.
#' \item "\code{*_model_visit_gamma_out.txt}" - contains each row for the nonzero indices of the vectorized estimated gamma matrix for each iteration.
#' \item "\code{*_CPO_out.txt}" - the (scaled) conditional predictive ordinates (CPO).
#' \item "\code{*_CPOsumy_out.txt}" - the (scaled) conditional predictive ordinates (CPO) with joint posterior predictive of the response variables.
#' \item "\code{*_WAIC_out.txt}" - the widely applicable information criterion (WAIC).
#' \item "\code{*_Y.txt}" - responses dataset.
#' \item "\code{*_X.txt}" - predictors dataset.
#' \item "\code{*_X0.txt}" - fixed predictors dataset.
#' }
#' \item call - the matched call.
#' }
#'
#' @references Russo D, Van Roy B, Kazerouni A, Osband I, Wen Z (2018). \emph{A tutorial on Thompson sampling.} Foundations and Trends in Machine Learning, 11: 1-96.
#' @references Madigan D, York J (1995). \emph{Bayesian graphical models for discrete data.} International Statistical Review, 63: 215–232.
#' @references Bottolo L, Banterle M, Richardson S, Ala-Korpela M, Jarvelin MR, Lewin A (2020). \emph{A computationally efficient Bayesian seemingly unrelated regressions model for high-dimensional quantitative trait loci discovery.} Journal of Royal Statistical Society: Series C, 70: 886-908.
#' @references Zhao Z, Banterle M, Bottolo L, Richardson S, Lewin A, Zucknick M (2021). \emph{BayesSUR: An R package for high-dimensional multivariate Bayesian variable and covariance selection in linear regression.} Journal of Statistical Software, 100: 1–32.
#' @references Zhao Z, Banterle M, Lewin A, Zucknick M (2023). \emph{Multivariate Bayesian structured variable selection for pharmacogenomic studies.} Journal of the Royal Statistical Society: Series C (Applied Statistics), qlad102.
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 5, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
#' )
#'
#' ## check output
#' # show the summary information
#' summary(fit)
#'
#' # show the estimated beta, gamma and graph of responses Gy
#' plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap")
#'
#' \dontrun{
#' ## Set up temporary work directory for saving a pdf figure
#' # td <- tempdir()
#' # oldwd <- getwd()
#' # setwd(td)
#'
#' ## Produce authentic math formulas in the graph
#' # plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap", fig.tex = TRUE)
#' # system(paste(getOption("pdfviewer"), "ParamEstimator.pdf"))
#' # setwd(oldwd)
#' }
#'
#' @export
BayesSUR <- function(data = NULL, Y, X, X_0 = NULL,
                     covariancePrior = "HIW", gammaPrior = "hotspot", 
                     betaPrior = "independent",
                     nIter = 10000, burnin = 5000, nChains = 2,
                     outFilePath = "", 
                     gammaSampler = "bandit", gammaInit = "R", mrfG = NULL,
                     standardize = TRUE, standardize.response = TRUE, 
                     maxThreads = 1, tick = 1000, 
                     output_gamma = TRUE, output_beta = TRUE, output_Gy = TRUE, 
                     output_sigmaRho = TRUE, output_pi = TRUE, 
                     output_tail = TRUE, output_model_size = TRUE, 
                     output_model_visit = FALSE, output_CPO = FALSE, 
                     output_Y = TRUE, output_X = TRUE, 
                     hyperpar = list(), tmpFolder = "tmp/") {
  # Check the directory for the output files
  if (outFilePath == "") {
    stop("Please specify a directory to save all output files!")
  }
  
  outFilePathLength <- nchar(outFilePath)
  if (substr(outFilePath, outFilePathLength, outFilePathLength) != "/") {
    outFilePath <- paste(outFilePath, "/", sep = "")
  }
  if (!file.exists(outFilePath)) {
    dir.create(outFilePath)
  }
  
  # Create temporary directory
  tmpFolderLength <- nchar(tmpFolder)
  if (substr(tmpFolder, tmpFolderLength, tmpFolderLength) != "/") {
    tmpFolder <- paste(tmpFolder, "/", sep = "")
  }
  tmpFolder <- paste(outFilePath, tmpFolder, sep = "")
  if (!file.exists(tmpFolder)) {
    dir.create(tmpFolder)
  }
  
  ## Check the input: reasoning is that the user provides either
  # a data matrix or a data path-to-file
  #     - in this case Y, X (and X_0) need to be provided as vectors of indexes
  # if the data matrix is not provided, the user will give 2/3 matrices for Y, X (and X_0)
  #     - in which case we write those in order into a new joint file
  # everything else throws an error
  
  # check the formula
  cl <- match.call()
  
  # we'll check in reverse order, is data NULL?
  if (is.null(data)) {
    # Y,X (and if there X_0) need to be valid numeric matrices then
    # check Y and X have comfortable number of observations
    npY <- dim(Y)
    if ((!is.numeric(Y)) || is.null(npY)) {
      my_stop("If 'data' is NULL, Y should be a numeric matrix", tmpFolder)
    }
    
    npX <- dim(X)
    if ((!is.numeric(X)) || is.null(npX) || (npX[1] != npY[1])) {
      my_stop("If 'data' is NULL, X should be a numeric matrix and the same 
              number of rows of Y", tmpFolder)
    }
    
    if (is.null(X_0)) {
      X_0 <- matrix(NA, nrow = npY[1], ncol = 0)
    } else {
      npX0 <- dim(X_0)
      if ((!is.numeric(X_0)) || is.null(npX0) || (npX0[1] != npY[1])) {
        my_stop("If 'data' is NULL and X_0 is provided, X_0 should be a numeric 
                matrix and the same number of rows of Y", tmpFolder)
      }
    }
    
    # Standarize the data
    if (standardize) {
      X <- scale(X)
      sd_X0 <- apply(X_0, 2, sd)
      X_0[, sd_X0 != 0] <- scale(X_0[, sd_X0 != 0])
    }
    if (standardize.response) Y <- scale(Y)
    
    # Write the three down in a single data file
    write.table(cbind(Y, X, X_0), paste(sep = "", tmpFolder, "data.txt"), 
                row.names = FALSE, col.names = FALSE)
    data <- paste(sep = "", tmpFolder, "data.txt")
    
    blockLabels <- c(rep(0, ncol(Y)), rep(1, ncol(X)), rep(2, ncol(X_0)))
    
    # Write the data in a output file
    write.table(Y, paste(sep = "", outFilePath, "data_Y.txt"), 
                row.names = FALSE, col.names = TRUE)
    write.table(X, paste(sep = "", outFilePath, "data_X.txt"), 
                row.names = FALSE, col.names = TRUE)
    write.table(X_0, paste(sep = "", outFilePath, "data_X0.txt"), 
                row.names = FALSE, col.names = TRUE)
  } else { # data is not null, so the user wants to use it to input the data
    
    # is the data given as matrix?
    ## If it's valid matrix, simply write it and re-assign the variable data to hold its path
    
    npData <- dim(data)
    if (is.numeric(data) || (!is.null(npData)) || (npData[2] >= 2)) {
      # Standarize the data
      if (standardize) {
        data[, X] <- scale(data[, X])
        data[, X_0] <- scale(data[, X_0])
      }
      if (standardize.response) data[, Y] <- scale(data[, Y])
      
      # Write the Y and X data in a output file
      write.table(data[, Y], paste(sep = "", outFilePath, "data_Y.txt"), 
                  row.names = FALSE, col.names = TRUE)
      write.table(data[, X], paste(sep = "", outFilePath, "data_X.txt"), 
                  row.names = FALSE, col.names = TRUE)
      write.table(data[, X_0], paste(sep = "", outFilePath, "data_X0.txt"), 
                  row.names = FALSE, col.names = TRUE)
      
      write.table(data, paste(sep = "", tmpFolder, "data.txt"), 
                  row.names = FALSE, col.names = FALSE)
      data <- paste(sep = "", tmpFolder, "data.txt")
    } else {
      my_stop("Y should be NULL or a numeric matrix with 2 or more columns!")
    }
    
    # is the data given as a string?
    if (is.character(data) && length(data) == 1) {
      if (substr(data, 1, 1) == "~") {
        data <- path.expand(data)
      }
    }
    
    ## at this point data contains the path to a file that exists
    # try and read one line to check dimensions
    dataHeader <- read.table(data, header = FALSE, nrows = 1)
    nVariables <- ncol(dataHeader)
    
    ## Y, X (and X_0) should be some fixed variables that needs to be included in the model
    if (is.null(X_0)) {
      X_0 <- as.numeric(c())
    }
    
    # be sure that they are vectors
    if (!(is.vector(Y, "numeric") && is.vector(X, "numeric") && 
          is.vector(X_0, "numeric"))) {
      my_stop("When the 'data' argument is set, Y,X and X_0 need to be 
              corresponding index vectors!", tmpFolder)
    }
    
    # check thay do not overlap
    if (length(c(intersect(Y, X), intersect(Y, X_0), intersect(X_0, X))) != 0) {
      my_stop("Y, X and X_0 need to be distinct index vectors!", tmpFolder)
    }
    
    # check if dimensions correspond -- higher dimensions gets an error
    if (length(c(Y, X, X_0)) > nVariables) {
      my_stop("When the 'data' argument is set, Y,X and X_0 need to be 
              corresponding index vectors!", tmpFolder)
    }
    # equal dimensions are ok, but lower dimensions means some columns of the data will be disregarded ( set to -1  )
    
    # We can now init the blockList
    blockLabels <- rep(NA, nVariables)
    blockLabels[Y] <- 0
    blockLabels[X] <- 1
    if (length(X_0) > 0) {
      blockLabels[X_0] <- 2
    }
    
    blockLabels[is.na(blockLabels)] <- -1
  }
  
  # cleanup file PATHS
  dataLength <- nchar(data)
  if (dataLength == 0) {
    my_stop("Please provide a correct path to a plain-text (.txt) file", 
            tmpFolder)
  }
  
  # magicly strip '/' from the start and '.txt' from the end of the data file name
  dataString <- 
    head(strsplit(tail(strsplit(data, split = c("/"))[[1]], 1), ".txt")[[1]], 1)
  
  ## Then init the structure graph
  # Consider that the indexes are written so that Y is 0 , X is 1 and (if there) X_0 is 2
  if (length(X_0) > 0) {
    structureGraph <- structureGraph <- matrix(c(0, 0, 0, 1, 0, 0, 2, 0, 0), 
                                               3, 3, byrow = TRUE)
  } else {
    structureGraph <- structureGraph <- matrix(c(0, 0, 1, 0), 2, 2, 
                                               byrow = TRUE)
  }
  
  ## Finally write blockLabels and structureGraph to a file
  write.table(blockLabels, paste(sep = "", tmpFolder, "blockLabels.txt"), 
              row.names = FALSE, col.names = FALSE)
  blockList <- paste(sep = "", tmpFolder, "blockLabels.txt")
  write.table(structureGraph, paste(sep = "", tmpFolder, "structureGraph.txt"), 
              row.names = FALSE, col.names = FALSE)
  structureGraph <- paste(sep = "", tmpFolder, "structureGraph.txt")
  
  # check how burnin was given
  if (burnin < 0) {
    my_stop("Burnin must be positive or 0", tmpFolder)
  } else {
    if (burnin > nIter) {
      my_stop("Burnin might not be greater than nIter", tmpFolder)
    } else {
      if (burnin < 1) { # given as a fraction
        burnin <- ceiling(nIter * burnin) # the zero case is taken into account here as well
      }
    }
  } # else assume is given as an absolute number
  
  ###############################
  ## prepare the print of hyperparameters corresponding the specified model
  hyperpar.all <- list(a_w = 2, b_w = 5, 
                       a_o = 2, b_o = sum(blockLabels == 1) - 2, 
                       a_pi = NA, b_pi = NA, 
                       nu = sum(blockLabels == 0) + 2, 
                       a_tau = 0.1, b_tau = 10, 
                       a_eta = 0.1, b_eta = 1, 
                       a_sigma = 1, b_sigma = 1, 
                       mrf_d = -3, mrf_e = 0.03)
  if (toupper(gammaPrior) %in% c("HOTSPOT", "HOTSPOTS", "HS")) {
    hyperpar.all$a_pi <- 2
    hyperpar.all$b_pi <- 1
    
    if (toupper(covariancePrior) %in% c("INDEPENDENT", "INDEP", "IG")) {
      hyperpar.all <- hyperpar.all[-c(7:11, 14:15)]
    }
    if (toupper(covariancePrior) %in% c("DENSE", "IW")) {
      hyperpar.all <- hyperpar.all[-c(10:11, 12:15)]
    }
    if (toupper(covariancePrior) %in% c("SPARSE", "HIW")) {
      hyperpar.all <- hyperpar.all[-c(12:15)]
    }
  }
  if (toupper(gammaPrior) %in% c("HIERARCHICAL", "H")) {
    hyperpar.all$a_pi <- 1
    hyperpar.all$b_pi <- sum(blockLabels == 0) - 1
    
    if (toupper(covariancePrior) %in% c("INDEPENDENT", "INDEP", "IG")) {
      hyperpar.all <- hyperpar.all[-c(3:4, 7:11, 14:15)]
    }
    if (toupper(covariancePrior) %in% c("DENSE", "IW")) {
      hyperpar.all <- hyperpar.all[-c(3:4, 10:11, 12:15)]
    }
    if (toupper(covariancePrior) %in% c("SPARSE", "HIW")) {
      hyperpar.all <- hyperpar.all[-c(3:4, 12:15)]
    }
  }
  if (toupper(gammaPrior) %in% c("MRF", "MARKOV RANDOM FIELD")) {
    if (is.null(mrfG)) {
      my_stop("Argument 'mrfG' was specified!", tmpFolder)
    }
    
    if (toupper(covariancePrior) %in% c("INDEPENDENT", "INDEP", "IG")) {
      hyperpar.all <- hyperpar.all[-c(3:6, 7:13)]
    }
    if (toupper(covariancePrior) %in% c("DENSE", "IW")) {
      hyperpar.all <- hyperpar.all[-c(3:6, 10:13)]
    }
    if (toupper(covariancePrior) %in% c("SPARSE", "HIW")) {
      hyperpar.all <- hyperpar.all[-c(3:6, 12:13)]
    }
  }
  if (toupper(betaPrior) == "REGROUP") {
    hyperpar.all$a_w0 <- hyperpar.all$a_w
    hyperpar.all$b_w0 <- hyperpar.all$b_w
  }
  
  if (length(hyperpar) > 0) {
    for (i in seq_along(hyperpar)) {
      if (names(hyperpar)[[i]] %in% names(hyperpar.all)) {
        hyperpar.all[[which(names(hyperpar.all) == names(hyperpar)[[i]])]] <- 
          hyperpar[[i]]
      }
      #if (!is.null(hyperpar$a_omega)) hyperpar.all$a_pi <- hyperpar$a_omega
      #if (!is.null(hyperpar$b_omega)) hyperpar.all$b_pi <- hyperpar$b_omega
    }
  }
  
  if (toupper(gammaPrior) %in% c("HIERARCHICAL", "H")) {
    ## re-use the hyperparameter names a_pi and b_pi for 
    ## Bernoulli prior's hyperparameters a_omega and b_omega
    #names(hyperpar.all)[names(hyperpar.all) == "a_pi"] <- "a_omega"
    #names(hyperpar.all)[names(hyperpar.all) == "b_pi"] <- "b_omega"
    if (!is.null(hyperpar$a_omega)) hyperpar.all$a_pi <- hyperpar$a_omega
    if (!is.null(hyperpar$b_omega)) hyperpar.all$b_pi <- hyperpar$b_omega
  }
  
  # method to use
  if (toupper(covariancePrior) %in% c("SPARSE", "HIW")) {
    covariancePrior <- "HIW"
  } else if (toupper(covariancePrior) %in% c("DENSE", "IW")) {
    covariancePrior <- "IW"
  } else if (toupper(covariancePrior) %in% c("INDEPENDENT", "INDEP", "IG")) {
    covariancePrior <- "IG"
  } else {
    my_stop("Unknown covariancePrior argument: only sparse (HIW), dense(IW) or 
            independent (IG) are available", tmpFolder)
  }
  
  # mrfG and gammaPrior
  if (gammaPrior == "") {
    if (is.null(mrfG)) {
      message("Using default prior for Gamma - hotspot prior\n")
      # mrfG=""
      gammaPrior <- "hotspot"
    } else {
      message("No value for gammaPrior was specified, but mrfG was given - 
              choosing MRF prior\n")
      gammaPrior <- "MRF"
    }
  } else {
    if (toupper(gammaPrior) %in% c("HOTSPOT", "HOTSPOTS", "HS")) {
      gammaPrior <- "hotspot"
    } else if (toupper(gammaPrior) %in% c("MRF", "MARKOV RANDOM FIELD")) {
      gammaPrior <- "MRF"
    } else if (toupper(gammaPrior) %in% c("HIERARCHICAL", "H")) {
      gammaPrior <- "hierarchical"
    } else {
      my_stop("Unknown gammaPrior argument: only hotspot, MRF or hierarchical 
              are available", tmpFolder)
    }
  }
  
  # if mrfG is not a string
  if (!(is.character(mrfG) && length(mrfG) == 1)) {
    # if it's a matrix
    if ((is.numeric(mrfG) || is.data.frame(mrfG)) && !is.null(dim(mrfG))) {
      mrfG <- mrfG - 1 # due to C++ indexing from 0
      if (ncol(mrfG) == 2) {
        mrfG <- cbind(mrfG, rep(1, nrow(mrfG)))
      }
      write.table(mrfG, paste(sep = "", outFilePath, "mrfG.txt"), 
                  row.names = FALSE, col.names = FALSE)
      mrfG <- paste(sep = "", outFilePath, "mrfG.txt")
    } else if (is.null(mrfG)) {
      # save a meaningless mrfG.txt file to pass the parameter to C++
      mrfG <- matrix(c(0, 0, 0), ncol = 3)
      write.table(mrfG, paste(sep = "", outFilePath, "mrfG.txt"), 
                  row.names = FALSE, col.names = FALSE)
      mrfG <- paste(sep = "", outFilePath, "mrfG.txt")
    } else {
      my_stop("Unknown mrfG argument: check the help function for possibile 
              values", tmpFolder)
    }
  }
  
  ## Set up the XML file for hyperparameters
  xml <- as_xml_document(
    list(hyperparameters = list(
      lapply(hyperpar.all, function(x) list(x)) # every element in the list should be a list
    ))
  )
  hyperParFile <- paste(sep = "", tmpFolder, "hyperpar.xml")
  write_xml(xml, file = hyperParFile)
  
  ## Create the return object
  ret <- list(status = 1, input = list(), output = list())
  class(ret) <- "BayesSUR"
  
  # Copy the inputs
  ret$input["nIter"] <- nIter
  ret$input["burnin"] <- burnin
  ret$input["tick"] <- tick
  ret$input["nChains"] <- nChains
  ret$input["covariancePrior"] <- covariancePrior
  ret$input["gammaPrior"] <- gammaPrior
  ret$input["gammaSampler"] <- gammaSampler
  ret$input["gammaInit"] <- gammaInit
  ret$input["mrfG"] <- mrfG
  
  ret$input$hyperParameters <- hyperpar.all
  
  methodString <-
    switch(covariancePrior,
           "HIW" = "SSUR",
           "IW"  = "dSUR",
           "IG"  = "HRR"
    )
  
  ret$call <- cl
  
  # Prepare path to outputs
  ret$output["outFilePath"] <- outFilePath
  
  ret$output["logP"] <- 
    paste(sep = "", dataString, "_", methodString, "_logP_out.txt")
  
  if (output_gamma) {
    ret$output["gamma"] <- 
      paste(sep = "", dataString, "_", methodString, "_gamma_out.txt")
  }
  
  if (gammaPrior %in% c("hierarchical", "hotspot") && output_pi) {
    ret$output["pi"] <- 
      paste(sep = "", dataString, "_", methodString, "_pi_out.txt")
  }
  
  if (gammaPrior == "hotspot" && output_tail) {
    ret$output["tail"] <- 
      paste(sep = "", dataString, "_", methodString, "_hotspot_tail_p_out.txt")
  }
  
  if (output_beta) {
    ret$output["beta"] <- 
      paste(sep = "", dataString, "_", methodString, "_beta_out.txt")
  }
  
  if (covariancePrior == "HIW" && output_Gy) {
    ret$output["Gy"] <- 
      paste(sep = "", dataString, "_", methodString, "_Gy_out.txt")
    ret$output["Gvisit"] <- 
      paste(sep = "", dataString, "_", methodString, "_Gy_visit.txt")
  }
  
  if (covariancePrior %in% c("HIW", "IW") && output_sigmaRho) {
    ret$output["sigmaRho"] <- 
      paste(sep = "", dataString, "_", methodString, "_sigmaRho_out.txt")
  }
  
  if (output_model_size) {
    ret$output["model_size"] <- 
      paste(sep = "", dataString, "_", methodString, "_model_size_out.txt")
  }
  
  if (output_CPO) {
    ret$output["CPO"] <- 
      paste(sep = "", dataString, "_", methodString, "_CPO_out.txt")
    ret$output["CPOsumy"] <- 
      paste(sep = "", dataString, "_", methodString, "_CPOsumy_out.txt")
    ret$output["WAIC"] <- 
      paste(sep = "", dataString, "_", methodString, "_WAIC_out.txt")
  }
  
  if (output_Y) {
    ret$output["Y"] <- paste(sep = "", "data_Y.txt")
  }
  
  if (output_X) {
    ret$output["X"] <- paste(sep = "", "data_X.txt")
    if (length(X_0) > 0) {
      ret$output["X0"] <- paste(sep = "", "data_X0.txt")
    }
  }
  
  # set.seed(seed)
  # betaPrior="independent"
  
  # set number of threads
  maxThreads <- min(maxThreads, detectCores())
  
  ret$status <- BayesSUR_internal(data, mrfG, blockList, structureGraph, 
                                  hyperParFile, outFilePath, nIter, burnin, 
                                  nChains, covariancePrior, gammaPrior, 
                                  gammaSampler, gammaInit, betaPrior, 
                                  maxThreads, tick, output_gamma, output_beta, 
                                  output_Gy, output_sigmaRho, output_pi, 
                                  output_tail, output_model_size, output_CPO, 
                                  output_model_visit)
  
  if (toupper(gammaPrior) %in% c("HIERARCHICAL", "H")) {
    ## bring back the re-used hyperparameter names a_pi and b_pi for 
    ## Bernoulli prior's hyperparameters a_omega and b_omega
    names(ret$input$hyperParameters)[names(ret$input$hyperParameters) == "a_pi"] <- "a_omega"
    names(ret$input$hyperParameters)[names(ret$input$hyperParameters) == "b_pi"] <- "b_omega"
  }
  ## save fitted object
  obj_BayesSUR <- list(status = ret$status, input = ret$input, 
                       output = ret$output, call = ret$call)
  save(obj_BayesSUR, file = paste(sep = "", outFilePath, "obj_BayesSUR.RData"))

  if (outFilePath != tmpFolder) {
    unlink(tmpFolder, recursive = TRUE)
  }

  return(ret)
}


my_stop <- function(msg, tmpFolder) {
  unlink(tmpFolder, recursive = TRUE)
  stop(msg)
}

#' @title extract the posterior mean of parameters
#' @description
#' Extract the posterior mean of the parameters of a \code{BayesSUR} class object.
#' @name getEstimator
#' 
#' @param object an object of class \code{BayesSUR}
#' @param estimator the name of one estimator. Default is the latent indicator 
#' estimator "\code{gamma}". Other options "\code{beta}", "\code{Gy}", 
#' "\code{CPO}" and "\code{logP}"
#' correspond the marginal (conditional) coefficient matrix if 
#' \code{beta.type="marginal"}(\code{"conditional"}), response graph and 
#' conditional predictive ordinate (CPO) respectively
#' @param Pmax threshold that truncate the estimator "\code{gamma}" or 
#' "\code{Gy}". Default is \code{0}. If \code{Pmax=0.5} and 
#' \code{beta.type="conditional"}, it gives median probability model betas
#' @param beta.type the type of output beta. Default is \code{marginal}, giving 
#' marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
#' estimation conditional on gamma=1
#'
#' @return Return the estimator from an object of class \code{BayesSUR}. It is 
#' a matrix if the length of argument \code{marginal} is greater than 1. 
#' Otherwise, it is a list
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' # extract the posterior mean of the coefficients matrix
#' beta_hat <- getEstimator(fit, estimator = "beta")
#'
#' @export
getEstimator <- function(object, estimator = "gamma", Pmax = 0, 
                         beta.type = "marginal") {
  object$output[-1] <- 
    paste(object$output$outFilePath, object$output[-1], sep = "")
  
  if (sum(!estimator %in% c("gamma", "beta", "Gy", "CPO", "logP")) > 0) {
    stop("Please specify correct 'estimator'!")
  } else {
    ret <- rep(list(NULL), length(estimator))
    names(ret) <- estimator
  }

  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify correct argument 'Pmax' in [0,1]!")
  }

  if ("gamma" %in% estimator) {
    ret$gamma <- as.matrix(read.table(object$output$gamma))
    if (Pmax > 0) {
      ret$gamma[ret$gamma <= Pmax] <- 0
    }
    rownames(ret$gamma) <- colnames(read.table(object$output$X, header = TRUE))
    colnames(ret$gamma) <- colnames(read.table(object$output$Y, header = TRUE))
  }

  if ("beta" %in% estimator) {
    ret$beta <- as.matrix(read.table(object$output$beta))

    if (sum(beta.type %in% c("marginal", "conditional")) > 0) {
      if (beta.type == "conditional") {
        gammas <- as.matrix(read.table(object$output$gamma))

        if ("X0" %in% names(object$output)) {
          X0 <- as.matrix(read.table(object$output$X0))
          ret$beta[-seq_len(ncol(X0)), ] <- 
            (gammas >= Pmax) * ret$beta[-seq_len(ncol(X0)), ] / gammas
        } else {
          ret$beta <- (gammas >= Pmax) * ret$beta / gammas
        }
        ret$beta[is.na(ret$beta)] <- 0
      }
    } else {
      stop("Please specify correct beta.type!")
    }

    colnames(ret$beta) <- colnames(read.table(object$output$Y, header = TRUE))
    if ("X0" %in% names(object$output)) {
      rownames(ret$beta) <- 
        c(colnames(read.table(object$output$X0, header = TRUE)), 
          colnames(read.table(object$output$X, header = TRUE)))
    } else {
      rownames(ret$beta) <- 
        colnames(read.table(object$output$X, header = TRUE))
    }
  }

  if ("Gy" %in% estimator) {
    covariancePrior <- object$input$covariancePrior
    if (covariancePrior == "HIW") {
      ret$Gy <- as.matrix(read.table(object$output$Gy))
    } else {
      stop("Gy is only estimated with hyper-inverse Wishart prior for the 
           covariance matrix of responses!")
    }
    if (Pmax > 0) {
      ret$Gy[ret$Gy <= Pmax] <- 0
    }
    rownames(ret$Gy) <- colnames(ret$Gy) <- 
      names(read.table(object$output$Y, header = TRUE))
  }

  if ("CPO" %in% estimator) {
    if (is.null(object$output$CPO)) {
      stop("Please specify argument output_CPO in BayesSUR()!")
    }

    ret$CPO <- as.matrix(read.table(object$output$CPO))

    rownames(ret$CPO) <- 
      rownames(as.matrix(read.table(object$output$Y, header = TRUE)))
    colnames(ret$CPO) <- 
      colnames(as.matrix(read.table(object$output$Y, header = TRUE)))
  }

  if ("logP" %in% estimator) {
    ret$logP <- t(as.matrix(read.table(object$output$logP)))
  }

  if (length(estimator) > 1) {
    return(ret)
  } else {
    return(ret[[1]])
  }
}

#' @title plot network representation of the associations between responses and predictors
#' @description
#' Plot the network representation of the associations between responses and
#' predictors, based on the estimated gamma matrix and graph of responses
#' from a "BayesSUR" class object.
#'
#' @importFrom graphics text
#' @importFrom grDevices gray adjustcolor
#' @importFrom igraph V E gsize layout_in_circle plot.igraph degree
#' @importFrom igraph layout_with_fr delete_vertices
#' @importFrom igraph graph_from_adjacency_matrix delete_edges ecount V<-
#' @name plotNetwork
#' @param x an object of class \code{BayesSUR}
#' @param includeResponse A vector of the response names which are shown in the network
#' @param excludeResponse A vector of the response names which are not shown in the network
#' @param includePredictor A vector of the predictor names which are shown in the network
#' @param excludePredictor A vector of the predictor names which are not shown in the network
#' @param MatrixGamma A matrix or dataframe of the latent indicator variable.
#' Default is \code{NULL} and to extrate it from object of class inheriting
#' from an object of class \code{BayesSUR}
#' @param PmaxPredictor cutpoint for thresholding the estimated latent
#' indicator variable. Default is 0.5
#' @param PmaxResponse cutpoint for thresholding the learning structure matrix
#' of multiple response variables. Default is 0.5
#' @param nodesizePredictor node size of Predictors in the output graph.
#' Default is 15
#' @param nodesizeResponse node size of response variables in the output graph.
#' Default is 25
#' @param no.isolates remove isolated nodes from responses graph and full
#' graph, may get problem if there are also isolated Predictors
#' @param lineup A ratio of the heights between responses' area and predictors'
#' @param gray.alpha the opacity. The default is 0.6
#' @param edgewith.response the edge width between response nodes
#' @param edgewith.predictor the edge width between the predictor and response node
#' @param edge.weight draw weighted edges after thresholding at 0.5. The
#' default value \code{FALSE} is not to draw weighted edges
#' @param label.predictor A vector of the names of predictors
#' @param label.response A vector of the names of response variables
#' @param color.predictor color of the predictor nodes
#' @param color.response color of the response nodes
#' @param name.predictors A subtitle for the predictors
#' @param name.responses A subtitle for the responses
#' @param color.edge.positive color of the edges corresponding to estimated
#' positive regression coefficients
#' @param color.edge.negative color of the edges corresponding to estimated
#' negative regression coefficients
#' @param vertex.frame.color color of the frame of the vertices. If you don't
#' want vertices to have a frame, supply NA as the color name
#' @param layoutInCircle place vertices on a circle, in the order of their
#' vertex ids. The default is \code{FALSE}
#' @param header the main title
#' @param beta.type the type of output beta. Default is \code{marginal}, giving
#' marginal beta estimation. If \code{beta.type="conditional"}, it gives beta
#' estimation conditional on gamma=1
#' @param ... other arguments
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' # draw network representation of the associations between responses and covariates
#' plotNetwork(fit)
#'
#' @export
plotNetwork <- function(x, includeResponse = NULL, excludeResponse = NULL,
                        includePredictor = NULL, excludePredictor = NULL,
                        MatrixGamma = NULL, PmaxPredictor = 0.5,
                        PmaxResponse = 0.5, nodesizePredictor = 2,
                        nodesizeResponse = 15, no.isolates = FALSE,
                        lineup = 1.2, gray.alpha = 0.6, edgewith.response = 5,
                        edgewith.predictor = 2, edge.weight = FALSE,
                        label.predictor = NULL, label.response = NULL,
                        color.predictor = NULL, color.response = NULL,
                        name.predictors = NULL, name.responses = NULL,
                        color.edge.positive = "gray", color.edge.negative = "gray",
                        vertex.frame.color = NA, layoutInCircle = FALSE,
                        header = "", beta.type = "conditional", ...) {
  if (!inherits(x, "BayesSUR")) {
    stop("Use only with a \"BayesSUR\" object")
  }

  if (PmaxPredictor < 0 || PmaxPredictor > 1) {
    stop("Please specify correct argument 'PmaxPredictor' in [0,1]!")
  }
  if (PmaxResponse < 0 || PmaxResponse > 1) {
    stop("Please specify correct argument 'PmaxResponse' in [0,1]!")
  }

  # x$output[-1] <- paste(x$output$outFilePath, x$output[-1], sep = "")
  # beta_hat <- as.matrix(read.table(x$output$beta))
  # gamma_hat <- as.matrix(read.table(x$output$gamma))
  beta_hat <- getEstimator(x,
    estimator = "beta",
    Pmax = PmaxPredictor, beta.type = beta.type
  )
  if ("X0" %in% names(x$output)) {
    beta_hat <- beta_hat[-seq_len(ncol(as.matrix(read.table(
      paste0(x$output$outFilePath, x$output$X0)
    )))), ]
  }
  gamma_hat <- getEstimator(x, estimator = "gamma", Pmax = PmaxPredictor)
  # colnames(gamma_hat) <- names(read.table(x$output$Y, header = TRUE))
  # rownames(gamma_hat) <- colnames(read.table(x$output$X, header = TRUE))
  # covariancePrior <- x$input$covariancePrior
  if (x$input$covariancePrior == "HIW") {
    # Gy_hat <- as.matrix(read.table(x$output$Gy))
    Gy_hat <- getEstimator(x, estimator = "Gy", Pmax = PmaxResponse)
  } else {
    stop("Gy is only estimated with hyper-inverse Wishart prior for the
         covariance matrix of responses!")
  }

  if (sum(colnames(gamma_hat) ==
    paste("V", seq_len(ncol(gamma_hat)), sep = "")) == ncol(gamma_hat)) {
    colnames(gamma_hat) <- paste("Y", seq_len(ncol(gamma_hat)), sep = "")
  }
  if (sum(rownames(gamma_hat) ==
    paste("V", seq_len(NROW(gamma_hat)), sep = "")) == NROW(gamma_hat)) {
    rownames(gamma_hat) <- paste("X", seq_len(NROW(gamma_hat)), sep = "")
  }

  # select the required resposes and predictors to plot the network
  excludeResponse.idx <- rep(FALSE, ncol(gamma_hat))
  excludePredictor.idx <- rep(FALSE, NROW(gamma_hat))
  if (!is.null(includeResponse)) {
    excludeResponse.idx <- c(!(colnames(gamma_hat) %in% includeResponse))
  }
  if (!is.null(excludeResponse)) {
    excludeResponse.idx <- c(excludeResponse.idx |
      c(colnames(gamma_hat) %in% excludeResponse))
  }
  if (!is.null(includePredictor)) {
    excludePredictor.idx <- c(!(rownames(gamma_hat) %in% includePredictor))
  }
  if (!is.null(excludePredictor)) {
    excludePredictor.idx <- c(excludePredictor.idx |
      c(rownames(gamma_hat) %in% excludePredictor))
  }

  Gy_thresh <- Gy_hat[!excludeResponse.idx, !excludeResponse.idx]

  # gamma_hat <- gamma_hat[!excludePredictor.idx, !excludeResponse.idx]
  # beta_hat <- beta_hat[!excludePredictor.idx, !excludeResponse.idx]
  exclude.row.idx <- c((rowSums(gamma_hat) == 0) | excludePredictor.idx)
  gamma_thresh <- gamma_hat[!exclude.row.idx, !excludeResponse.idx]
  gamma_thresh <- matrix(gamma_thresh,
    nrow = sum(!exclude.row.idx),
    ncol = sum(!excludeResponse.idx)
  )
  beta_thresh <- beta_hat[!exclude.row.idx, !excludeResponse.idx]
  beta_thresh <- matrix(beta_thresh,
    nrow = sum(!exclude.row.idx),
    ncol = sum(!excludeResponse.idx)
  )
  colnames(gamma_thresh) <- colnames(gamma_hat)[!excludeResponse.idx]
  rownames(gamma_thresh) <- rownames(gamma_hat)[!exclude.row.idx]
  rownames(Gy_thresh) <- colnames(Gy_thresh) <- colnames(gamma_thresh)

  # if (edge.weight) {
  #   Gy_thresh <- Gy_hat
  #   Gy_thresh[Gy_hat <= PmaxResponse] <- 0
  #
  #   gamma_thresh <- gamma_hat
  #   gamma_thresh[gamma_hat <= PmaxPredictor] <- 0
  # } else {
  #   Gy_thresh <- as.matrix(Gy_hat > PmaxResponse)
  #   gamma_thresh <- as.matrix(gamma_hat > PmaxPredictor)
  # }
  if (edge.weight) {
    Gy_thresh <- matrix(as.numeric(Gy_thresh != 0),
      nrow = NROW(Gy_thresh),
      ncol = ncol(Gy_thresh)
    )
    gamma_hat <- matrix(as.numeric(gamma_hat != 0),
      nrow = NROW(gamma_hat),
      ncol = ncol(gamma_hat)
    )
  }

  if (sum(rowSums(gamma_hat) != 0) == 0) {
    stop(paste("There were no predictors with mPIP gamma > ", PmaxPredictor,
      ". Not able to draw a network!",
      sep = ""
    ))
  }

  gamma_thresh <- gamma_thresh[rev(seq_len(NROW(gamma_thresh))), ]
  beta_thresh <- beta_thresh[rev(seq_len(NROW(beta_thresh))), ]
  plotSEMgraph(Gy_thresh,
    t(gamma_thresh),
    t(beta_thresh),
    nodesizeSNP = nodesizePredictor,
    nodesizeMET = nodesizeResponse,
    no.isolates = no.isolates,
    edgewith.response = edgewith.response,
    edgewith.predictor = edgewith.predictor,
    edge.weight = edge.weight,
    label.predictor = label.predictor,
    label.response = label.response,
    color.predictor = color.predictor,
    color.response = color.response,
    name.predictors = name.predictors,
    name.responses = name.responses,
    color.edge.positive = color.edge.positive,
    color.edge.negative = color.edge.negative,
    vertex.frame.color = vertex.frame.color,
    layoutInCircle = layoutInCircle, ...
  )
  title(paste("\n\n", header, sep = ""), outer = TRUE)
}
plotSEMgraph <- function(ADJmatrix,
                         GAMmatrix,
                         BETAmatrix,
                         nodesizeSNP = 2,
                         nodesizeMET = 25,
                         no.isolates = FALSE,
                         lineup = 1,
                         gray.alpha = 0.6,
                         edgewith.response = 5,
                         edgewith.predictor = 2,
                         label.predictor = NULL,
                         label.response = NULL,
                         color.predictor = NULL,
                         color.response = NULL,
                         name.predictors = NULL,
                         name.responses = NULL,
                         color.edge.positive = "gray",
                         color.edge.negative = "gray",
                         edge.weight = FALSE,
                         vertex.frame.color = NA,
                         layoutInCircle = FALSE, ...) {
  ## give warnings for re-defined arguments
  if (exists("edge.width")) {
    warning("Argument 'edge.width' was re-defined into new argments
            'edgewith.response' and 'edgewith.predictor' in this function!")
  }
  if (exists("edge.color")) {
    warning("Argument 'edge.color' cannot be changed  in this function!")
  }
  if (exists("edge.arrow.size")) {
    warning("Argument 'edge.arrow.size' cannot be changed in this function!")
  }

  # ADJmatrix must be a square qxq adjacency matrix (or data frame)
  qq <- dim(ADJmatrix)[1]
  if (dim(ADJmatrix)[2] != qq) stop("adjacency matrix not square")

  # GAMmatrix must be a qxp binary matrix (or data frame)
  pp <- dim(GAMmatrix)[2]
  if (dim(GAMmatrix)[1] != qq) stop("Gamma and Adjacency have different no. q")

  # join mets block (adjency) and lower triangle (gamma)
  semgraph <- rbind(ADJmatrix, t(GAMmatrix))

  # add zero blocks for lower triangle and snp block
  zeroblock <- matrix(rep(0, pp * (pp + qq)), nrow = qq + pp, ncol = pp)
  zeroblock <- data.frame(zeroblock)
  colnames(zeroblock) <- colnames(GAMmatrix)
  # rownames(zeroblock) <- rownames(semgraph)

  semgraph <- cbind(semgraph, zeroblock)

  # igraph objects
  graphADJ <- graph_from_adjacency_matrix(as.matrix(ADJmatrix),
    weighted = TRUE,
    diag = FALSE, mode = "undirected"
  )
  graphSEM <- graph_from_adjacency_matrix(as.matrix(semgraph),
    weighted = TRUE,
    diag = FALSE, mode = "directed"
  )

  # don't plot isolated nodes?
  if (no.isolates) {
    graphADJ <- delete_vertices(graphADJ, degree(graphADJ) == 0)
    graphSEM <- delete_vertices(graphSEM, degree(graphSEM) == 0)
    message("Removing isolated nodes from Adjacency and Full SEM, may get
            problem if there are also isolated SNPs.")
  }

  # get co-ords for undirected edges using layout function (scaled)
  lladj <- igraph::layout_with_fr(graphADJ)
  lmax <- max(lladj[, 1])
  lmin <- min(lladj[, 1])
  lladj[, 1] <- (lladj[, 1] - lmin) / (lmax - lmin)
  lmax <- max(lladj[, 2])
  lmin <- min(lladj[, 2])
  lladj[, 2] <- (lladj[, 2] - lmin) / (lmax - lmin)

  # plot adjacency only
  # plot(graphADJ,vertex.size=15,edge.width=2,edge.color="black",layout=lladj)

  # line up snps
  lymax <- max(lladj[, 2]) + (max(lladj[, 2]) -
    min(lladj[, 2])) * (lineup - 1) / 2
  lymin <- min(lladj[, 2]) + (max(lladj[, 2]) -
    min(lladj[, 2])) * (1 - lineup) / 2
  llsnps <- matrix(c(rep(-0.5, pp), lymin + (1:pp) * 1.0 *
    (lymax - lymin) / pp), nrow = pp, ncol = 2)

  llsem <- rbind(lladj, llsnps)

  ### plot SEM

  # plot snps and mets nodes differently
  # set node sizes directly in graph object
  V(graphSEM)$size <- c(rep(nodesizeMET, qq), rep(nodesizeSNP, pp))

  n.edgeADJ <- gsize(graphADJ)
  n.edgeGAM <- gsize(graphSEM) - n.edgeADJ

  V(graphSEM)$label.color <- "black"

  V(graphSEM)$color <- c(
    rep("dodgerblue", NROW(GAMmatrix)),
    rep("red", ncol(GAMmatrix))
  )
  if (!is.null(color.predictor)) {
    V(graphSEM)$color[-c(seq_len(NROW(GAMmatrix)))] <- color.predictor
  }
  if (!is.null(color.response)) {
    V(graphSEM)$color[seq_len(NROW(GAMmatrix))] <- color.response
  }

  V(graphSEM)$label <- c(rownames(GAMmatrix), colnames(GAMmatrix))
  if (!is.null(label.predictor)) {
    V(graphSEM)$label[-c(seq_len(NROW(GAMmatrix)))] <- label.predictor
  }
  if (!is.null(label.response)) {
    V(graphSEM)$label[seq_len(NROW(GAMmatrix))] <- label.response
  }

  if (edge.weight) {
    edge.width <- E(graphSEM)$weight * ifelse(edge.weight, 5, 1)
  } else {
    edge.width <- c(
      rep(edgewith.response, 2 * n.edgeADJ),
      rep(edgewith.predictor, 2 * n.edgeGAM)
    )
  }

  if (!layoutInCircle) {
    layoutSEM <- llsem
  } else {
    layoutSEM <- layout_in_circle(graphSEM)
  }

  if (n.edgeADJ > 0) {
    graphSEMresponses <- delete_edges(
      graphSEM, E(graphSEM)[(1:ecount(graphSEM))[-c(1:(2 * n.edgeADJ))]]
    )
    graphSEMpredictor2responses <-
      delete_edges(
        graphSEM,
        E(graphSEM)[(1:ecount(graphSEM))[c(1:(2 * n.edgeADJ))]]
      )
  } else {
    graphSEMresponses <- graphSEM
    graphSEMpredictor2responses <- graphSEM
  }
  # plot undirected graph between response variables
  plot.igraph(graphSEMresponses,
    edge.arrow.size = 0,
    edge.width = edge.width[1:(2 * n.edgeADJ)],
    vertex.frame.color = vertex.frame.color,
    edge.color = rep(gray(0), 2 * n.edgeADJ),
    layout = layoutSEM, ...
  )

  # plot directed graph between predictors and response variables
  edge.colors <- matrix("white", nrow = NROW(BETAmatrix), ncol = ncol(BETAmatrix))
  edge.colors[BETAmatrix < 0] <- color.edge.negative
  edge.colors[BETAmatrix > 0] <- color.edge.positive
  edge.colors <- as.vector(edge.colors[edge.colors != "white"])

  plot.igraph(graphSEMpredictor2responses,
    edge.arrow.size = 0.5,
    edge.width = edge.width[-c(1:(2 * n.edgeADJ))],
    vertex.frame.color = vertex.frame.color,
    # edge.color = rep(gray(0.7, alpha = gray.alpha), 2 * n.edgeGAM),
    edge.color = adjustcolor(edge.colors, gray.alpha),
    layout = layoutSEM, add = TRUE, ...
  )

  if (!is.null(name.predictors)) text(-1, -1.3, name.predictors, cex = 1.2)
  if (!is.null(name.responses)) text(0.4, -1.3, name.responses, cex = 1.2)
}

#' @title expected log pointwise predictive density
#' @description
#' Measure the prediction accuracy by the elpd (expected log pointwise 
#' predictive density). The out-of-sample predictive fit can either be 
#' estimated by Bayesian leave-one-out cross-validation (LOO) or by widely 
#' applicable information criterion (WAIC) (Vehtari et al. 2017).
#' @name elpd
#' 
#' @param object an object of class \code{BayesSUR}
#' @param method the name of the prediction accuracy index. Default is the 
#' \code{"LOO"} (Bayesian LOO estimate of out-of-sample predictive fit). The 
#' other index is the \code{"WAIC"} (widely applicable information criterion).
#' For the HRR models, both "\code{LOO}" and "\code{WAIC}" are computed based 
#' on the multivate t-distribution of the posterior predictive rather than 
#' approximation of importance sampling.
#'
#' @return Return the predictiion accuracy measure from an object of class 
#' \code{BayesSUR}. It is elpd.loo if the argumnet \code{method="LOO"} and 
#' elpd.WAIC if \code{method="WAIC"}.
#'
#' @references Vehtari, A., Gelman, A., Gabry, J. (2017). \emph{Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC.} Statistics and Computing, 27(5): 1413–1432.
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
#' )
#'
#' ## check output
#' # print prediction accuracy elpd (expected log pointwise predictive density)
#' # by the Bayesian LOO estimate of out-of-sample predictive fit
#' elpd(fit, method = "LOO")
#'
#' @export
elpd <- function(object, method = "LOO") {
  object$output[-1] <- 
    paste(object$output$outFilePath, object$output[-1], sep = "")

  if (is.null(object$output$CPO)) {
    stop("Please specify the argument 'output_CPO = TRUE' in BayesSUR()!")
  }

  if (toupper(method) == "LOO") {
    elpd <- sum(log(read.table(object$output$CPO)))
    names(elpd) <- "elpd.loo"
  } else if (toupper(method) == "WAIC") {
    elpd <- sum(read.table(object$output$WAIC))
    names(elpd) <- "elpd.waic"
  } else {
    stop("Please give the correct method name!")
  }

  return(elpd)
}

#' @title predict method for class \code{BayesSUR}
#' @description
#' Predict responses corresponding to the posterior mean of the coefficients, 
#' return posterior mean of coefficients or indices of nonzero coefficients of 
#' a \code{BayesSUR} class object.
#' @name predict.BayesSUR
#'
#' @param object an object of class \code{BayesSUR}
#' @param newx Matrix of new values for x at which predictions are to be made
#' @param type Type of prediction required. \code{type="response"} gives the 
#' fitted responses; \code{type="coefficients"} returns the estimated 
#' coefficients depending on the arguments \code{beta.type} and \code{Pmax}.
#' \code{type="nonzero"} returns a list of the indices of the nonzero 
#' coefficients corresponding to the estimated latent indicator variable 
#' thresholding at \code{Pmax}
#' @param beta.type the type of estimated coefficients beta for prediction. 
#' Default is \code{marginal}, giving marginal beta estimation. If 
#' \code{beta.type="conditional"}, it gives conditional beta estimation
#' @param Pmax If \code{type="nonzero"}, it is a threshold for the estimated 
#' latent indicator variable. If \code{type="coefficients"}, 
#' \code{beta.type="conditional"} and \code{Pmax=0.5}, it gives median 
#' probability model betas. Default is 0
#' @param ... other arguments
#'
#' @return Predicted values extracted from an object of class \code{BayesSUR}. 
#' If the \code{BayesSUR} specified data standardization, the fitted values 
#' are base based on standardized data.
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 20, burnin = 10, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check prediction
#' predict.val <- predict(fit, newx = exampleEQTL[["blockList"]][[2]])
#'
#' @export
predict.BayesSUR <- function(object, newx, type = "response", 
                             beta.type = "marginal", Pmax = 0, ...) {
  if (length(type) > 1) {
    warning("'type' has length > 1 and only the first element will be used")
    type <- type[1]
  }
  if (!(type %in% c("response", "coefficients", "nonzero"))) {
    stop("Please specify correct 'type'!")
  }
  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify correct argument 'Pmax' in [0,1]!")
  }
  if (!(beta.type %in% c("marginal", "conditional"))) {
    stop("Please specify acorrect 'beta.type'!")
  }

  if ((type %in% c("response", "coefficients")) && (Pmax > 0) && 
      (beta.type == "marginal")) {
    stop("Pmax > 0 is valid only if the arguments type='coefficients' and 
         beta.type='conditional'!")
  }

  gamma_hat <- getEstimator(object, estimator = "gamma", Pmax = Pmax, ...)
  beta_hat <- getEstimator(object, estimator = "beta", Pmax = Pmax, 
                           beta.type = beta.type, ...)

  object$output[-1] <- paste(object$output$outFilePath, 
                             object$output[-1], sep = "")
  X <- as.matrix(read.table(object$output$X, header = TRUE))
  if ("X0" %in% names(object$output)) {
    X0 <- as.matrix(read.table(object$output$X0))
  } else {
    X0 <- NULL
  }

  if (missing(newx)) {
    y.pred <- cbind(X0, X) %*% beta_hat
  } else {
    y.pred <- newx %*% beta_hat
  }

  gamma_out <- which(gamma_hat == 1, arr.ind = TRUE)
  colnames(gamma_out) <- c("predictors", "response")

  if (type == "response") {
    return(y.pred)
  }
  if (type == "coefficients") {
    return(beta_hat)
  }
  if (type == "nonzero") {
    return(gamma_out)
  }
}

#' @title plot graph for response variables
#' @description
#' Plot the estimated graph for multiple response variables from a 
#' \code{BayesSUR} class object.
#' @importFrom igraph E plot.igraph graph_from_adjacency_matrix
#' @importFrom graphics par
#' @name plotGraph
#' 
#' @param x either an object of class \code{BayesSUR} (default) or a symmetric 
#' numeric matrix representing an adjacency matrix for a given graph structure.
#' If x is an adjacency matrix, argument \code{main="Given graph of responses"} 
#' by default.
#' @param Pmax a value for thresholding the learning structure matrix of 
#' multiple response variables. Default is 0.5
#' @param main an overall title for the plot
#' @param edge.width edge width. Default is 2
#' @param edge.weight draw weighted edges after thresholding at 0.5. The 
#' default value \code{FALSE} is not to draw weighted edges
#' @param vertex.label character vector used to label the nodes
#' @param vertex.label.color label color. Default is \code{"black"}
#' @param vertex.size node size. Default is 30
#' @param vertex.color node color. Default is \code{"dodgerblue"}
#' @param vertex.frame.color node color. Default is \code{"NA"}
#' @param ... other arguments
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' # show the graph relationship between responses
#' plotGraph(fit, estimator = "Gy")
#'
#' @export
plotGraph <- function(x, Pmax = 0.5, main = "Estimated graph of responses", 
                      edge.width = 2, edge.weight = FALSE, vertex.label = NULL, 
                      vertex.label.color = "black", vertex.size = 30, 
                      vertex.color = "dodgerblue", vertex.frame.color = NA, ...) {
  if (!inherits(x, "BayesSUR")) {
    if (is.matrix(x) && is.numeric(x)) {
      if (!((dim(x)[1] == dim(x)[2]) && (sum(dim(x)) > 2))) {
        stop("Use only with a \"BayesSUR\" object or numeric square matrix")
      }
      Gy_hat <- x
      if (!is.null(vertex.label)) {
        rownames(Gy_hat) <- colnames(Gy_hat) <- vertex.label
      }
      if (main == "Estimated graph of responses") {
        main <- "Given graph of responses"
      }
    } else {
      stop("Use only with a \"BayesSUR\" object or numeric square matrix")
    }
  } else {
    x$output[-1] <- paste0(x$output$outFilePath, x$output[-1])
    covariancePrior <- x$input$covariancePrior
    if (covariancePrior == "HIW") {
      Gy_hat <- as.matrix(read.table(x$output$Gy))
    } else {
      stop("Gy is only estimated with hyper-inverse Wishart prior for the 
           covariance matrix of responses!")
    }

    if (!is.null(vertex.label)) {
      rownames(Gy_hat) <- colnames(Gy_hat) <- vertex.label
    } else {
      rownames(Gy_hat) <- colnames(Gy_hat) <- 
        names(read.table(x$output$Y, header = TRUE))
    }
  }

  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify correct argument 'Pmax' in [0,1]!")
  }

  if (edge.weight) {
    Gy_thresh <- Gy_hat
    Gy_thresh[Gy_hat <= Pmax] <- 0
  } else {
    Gy_thresh <- as.matrix(Gy_hat > Pmax)
  }

  net <- graph_from_adjacency_matrix(Gy_thresh, weighted = TRUE, 
                                     mode = "undirected", diag = FALSE)

  if (edge.weight) {
    plot.igraph(net, 
                main = main, 
                edge.width = E(net)$weight * 2, 
                vertex.label = vertex.label, 
                vertex.color = vertex.color, 
                vertex.frame.color = vertex.frame.color, ...)
  } else {
    plot.igraph(net, 
                main = main, 
                edge.width = edge.width, 
                vertex.label = vertex.label, 
                vertex.color = vertex.color, 
                vertex.frame.color = vertex.frame.color, 
                vertex.label.color = vertex.label.color, 
                vertex.size = vertex.size, ...)
  }
}

#' @title summary method for class \code{BayesSUR}
#' @description
#' Summary method for class \code{BayesSUR}. It includes the argument matching 
#' information, Top predictors/responses on average mPIP across all 
#' responses/predictors, elpd estimates, MCMC specification, model 
#' specification and hyper-parameters. The summarized number of the selected 
#' variable corresponds to the posterior mean of the latent indicator variable 
#' thresholding at 0.5 by default.
#'
#' @importFrom Matrix Matrix
#' @name summary.BayesSUR
#' @param object an object of class \code{BayesSUR}
#' @param Pmax threshold that truncates the estimated coefficients based on 
#' thresholding the estimated latent indicator variable. Default is 0.5
#' @param ... other arguments
#'
#' @return Return a result summary from an object of class \code{BayesSUR}, 
#' including the CPOs, number of selected predictors with mPIP>\code{Pmax}, 
#' top 10 predictors on average mPIP across all responses, top 10 responses on 
#' average mPIP across all predictors, Expected log pointwise predictive 
#' density (elpd) estimates, MCMC specification, model specification (i.e., 
#' covariance prior and gamma prior) and hyper-parameters.
#'
#' @examples
#' data(exampleEQTL, package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
#' )
#'
#' ## check output
#' # show the summary information
#' summary(fit)
#'
#' @export
summary.BayesSUR <- function(object, Pmax = 0.5, ...) {
  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify correct argument 'Pmax' in (0,1)!")
  }

  ans <- list(status = object$status)
  if (is.null(object$output$CPO)) {
    ans$elpd <- NA
  } else {
    ans$elpd <- c(elpd(object, method = "LOO"), elpd(object, method = "WAIC"))
  }

  object$output[-1] <- 
    paste(object$output$outFilePath, object$output[-1], sep = "")
  if (is.null(object$output$CPO)) {
    ans$CPO <- NA
  } else {
    ans$CPO <- 
      summary.default(as.vector(as.matrix(read.table(object$output$CPO))))[-4]
  }

  gamma <- as.matrix(read.table(object$output$gamma))
  ans$df <- sum(gamma > Pmax)

  # extract top 10 covariates based on average mPIP across all responses
  mean.predictors <- rowMeans(gamma)
  top10.predictors <- mean.predictors[
    sort.list(mean.predictors, decreasing = TRUE)[1:min(10, nrow(gamma))]]
  names(top10.predictors) <- names(read.table(object$output$X, header = TRUE))[
    sort.list(mean.predictors, decreasing = TRUE)[1:min(10, nrow(gamma))]]

  # extract top 10 response variables based on average mPIP across responses
  mean.responses <- colMeans(gamma)
  top10.responses <- mean.responses[
    sort.list(mean.responses, decreasing = TRUE)[1:min(10, ncol(gamma))]]
  names(top10.responses) <- names(read.table(object$output$Y, header = TRUE))[
    sort.list(mean.responses, decreasing = TRUE)[1:min(10, ncol(gamma))]]

  ans$chainParameters <- object$input[1:3]
  ans$modelParameters <- object$input[4:9]
  ans$hyperParameters <- object$input$hyperparameters
  ans$outputFiles <- object$output
  ans$outputFiles["outFilePath"] <- NULL

  cat("\nCall:\n  ", 
      paste(unlist(strsplit(deparse(object$call), ","))[1:3], 
            c(",", ",", ", ...)"), sep = "", collapse = ""), "\n", sep = "")
  cat("\nCPOs:\n")
  print(ans$CPO)
  cat("\nNumber of selected predictors (mPIP > ", Pmax, "): ", 
      sum(gamma > Pmax), " of ", ncol(gamma), "x", nrow(gamma), 
      "\n", sep = "")
  cat("\nTop", min(10, nrow(gamma)), 
      "predictors on average mPIP across all responses:\n")
  print(top10.predictors)

  cat("\nTop", min(10, ncol(gamma)), 
      "responses on average mPIP across all predictors:\n")
  print(top10.responses)

  cat("\nExpected log pointwise predictive density (elpd) estimates:\n", 
      "  elpd.LOO = ", ans$elpd[1], ",  elpd.WAIC = ", ans$elpd[2], "\n", 
      sep = "")
  cat("\nMCMC specification:\n", "  iterations = ", ans$chainParameters$nIter, 
      ",  burn-in = ", ans$chainParameters$burnin, 
      ",  chains = ", ans$chainParameters$nChains,
      "\n  gamma local move sampler: ", ans$modelParameters$gammaSampler, 
      "\n  gamma initialisation: ", ans$modelParameters$gammaInit, "\n",
      sep = "")
  cat("\nModel specification:\n", "  covariance prior: ", 
      ans$modelParameters$covariancePrior, "\n  gamma prior: ", 
      ans$modelParameters$gammaPrior, "\n", sep = "")

  if (is.null(ans$hyperParameters)) {
    cat("\nHyper-parameters:\n")
    print(unlist(object$input$hyperParameters))
  }
  cat("\n")

  invisible(ans)
}

#' @title plot heatmap of estimators
#' @description
#' Plot the posterior mean estimators from a \code{BayesSUR} class object, 
#' including the coefficients beta, latent indicator variable gamma and 
#' graph of responses.
#' @importFrom graphics axis box text mtext par image
#' @importFrom grDevices colorRampPalette dev.off grey
#' @importFrom stats sd
#' @importFrom tikzDevice tikz
#' @name plotEstimator
#' 
#' @param x an object of class \code{BayesSUR}
#' @param estimator print the heatmap of estimators. The value "beta" is for 
#' the estimated coefficients matrix, "gamma" for the latent indicator matrix 
#' and "Gy" for the graph of responses
#' @param colorScale.gamma value palette for gamma
#' @param colorScale.beta a vector of three colors for diverging color schemes
#' @param legend.cex.axis magnification of axis annotation relative to cex
#' @param name.responses a vector of the response names. The default is 
#' \code{NA} only to show the locations. The value \code{"auto"} show the 
#' response names from the orginal data.
#' @param name.predictors a vector of the predictor names. The default is 
#' \code{NA} only to show the locations. The value \code{"auto"} show the 
#' predictor names from the orginal data.
#' @param xlab a title for the x axis
#' @param ylab a title for the y axis
#' @param fig.tex print the figure through LaTex. Default is \code{FALSE}
#' @param output the file name of printed figure
#' @param header the main title
#' @param header.cex size of the main title for all estimators
#' @param cex.main size of the title for each estimator
#' @param mgp the margin line (in mex units) for the axis title, axis labels 
#' and axis line
#' @param title.beta a title for the printed "beta"
#' @param title.gamma a title for the printed "gamma"
#' @param title.Gy a title for the printed "Gy"
#' @param tick a logical value specifying whether tickmarks and an axis line 
#' should be drawn. Default is \code{FALSE}
#' @param beta.type the type of output beta. Default is \code{marginal}, giving 
#' marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
#' estimation conditional on gamma=1
#' @param Pmax threshold that truncate the estimator "\code{gamma}" or 
#' "\code{Gy}". Default is \code{0}. If \code{Pmax=0.5} and 
#' \code{beta.type="conditional"}, it gives median probability model betas
#' @param ... other arguments
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' # Plot the estimators from the fitted object
#' plotEstimator(fit, estimator = c("beta", "gamma", "Gy"))
#'
#' \dontrun{
#' ## Set up temporary work directory for saving a pdf figure
#' # td <- tempdir()
#' # oldwd <- getwd()
#' # setwd(td)
#'
#' ## Produce authentic math formulas in the graph
#' # plotEstimator(fit, estimator = c("beta", "gamma", "Gy"), fig.tex = TRUE)
#' # system(paste(getOption("pdfviewer"), "ParamEstimator.pdf"))
#' # setwd(oldwd)
#' }
#'
#' @export
plotEstimator <- function(x, estimator = NULL,
                          colorScale.gamma = grey((100:0) / 100),
                          colorScale.beta = c("blue", "white", "red"),
                          legend.cex.axis = 1, name.responses = NA,
                          name.predictors = NA, xlab = "", ylab = "",
                          fig.tex = FALSE, output = "ParamEstimator",
                          header = "", header.cex = 2, tick = FALSE,
                          mgp = c(2.5, 1, 0), cex.main = 1.5,
                          title.beta = NA, title.gamma = NA, title.Gy = NA,
                          beta.type = "marginal", Pmax = 0, ...) {
  if (!inherits(x, "BayesSUR")) {
    stop("Use only with a \"BayesSUR\" object")
  }
  if (sum(!estimator %in% c("beta", "gamma", "Gy")) > 0) {
    stop("Please specify correct argument estimator!")
  }
  
  beta_hat <- getEstimator(x,
                           estimator = "beta",
                           beta.type = beta.type, Pmax = Pmax
  )
  x$output[-1] <- paste0(x$output$outFilePath, x$output[-1])
  gamma_hat <- as.matrix(read.table(x$output$gamma))
  response_idx <- seq_len(ncol(gamma_hat))
  predictor_idx <- seq_len(nrow(gamma_hat))
  nonpen <- nrow(beta_hat) - nrow(gamma_hat)
  predictor_idx0 <- c(seq_len(nonpen), nonpen + predictor_idx)
  if (nonpen > 0) {
    rownames(beta_hat) <- c(
      colnames(read.table(x$output$X0, header = TRUE)),
      colnames(read.table(x$output$X, header = TRUE))
    )
  } else {
    rownames(beta_hat) <- colnames(read.table(x$output$X, header = TRUE))
  }
  colnames(beta_hat) <- colnames(read.table(x$output$Y, header = TRUE))
  
  covariancePrior <- x$input$covariancePrior
  if ((covariancePrior == "HIW") && ("Gy" %in% estimator)) {
    Gy_hat <- as.matrix(read.table(x$output$Gy))
    colnames(Gy_hat) <- rownames(Gy_hat) <-
      colnames(read.table(x$output$Y, header = TRUE))
  }
  
  # specify the labels of axes
  if (is.na(name.responses)[1]) {
    name.responses <- response_idx
  } else if (name.responses[1] == "auto") {
    name.responses <- colnames(beta_hat)
  } else if (is.character(name.responses)) {
    if (length(name.responses) != ncol(beta_hat)) {
      stop("The length of the given response names are not consistent with the data!")
    }
  }
  
  if (is.na(name.predictors)) {
    name.predictors <- predictor_idx0
  } else if (name.predictors[1] == "auto") {
    name.predictors <- rownames(beta_hat)
  } else if (is.character(name.predictors)) {
    if (length(name.predictors) != nrow(beta_hat)) {
      stop("The length of the given predictor names are not consistent with the data!")
    }
  }
  
  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  if (!fig.tex) {
    par(mar = c(6, 6, 5.1, 4.1))
    par(mfrow = c(1, sum(estimator %in% c("beta", "gamma", "Gy"))))
    
    if ("beta" %in% estimator) {
      # floor(100*constant)+100-1 colours that your want in the legend bar which has the white middle color
      if (stats::sd(beta_hat) == 0) {
        colorbar <- colorRampPalette(c(colorScale.beta[2], colorScale.beta[2]))(1000)
      } else {
        colorbar <- c(
          colorRampPalette(c(colorScale.beta[1], colorScale.beta[2]))(
            floor(1000 / (-(max(beta_hat) - min(beta_hat)) / min(beta_hat) - 1))),
          colorRampPalette(c(colorScale.beta[2], colorScale.beta[3]))(1000)[-1]
        )
      }
      if (is.na(title.beta)) title.beta <- expression(hat(bold(B)))
      
      image(
        z = beta_hat, x = predictor_idx0, y = response_idx,
        col = colorbar, mgp = mgp,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE),
        xlab = xlab, ylab = ylab, main = title.beta,
        cex.main = cex.main, cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorbar,
        zlim = c(min(beta_hat), max(beta_hat)),
        legend.cex.axis = legend.cex.axis
      )
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2, at = response_idx, labels = name.responses, tick = tick)
        axis(1, at = predictor_idx0, labels = name.predictors, tick = tick)
      }
    }
    if ("gamma" %in% estimator) {
      if (is.na(title.gamma)) title.gamma <- expression(hat(bold(Gamma)))
      image(
        z = gamma_hat, x = predictor_idx, y = response_idx,
        col = colorScale.gamma, mgp = mgp,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE),
        xlab = xlab, ylab = ylab, main = title.gamma,
        cex.main = cex.main, cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorScale.gamma, zlim = c(0, 1),
        legend.cex.axis = legend.cex.axis
      )
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2, at = response_idx, labels = name.responses, tick = tick)
        if (nonpen > 0) {
          name.predictors <- name.predictors[-c(1:nonpen)]
        }
        axis(1, at = predictor_idx, labels = name.predictors, tick = tick)
      }
    }
    
    if ("Gy" %in% estimator) {
      if (is.na(title.Gy)) title.Gy <- "Estimated graph of responses"
      image(
        z = Gy_hat + diag(ncol(Gy_hat)),
        x = response_idx, y = response_idx,
        col = colorScale.gamma, mgp = mgp,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE),
        xlab = ylab, ylab = ylab, main = title.Gy,
        cex.main = cex.main, cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorScale.gamma,
        zlim = c(min(Gy_hat), max(Gy_hat)),
        legend.cex.axis = legend.cex.axis
      )
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2, at = response_idx, labels = name.responses, tick = tick)
        axis(1, at = response_idx, labels = name.responses, tick = tick)
      }
    }
    title(paste0("\n", header), cex.main = header.cex, outer = TRUE)
  } else {
    options(tikzMetricPackages = c(
      "\\usepackage{amsmath}", "\\usepackage{bm}",
      "\\usetikzlibrary{calc}"
    ))
    tikz(paste0(output, ".tex"),
         width = 3.6 * sum(estimator %in% c("beta", "gamma", "Gy")),
         height = 4,
         standAlone = TRUE,
         packages = c(
           "\\usepackage{tikz}",
           "\\usepackage{amsmath}",
           "\\usepackage{bm}",
           "\\usepackage[active,tightpage,psfixbb]{preview}",
           "\\PreviewEnvironment{pgfpicture}"
         )
    )
    
    par(mfrow = c(1, sum(estimator %in% c("beta", "gamma", "Gy"))))
    par(mar = c(6, 6, 4, 4) + 0.1)
    if ("beta" %in% estimator) {
      # floor(100*constant)+100-1 colours that your want in the legend bar which has the white middle color
      colorbar <-
        c(
          colorRampPalette(c(colorScale.beta[1], colorScale.beta[2]))(
            floor(1000 / (-(max(beta_hat) - min(beta_hat)) / min(beta_hat) - 1))),
          colorRampPalette(c(colorScale.beta[2], colorScale.beta[3]))(1000)[-1]
        )
      if (is.na(title.beta)) {
        title.beta <- paste("Estimator", "$\\hat{\\bm{B}}$")
      }
      
      image(
        z = beta_hat, x = predictor_idx0, y = response_idx, col = colorbar,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE), mgp = mgp,
        xlab = xlab, ylab = ylab, main = title.beta,
        cex.main = cex.main, cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorbar,
        zlim = c(min(beta_hat), max(beta_hat)),
        legend.cex.axis = legend.cex.axis
      )
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2, at = response_idx, labels = name.responses, tick = tick)
        # opar <- par(cex.axis=1)
        axis(1, at = predictor_idx0, labels = name.predictors, tick = tick)
      }
    }
    if ("gamma" %in% estimator) {
      if (is.na(title.gamma)) {
        title.gamma <- paste("Estimator", "$\\hat{\\mathbf{\\Gamma}}$")
      }
      image(
        z = gamma_hat, x = predictor_idx, y = seq_len(ncol(gamma_hat)),
        col = colorScale.gamma,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE),
        xlab = xlab, ylab = ylab, main = title.gamma, mgp = mgp,
        cex.main = cex.main, cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorScale.gamma,
        zlim = c(0, 1), legend.cex.axis = legend.cex.axis
      )
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2,
             at = seq_len(ncol(gamma_hat)),
             labels = name.responses, tick = tick
        )
        if (nonpen > 0) {
          name.predictors <- name.predictors[-c(1:nonpen)]
        }
        axis(1, at = predictor_idx, labels = name.predictors, tick = tick)
      }
    }
    
    if ("Gy" %in% estimator) {
      if (is.na(title.Gy)) {
        title.Gy <- paste("Estimator", "$\\hat{\\mathcal{G}}$")
      }
      image(
        z = Gy_hat + diag(ncol(Gy_hat)),
        x = seq_len(nrow(Gy_hat)),
        y = seq_len(nrow(Gy_hat)),
        col = colorScale.gamma,
        axes = ifelse(is.na(name.responses)[1], TRUE, FALSE),
        mgp = mgp,
        xlab = ylab,
        ylab = ylab,
        main = title.Gy,
        cex.main = cex.main,
        cex.lab = 1.5, ...
      )
      box()
      vertical.image.legend(
        color = colorScale.gamma,
        zlim = c(min(Gy_hat), max(Gy_hat)),
        legend.cex.axis = legend.cex.axis
      )
      
      if (!is.na(name.responses)[1]) {
        par(las = 2, cex.axis = 1)
        axis(2,
             at = seq_len(ncol(Gy_hat)),
             labels = name.responses, tick = tick
        )
        axis(1,
             at = seq_len(nrow(Gy_hat)),
             labels = name.responses, tick = tick
        )
      }
    }
    title(paste0("\n", header), cex.main = header.cex, outer = TRUE)
    dev.off()
    tools::texi2pdf(paste0(output, ".tex"))
  }
}

# the function vertical.image.legend() is adapted from the R package "aqfig"
vertical.image.legend <- function(zlim, color, legend.cex.axis = 1) {
  starting.par.settings <- par(no.readonly = TRUE)
  mai <- par("mai")
  fin <- par("fin")
  x.legend.fig <- c(1 - (mai[4] / fin[1]), 1)
  y.legend.fig <- c(mai[1] / fin[2], 1 - (mai[3] / fin[2]))
  x.legend.plt <- c(x.legend.fig[1] + (0.18 * (x.legend.fig[2] -
    x.legend.fig[1])), x.legend.fig[2] - (0.6 * (x.legend.fig[2] -
    x.legend.fig[1])))
  y.legend.plt <- y.legend.fig
  cut.pts <- seq(zlim[1], zlim[2], length = length(color) + 1)
  z <- (cut.pts[seq_along(color)] + cut.pts[2:(length(color) + 1)]) / 2
  par(new = TRUE, pty = "m", plt = c(x.legend.plt, y.legend.plt))
  # If z is not increasing, only two values
  if (all(diff(z) > 0)) {
    image(
      x = 1.5, y = z, z = matrix(z, nrow = 1, ncol = length(color)),
      col = color, xlab = "", ylab = "", xaxt = "n", yaxt = "n"
    )
    axis(4, mgp = c(3, 0.2, 0), las = 2, cex.axis = legend.cex.axis, tcl = -0.1)
    box()
  }
  mfg.settings <- par()$mfg
  par(starting.par.settings)
  par(mfg = mfg.settings, new = FALSE)
}

#' @title create a selection of plots
#' @description
#' plot method for class \code{BayesSUR}. This is the main plot function to be 
#' called by the user. This function calls one or several of the following 
#' functions: \code{plotEstimator()}, \code{plotGraph()}, \code{plotMCMCdiag()}, 
#' \code{plotManhattan()}, \code{plotNetwork()}, \code{plotCPO()}.
#' @importFrom grDevices dev.hold dev.flush devAskNewPage
#' @name plot.BayesSUR
#' 
#' @param x an object of class \code{BayesSUR}
#' @param estimator It is in \code{c(NULL, 'beta', 'gamma', 'Gy', 'logP', 'CPO')} 
#' and works by combining with argument \code{type}.
#' \itemize{
#'   \item If \code{estimator} is in \code{c("beta", "gamma", "Gy")} and 
#'   argument \code{type="heatmap"}, it prints heatmaps of the specified 
#'   estimator in \code{estimator} by a call to to function 
#'   \code{plotEstimator()} for more other arguments.
#'   \item If \code{estimator="Gy"} and argument \code{type="graph"}, it prints 
#'   a structure graph of \code{"Gy"} by a call to function \code{plotGraph()} 
#'   for more other arguments.
#'   \item If \code{estimator=c("gamma", "Gy")} and argument 
#'   \code{type="network"}, it prints the estimated network between the 
#'   response variables and predictors with nonzero coefficients by a call to 
#'   function \code{plotMCMCdiag()} for more other arguments.
#'   \item If \code{estimator=NULL} (default) and \code{type=NULL} (default), 
#'   it interactively prints the plots of estimators (i.e., beta, gamma 
#'   and (or) Gy), response graph Gy, network, Manhattan and MCMC diagnostics.
#' }
#' @param type It is one of \code{NULL}, \code{"heatmap"}, \code{"graph"}, 
#' \code{"network"}, \code{"Manhattan"} and \code{"diagnostics"}, and works by 
#' combining with argument \code{estimator}.
#' \itemize{
#'   \item If \code{type="Manhattan"} and argument \code{estimator="gamma"}, 
#'   it prints Manhattan-like plots for marginal posterior inclusion 
#'   probabilities (mPIP) and numbers of associated response variables for 
#'   individual predictors by a call to function \code{plotManhattan()} for 
#'   more other arguments.
#'   \item If \code{type="diagnostics"} and argument \code{estimator="logP"} 
#'   it shows trace plots and diagnostic density plots of a fitted model by a 
#'   call to function \code{plotMCMCdiag()} for more other arguments.
#'   \item If \code{type="diagnostics"} and argument \code{estimator="CPO"}, 
#'   it shows the conditional predictive ordinate (CPO) for each individual of 
#'   a fitted model by a call to function \code{plotCPO()} for more other arguments.
#' }
#' @param ... other arguments, see functions \code{plotEstimator()}, 
#' \code{plotGraph()}, \code{plotNetwork()}, \code{plotManhattan()}, 
#' \code{plotMCMCdiag()} or \code{plotCPO()}
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 2, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' \dontrun{
#' ## Show the interactive plots. Note that it needs at least 2000*(nbloc+1) iterations
#' ## for the diagnostic plots where nbloc=3 by default
#' # plot(fit)
#' }
#'
#' ## plot heatmaps of the estimated beta, gamma and Gy
#' plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap")
#'
#' ## plot estimated graph of responses Gy
#' plot(fit, estimator = "Gy", type = "graph")
#'
#' ## plot network between response variables and associated predictors
#' plot(fit, estimator = c("gamma", "Gy"), type = "network")
#'
#' ## print Manhattan-like plots
#' plot(fit, estimator = "gamma", type = "Manhattan")
#'
#' ## print MCMC diagnostic plots
#' #plot(fit, estimator = "logP", type = "diagnostics")
#'
#' @export
plot.BayesSUR <- function(x, estimator = NULL, type = NULL, ...) {
  if (!inherits(x, "BayesSUR")) {
    stop("Use only with \"BayesSUR\" objects")
  }

  if (!is.null(estimator)) {
    if (sum(!(estimator %in% c("beta", "gamma", "Gy", "logP", "CPO")))) {
      stop("'estimator' should be in c(NULL, 'beta', 'gamma', 'Gy', 'logP', 'CPO')!")
    }
  } else {
    if (!is.null(type)) {
      stop("If 'estimator = NULL', 'type' has to be 'NULL'!")
    }
  }

  if (!is.null(type)) {
    if (!(type %in% c("heatmap", "graph", "Manhattan", "network", "diagnostics"))) {
      stop("Please specify correct type!")
    }

    if (!(((sum(estimator %in% c("beta", "gamma", "Gy")) > 0) && (type == "heatmap")) ||
      ((length(estimator) == 1) && (estimator[1] == "Gy") && (type == "graph")) ||
      ((length(estimator) == 2) && (sum(estimator %in% c("gamma", "Gy")) == 2) && (type == "network")) ||
      ((length(estimator) == 1) && (estimator[1] == "gamma") && (type == "Manhattan")) ||
      ((length(estimator) == 1) && (estimator[1] == "logP") && (type == "diagnostics")) ||
      ((length(estimator) == 1) && (estimator[1] == "CPO") && (type == "diagnostics")))
    ) {
      stop("Please specify correct argument!")
    }

    ## refer to function plotEstimator()
    if ((sum(estimator %in% c("beta", "gamma", "Gy")) > 0) && 
        (type == "heatmap")) {
      plotEstimator(x, estimator, ...)
    }

    ## refer to function plotGraph()
    if ((length(estimator) == 1) && (estimator[1] == "Gy") && 
        (type == "graph")) {
      plotGraph(x, ...)
    }

    ## refer to function plotNetwork()
    if ((length(estimator) == 2) && 
        (sum(estimator %in% c("gamma", "Gy")) == 2) && (type == "network")) {
      plotNetwork(x, ...)
    }

    ## refer to function plotManhattan()
    if ((length(estimator) == 1) && (estimator[1] == "gamma") && 
        (type == "Manhattan")) {
      plotManhattan(x, ...)
    }

    ## refer to function plotMCMCdiag()
    if ((length(estimator) == 1) && (estimator[1] == "logP") && 
        (type == "diagnostics")) {
      plotMCMCdiag(x, ...)
    }

    ## refer to function plotCPO()
    if ((length(estimator) == 1) && (estimator[1] == "CPO") && 
        (type == "diagnostics")) {
      plotCPO(x, ...)
    }
  } else {
    if (!is.null(estimator)) {
      stop("If 'type = NULL', 'estimator' has to be 'NULL'!")
    }

    ## print plots interactively
    if (is.null(estimator[1])) {
      show <- rep(FALSE, 5)
      show[1:4] <- TRUE

      if (x$input$covariancePrior == "IG") {
        show[2:3] <- FALSE
      }

      devAskNewPage(TRUE)

      if (show[1L]) {
        dev.hold()
        plotEstimator(x, estimator = c("beta", "gamma", "Gy"), 
                      header = "\nEstimators", ...)
        dev.flush()
      }
      if (show[2L]) {
        dev.hold()
        plotGraph(x, ...)
        dev.flush()
      }
      if (show[3L]) {
        dev.hold()
        plotNetwork(x, header = "\n\nNetwork respresentation", ...)
        dev.flush()
      }
      if (show[4L]) {
        dev.hold()
        plotManhattan(x, header = "\n\nManhattan-like plots", ...)
        dev.flush()
      }
      if (show[5L]) {
        dev.hold()
        plotMCMCdiag(x, header = "\nMCMC diagnostic plots", ...)
        dev.flush()
      }

      devAskNewPage(options("device.ask.default")[[1]])
    } else {
      stop("Please specify correct argument!")
    }
  }
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @title BayesSUR_internal
#' @description
#' Run a SUR Bayesian sampler -- internal function
#' @name BayesSUR_internal
#' @param dataFile path to data file
#' @param outFilePath path to where the output is to be written
#' @param nIter number of iterations
#' @param nChains number of parallel chains to run
#'
#' NOTE THAT THIS IS BASICALLY JUST A WRAPPER
NULL

BayesSUR_internal <- function(dataFile, mrfGFile, blockFile, structureGraphFile, hyperParFile, outFilePath, nIter = 10L, burnin = 0L, nChains = 1L, covariancePrior = "HIW", gammaPrior = "hotspot", gammaSampler = "bandit", gammaInit = "MLE", betaPrior = "independent", maxThreads = 1L, tick = 1000L, output_gamma = TRUE, output_beta = TRUE, output_Gy = TRUE, output_sigmaRho = TRUE, output_pi = TRUE, output_tail = TRUE, output_model_size = TRUE, output_CPO = TRUE, output_model_visit = FALSE) {
    .Call('_BayesSUR_BayesSUR_internal', PACKAGE = 'BayesSUR', dataFile, mrfGFile, blockFile, structureGraphFile, hyperParFile, outFilePath, nIter, burnin, nChains, covariancePrior, gammaPrior, gammaSampler, gammaInit, betaPrior, maxThreads, tick, output_gamma, output_beta, output_Gy, output_sigmaRho, output_pi, output_tail, output_model_size, output_CPO, output_model_visit)
}

randU01 <- function() {
    .Call('_BayesSUR_randU01', PACKAGE = 'BayesSUR')
}

randLogU01 <- function() {
    .Call('_BayesSUR_randLogU01', PACKAGE = 'BayesSUR')
}

randIntUniform <- function(a, b) {
    .Call('_BayesSUR_randIntUniform', PACKAGE = 'BayesSUR', a, b)
}

randExponential <- function(lambda) {
    .Call('_BayesSUR_randExponential', PACKAGE = 'BayesSUR', lambda)
}

randVecExponential <- function(n, lambda) {
    .Call('_BayesSUR_randVecExponential', PACKAGE = 'BayesSUR', n, lambda)
}

randBinomial <- function(n, p) {
    .Call('_BayesSUR_randBinomial', PACKAGE = 'BayesSUR', n, p)
}

randMultinomial <- function(n, prob) {
    .Call('_BayesSUR_randMultinomial', PACKAGE = 'BayesSUR', n, prob)
}

randNormal <- function(m = 0., sigmaSquare = 1.) {
    .Call('_BayesSUR_randNormal', PACKAGE = 'BayesSUR', m, sigmaSquare)
}

randVecNormal <- function(n, m = 0., sigmaSquare = 1.) {
    .Call('_BayesSUR_randVecNormal', PACKAGE = 'BayesSUR', n, m, sigmaSquare)
}

randT <- function(nu) {
    .Call('_BayesSUR_randT', PACKAGE = 'BayesSUR', nu)
}

randVecT <- function(n, nu) {
    .Call('_BayesSUR_randVecT', PACKAGE = 'BayesSUR', n, nu)
}

randMvT <- function(nu, m, Sigma) {
    .Call('_BayesSUR_randMvT', PACKAGE = 'BayesSUR', nu, m, Sigma)
}

randGamma <- function(shape, scale) {
    .Call('_BayesSUR_randGamma', PACKAGE = 'BayesSUR', shape, scale)
}

randIGamma <- function(shape, scale) {
    .Call('_BayesSUR_randIGamma', PACKAGE = 'BayesSUR', shape, scale)
}

randWishart <- function(df, S) {
    .Call('_BayesSUR_randWishart', PACKAGE = 'BayesSUR', df, S)
}

randBeta <- function(a, b) {
    .Call('_BayesSUR_randBeta', PACKAGE = 'BayesSUR', a, b)
}

randBernoulli <- function(pi) {
    .Call('_BayesSUR_randBernoulli', PACKAGE = 'BayesSUR', pi)
}

#' @title coef method for class \code{BayesSUR}
#' @description
#' Extract the posterior mean of the coefficients of a \code{BayesSUR} class object
#' @name coef.BayesSUR
#'
#' @param object an object of class \code{BayesSUR}
#' @param beta.type type of output beta. Default is \code{marginal}, giving 
#' marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
#' estimation conditional on gamma=1.
#' @param Pmax If \code{Pmax=0.5} and \code{beta.type="conditional"}, it gives 
#' median probability model betas. Default is 0.
#' @param ... other arguments
#'
#' @return Estimated coefficients are from an object of class \code{BayesSUR}. 
#' If the \code{BayesSUR} specified data standardization, the fitted values 
#' are base based on standardized data.
#'
#' @examples
#' data("exampleQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check prediction
#' beta.hat <- coef(fit)
#'
#' @export
coef.BayesSUR <- function(object, beta.type = "marginal", Pmax = 0, ...) {
  if (!(beta.type %in% c("marginal", "conditional"))) {
    stop("Please specify correct 'beta.type'!")
  }
  if (Pmax < 0 || Pmax > 1) {
    stop("Please specify a correct argument 'Pmax' in [0,1]!")
  }
  if ((Pmax > 0) && (beta.type == "marginal")) {
    stop("Pmax > 0 is valid only if the argument beta.type = 'conditional'!")
  }
  getEstimator(object, estimator = "beta", Pmax = Pmax, 
               beta.type = beta.type, ...)
}

#' @title Simulated data set to mimic a small expression quantitative trait loci (eQTL) example
#'
#' @description
#' Simulated data set to mimic a small expression quantitative trait loci 
#' (eQTL) example, with p=150 single nucleotide polymorphisms (SNPs) as 
#' explanatory variables, s=10 gene expression features as response variables 
#' and data for n=100 observations. Loading the data will load the associated 
#' blockList object needed to fit the model with BayesSUR(). The R code for 
#' generating the simulated data is given in the Examples paragraph.
#'
#' #importFrom BDgraph rgwish
#' #importFrom gRbase mcsMAT
#' #importFrom scrime simulateSNPs
#'
#' @examples
#' # Load the eQTL sample dataset
#' data("exampleEQTL", package = "BayesSUR")
#' str(exampleEQTL)
#'
#' \dontrun{
#' # ===============
#' # The code below is to show how to generate the dataset "exampleEQTL.rda" above
#' # ===============
#'
#' requireNamespace("BDgraph", quietly = TRUE)
#' requireNamespace("gRbase", quietly = TRUE)
#' requireNamespace("scrime", quietly = TRUE)
#'
#' ########################### Problem Dimensions
#' n <- 100
#' p <- 150
#' s <- 10
#'
#' ############################ Select a set of n x p (SNPs) covariates
#'
#' ## The synthetic data in the paper use a subset of the real SNPs as covariates,
#' # but as the NFBC66 dataset is confidential we'll use scrime to sample similar data
#'
#' x <- scrime::simulateSNPs(c(n, 10), p, c(3, 2), prop.explain = c(0.9, 0.95))$data[1:n, ]
#' x <- cbind(rep(1, n), x)
#'
#' ####################################################################
#'
#' graph_pattern <- 2
#'
#' snr <- 25
#'
#' corr_param <- 0.9
#'
#' ### Create the underlying graph
#' if (graph_pattern == 1) {
#'   ### 1) Random but full
#'   G <- matrix(1, s, s)
#'   Prime <- list(c(1:s))
#'   Res <- Prime
#'   Sep <- list()
#' } else if (graph_pattern == 2) {
#'   ### 2) Block Diagonal structure
#'   Prime <- list(
#'     c(1:floor(s * 2 / 3)),
#'     c((floor(s * 2 / 3) + 1):(ceiling(s * 4 / 5) - 1)),
#'     c(ceiling(s * 4 / 5):s)
#'   )
#'
#'   Res <- Prime
#'   Sep <- lapply(Res, function(x) which(x == -99))
#'
#'   G <- matrix(0, s, s)
#'   for (i in Prime) {
#'     G[i, i] <- 1
#'   }
#' } else if (graph_pattern == 3) {
#'   ### 3) Decomposable model
#'   Prime <- list(
#'     c(1:floor(s * 5 / 12), ceiling(s * 9 / 10):s),
#'     c(floor(s * 2 / 9):(ceiling(s * 2 / 3) - 1)),
#'     c(ceiling(s * 2 / 3):(ceiling(s * 4 / 5) - 1)),
#'     c(ceiling(s * 4 / 5):s)
#'   )
#'
#'   Sep <- list()
#'   H <- list()
#'   for (i in 2:length(Prime)) {
#'     H <- union(H, Prime[[i - 1]])
#'     Sep[[i - 1]] <- intersect(H, Prime[[i]])
#'   }
#'
#'   Res <- list()
#'   Res[[1]] <- Prime[[1]]
#'   for (i in 2:length(Prime)) {
#'     Res[[i]] <- setdiff(Prime[[i]], Sep[[i - 1]])
#'   }
#'
#'   G <- matrix(0, s, s)
#'   for (i in Prime) {
#'     G[i, i] <- 1
#'   }
#'
#'   ## decomp check
#'   dimnames(G) <- list(1:s, 1:s)
#'   length(gRbase::mcsMAT(G - diag(s))) > 0
#' } else if (graph_pattern == 4) {
#'   ### 4) Non-decomposable model
#'   nblocks <- 5
#'   nElemPerBlock <- c(
#'     floor(s / 4), floor(s / 2) - 1 - floor(s / 4),
#'     ceiling(s * 2 / 3) - 1 - floor(s / 2), 7
#'   )
#'   nElemPerBlock <- c(nElemPerBlock, s - sum(nElemPerBlock))
#'   res <- 1:s
#'   blockIdx <- list()
#'   for (i in 1:nblocks) {
#'     # blockIdx[[i]] = sample(res,nElemPerBlock[i])
#'     blockIdx[[i]] <- res[1:nElemPerBlock[i]]
#'     res <- setdiff(res, blockIdx[[i]])
#'   }
#'
#'   G <- matrix(0, s, s)
#'   ## add diagonal
#'   for (i in 1:nblocks) {
#'     G[blockIdx[[i]], blockIdx[[i]]] <- 1
#'   }
#'   ## add cycle
#'   G[blockIdx[[1]], blockIdx[[2]]] <- 1
#'   G[blockIdx[[2]], blockIdx[[1]]] <- 1
#'   G[blockIdx[[1]], blockIdx[[5]]] <- 1
#'   G[blockIdx[[5]], blockIdx[[1]]] <- 1
#'   G[blockIdx[[2]], blockIdx[[3]]] <- 1
#'   G[blockIdx[[3]], blockIdx[[2]]] <- 1
#'   G[blockIdx[[3]], blockIdx[[5]]] <- 1
#'   G[blockIdx[[5]], blockIdx[[3]]] <- 1
#'
#'   ## decomp check
#'   dimnames(G) <- list(1:s, 1:s)
#'   length(gRbase::mcsMAT(G - diag(s))) > 0
#'
#'   # Prime = blockIdx
#'   Res <- blockIdx ## this is not correct but not used in the non-decomp case
#' }
#'
#' ### Gamma Pattern
#' gamma <- matrix(0, p + 1, s)
#' gamma[1, ] <- 1
#'
#'
#' ### 2) Extra Patterns
#'
#' ## outcomes (correlated in the decomp model) have some predictors in common
#' gamma[6:10, 6:9] <- 1
#'
#' ## outcomes (correlated in the decomp model) have some predictors in common
#' # gamma[16:20,14:15] = 1
#'
#' ## outcomes (sort-of correlated [pair-wise] in the decomp model)
#' # have predictors in common 6:15
#' gamma[26:30, 4:8] <- 1
#'
#' ## outcomes (NOT correlated in the decomp model) have predictors in common 16:17
#' gamma[36:40, c(3:5, 9:10)] <- 1
#'
#' ## these predictors are associated with ALL the outcomes
#' gamma[46:50, ] <- 1
#'
#' combn11 <- combn(rep((6:9 - 1) * p, each = length(6:10 - 1)) + rep(6:10 - 1, 
#'                   times = length(6:9)), 2)
#' combn31 <- combn(rep((4:8 - 1) * p, each = length(26:30 - 1)) + rep(26:30 - 1, 
#'                   times = length(4:8)), 2)
#' combn32 <- combn(rep((4:8 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
#'                   times = length(4:8)), 2)
#' combn41 <- combn(rep((3:5 - 1) * p, each = length(36:40 - 1)) + rep(36:40 - 1, 
#'                   times = length(3:5)), 2)
#' combn42 <- combn(rep((3:5 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
#'                   times = length(3:5)), 2)
#' combn51 <- combn(rep((9:10 - 1) * p, each = length(36:40 - 1)) + rep(36:40 - 1, 
#'                   times = length(9:10)), 2)
#' combn52 <- combn(rep((9:10 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
#'                   times = length(9:10)), 2)
#'
#' Gmrf <- rbind(t(combn11), t(combn31), t(combn32), t(combn41), t(combn42), t(combn51), t(combn52))
#'
#' ## get for every correlated bunch in the decomposable model,
#'
#' if (graph_pattern < 4) {
#'   # a different set of predictors
#'   for (i in 1:length(Prime)) {
#'     gamma[6:10 + (i + 6) * 10, Prime[[i]]] <- 1
#'   } ## for each Prime component
#'
#'   ## for every Residual instead
#'   for (i in 1:length(Res)) {
#'     gamma[6:10 + (i + 10) * 10, Res[[i]]] <- 1
#'   }
#' } else {
#'   for (i in 1:length(Prime)) {
#'     gamma[6:10 + (i + 4) * 10, Prime[[i]]] <- 1
#'   } ## for each Prime component
#'
#'   ## for every Residual instead
#'   for (i in 1:length(Res)) {
#'     gamma[6:10 + (i + 9) * 10, Res[[i]]] <- 1
#'   }
#' }
#'
#' #### Sample the betas
#' sd_b <- 1
#' b <- matrix(rnorm((p + 1) * s, 0, sd_b), p + 1, s)
#'
#' xb <- matrix(NA, n, s)
#'
#' for (i in 1:s) {
#'   if (sum(gamma[, i]) > 1) {
#'     xb[, i] <- x[, gamma[, i] == 1] %*% b[gamma[, i] == 1, i]
#'   } else {
#'     xb[, i] <- rep(1, n) * b[1, i]
#'   }
#' }
#'
#' ## Sample the variance
#' v_r <- mean(diag(var(xb))) / snr
#'
#' nu <- s + 1
#'
#' M <- matrix(corr_param, s, s)
#' diag(M) <- rep(1, s)
#'
#' P <- BDgraph::rgwish(n = 1, adj = G, b = 3, D = v_r * M)
#'
#' var <- solve(P)
#'
#' factor <- 10
#' factor_min <- 0.01
#' factor_max <- 1000
#' count <- 0
#' maxit <- 10000
#'
#' factor_prev <- 1
#'
#' repeat{
#'   var <- var / factor * factor_prev
#'
#'   ### Sample the errors and the Ys
#'   cVar <- chol(as.matrix(var))
#'   # err = matrix(rnorm(n*s),n,s) %*% cVar
#'   err <- matrix(rnorm(n * s, sd = 0.5), n, s) %*% cVar
#'   y <- xb + err
#'
#'   ## Reparametrisation ( assuming PEO is 1:s )
#'   cVar <- t(cVar) # make it lower-tri
#'   S <- diag(diag(cVar))
#'   sigma <- S * S
#'   L <- cVar %*% solve(S)
#'   rho <- diag(s) - solve(L)
#'
#'   ### S/N Ratio
#'   emp_snr <- mean(diag(var(xb) %*% solve(sigma)))
#'   emp_g_snr <- mean(diag(var((err) %*% t(rho)) %*% solve(sigma)))
#'
#'   ##############
#'
#'   if (abs(emp_snr - snr) < (snr / 10) | count > maxit) {
#'     break
#'   } else {
#'     if (emp_snr < snr) { # increase factor
#'       factor_min <- factor
#'     } else { # decrease factor
#'       factor_max <- factor
#'     }
#'     factor_prev <- factor
#'     factor <- (factor_min + factor_max) / 2
#'   }
#'   count <- count + 1
#' }
#'
#' #################
#' colnames(y) <- paste("GEX", 1:ncol(y), sep = "")
#' colnames(G) <- colnames(y)
#' Gy <- G
#' gamma <- gamma[-1, ]
#' mrfG <- Gmrf[!duplicated(Gmrf), ]
#' data <- cbind(y, x[, -1]) # leave out the intercept because is coded inside already
#'
#' exampleEQTL <- list(data = data, blockList = list(1:s, s + 1:p))
#'
#' ## Write data file to the user's directory by save()
#' }
#'
"exampleEQTL"

#' @title print method for class \code{BayesSUR}
#' @description
#' Print a short summary of a \code{BayesSUR} class object. It includes the 
#' argument matching information, number of selected predictors based on 
#' thresholding the posterior mean of the latent indicator variable at 0.5 
#' by default.
#' 
#' @name print.BayesSUR
#' @param x an object of class \code{BayesSUR}
#' @param Pmax threshold that truncates the estimated coefficients based on 
#' thresholding the estimated latent indicator variable. Default is 0.5
#' @param ... other arguments
#'
#' @return Return a short summary from an object of class \code{BayesSUR}, 
#' including the number of selected predictors with mPIP>\code{Pmax} and the 
#' expected log pointwise predictive density estimates (i.e., elpd.LOO and 
#' elpd.WAIC).
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
#' )
#'
#' ## check output
#' # show the print information
#' print(fit)
#'
#' @export
print.BayesSUR <- function(x, Pmax = 0.5, ...) {
  gamma <- as.matrix(read.table(paste(x$output$outFilePath, 
                                      x$output$gamma, sep = "")))

  call.string <- unlist(strsplit(deparse(x$call), ","))
  call.string[which(call.string == " ") + 1] <- 
    substr(call.string[which(call.string == " ") + 1], 4, 
           nchar(call.string[which(call.string == " ") + 1]))
  call.string <- call.string[call.string != " "]

  if (length(call.string) <= 3) {
    cat("\nCall:\n ", paste(call.string, 
                            c(rep(",", length(call.string) - 1), ""), 
                            sep = "", collapse = ""), "\n", sep = "")
  } else {
    cat("\nCall:\n ", paste(call.string[1:3], c(",", ",", ", ...)"), sep = "", 
                            collapse = ""), "\n", sep = "")
  }

  cat("\nNumber of selected predictors (mPIP > ", Pmax, "): ", 
      sum(gamma > Pmax), " of ", ncol(gamma), "x", nrow(gamma), 
      "\n", sep = "")
  cat("\nExpected log pointwise predictive density (elpd):\n", 
      " elpd.LOO = ", elpd(x, method = "LOO"), 
      ",  elpd.WAIC = ", elpd(x, method = "WAIC"), "\n\n", sep = "")
}

#' @title plot Manhattan-like plots
#' @description
#' Plot Manhattan-like plots for marginal posterior inclusion probabilities 
#' (mPIP) and numbers of responses of association for predictors of a 
#' \code{BayesSUR} class object.
#' @importFrom graphics axis box text par plot.default segments
#' @name plotManhattan
#' 
#' @param x an object of class \code{BayesSUR}
#' @param manhattan value(s) in \code{c('mPIP', 'numResponse')}. 
#' \code{manhattan='mPIP'} shows the Manhattan-like plot of the marginal 
#' posterior inclusion probabilities (mPIP). \code{manhattan='numResponse'} 
#' shows the Manhattan-like plot of the number of responses. The default is 
#' to show both figures.
#' @param x.loc a vector of features distance
#' @param axis.label a vector of predictor names which are shown in the 
#' Manhattan-like plot. The value \code{"NULL"} only showing the indices. The 
#' default \code{"auto"} show the predictor names from the original data.
#' @param mark.responses a vector of response names which are shown in the 
#' Manhattan-like plot for the mPIP
#' @param mark.pos the location of the marked text relative to the point
#' @param xlab1 a title for the x axis of Manhattan-like plot for the mPIP
#' @param ylab1 a title for the y axis of Manhattan-like plot for the mPIP
#' @param xlab2 a title for the x axis of Manhattan-like plot for the numbers of responses
#' @param ylab2 a title for the y axis of Manhattan-like plot for the numbers of responses
#' @param threshold threshold for showing number of response variables 
#' significantly associated with each feature
#' @param las graphical parameter of plot.default
#' @param cex.axis graphical parameter of plot.default
#' @param mark.color the color of the marked text. The default color is red.
#' @param mark.cex the fontsize of the marked text. The default fontsize is 0.8
#' @param header the main title
#' @param ... other arguments
#'
#' @examples
#' data("exampleEQTL", package = "BayesSUR")
#' hyperpar <- list(a_w = 2, b_w = 5)
#'
#' set.seed(9173)
#' fit <- BayesSUR(
#'   Y = exampleEQTL[["blockList"]][[1]],
#'   X = exampleEQTL[["blockList"]][[2]],
#'   data = exampleEQTL[["data"]], outFilePath = tempdir(),
#'   nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
#'   hyperpar = hyperpar, tmpFolder = "tmp/"
#' )
#'
#' ## check output
#' # show the Manhattan-like plots
#' plotManhattan(fit)
#'
#' @export
plotManhattan <- function(x, manhattan = c("mPIP", "numResponse"), 
                          x.loc = FALSE, axis.label = "auto", 
                          mark.responses = NULL, xlab1 = "Predictors", 
                          ylab1 = "mPIP", xlab2 = "Predictors", 
                          ylab2 = "No. of responses", threshold = 0.5, las = 0, 
                          cex.axis = 1, mark.pos = c(0, 0), mark.color = 2, 
                          mark.cex = 0.8, header = "", ...) {
  if (!inherits(x, "BayesSUR")) {
    stop("Use only with a \"BayesSUR\" object")
  }

  if (threshold < 0 || threshold > 1) {
    stop("Please specify orrect argument 'threshold' in [0,1]!")
  }

  x$output[-1] <- paste0(x$output$outFilePath, x$output[-1])
  gamma <- as.matrix(read.table(x$output$gamma))
  rownames(gamma) <- colnames(read.table(x$output$X, header = TRUE))

  if (is.null(axis.label)) {
    x.loc <- seq_len(nrow(gamma))
    names(x.loc) <- seq_len(nrow(gamma))
  } else {
    # name.predictors <- colnames(read.table(x$output$X,header=T))
    name.predictors <- rownames(gamma)
    if (axis.label[1] == "auto") {
      x.loc <- seq_len(nrow(gamma))
      names(x.loc) <- name.predictors
    } else {
      if ((!match(axis.label, name.predictors)[1]) & (!x.loc[1])) {
        stop("The given predictor names are not consistent with the data")
      }

      if (!x.loc[1]) x.loc <- match(axis.label, name.predictors)
      names(x.loc) <- axis.label
    }
  }

  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  if (sum(manhattan %in% c("mPIP", "numResponse")) == 2) {
    par(mfrow = c(2, 1))
  } else {
    par(mfrow = c(1, 1))
  }

  # Manhattan plot for marginal posterior inclusion probabilities (mPIP)
  if ("mPIP" %in% manhattan) {
    par(mar = c(4, 4, 4, 2))

    plot.default(as.vector(gamma) ~ 
                   rep(seq_len(nrow(gamma)), times = ncol(gamma)), 
                 xlim = c(1, nrow(gamma)), ylim = c(0, max(gamma)), 
                 xaxt = "n", bty = "n", ylab = ylab1, xlab = xlab1, 
                 main = "", pch = 19, ...)
    axis(1, at = x.loc, labels = names(x.loc), las = las, cex.axis = cex.axis)
    box()

    # mark the names of the specified response variables corresponding to the given predictors
    if (!is.null(mark.responses)) {
      name.responses <- colnames(read.table(x$output$Y, header = TRUE))
      if (!is.na(match(mark.responses, name.responses)[1])) {
        text(rep(x.loc, times = length(mark.responses)) + mark.pos[1], 
             as.vector(gamma[x.loc, name.responses %in% mark.responses]) + 
               mark.pos[2], labels = rep(mark.responses, each = length(x.loc)), 
             col = mark.color, cex = mark.cex)
      } else {
        stop("The given response names are not consistent with the data")
      }
    }
  }

  # Manhattan plot for numbers of responses
  if ("numResponse" %in% manhattan) {
    par(mar = c(6, 4, 3, 2))
    no.gamma <- rowSums(gamma >= threshold)
    plot.default(no.gamma ~ c(seq_len(nrow(gamma))), xlim = c(1, nrow(gamma)), 
                 ylim = c(0, max(no.gamma) + 0.3), type = "n", xaxt = "n", 
                 ylab = ylab2, xlab = xlab2, main = "", ...)
    segments(seq_len(nrow(gamma)), 0, seq_len(nrow(gamma)), no.gamma)
    axis(1, at = x.loc, labels = names(x.loc), las = las, cex.axis = cex.axis)

    text(nrow(gamma) / 8, -max(no.gamma) / 1.3, 
         paste("NOTE: Number of responses with mPIP >=", threshold), 
         cex = .5, xpd = NA)
  }

  title(paste0("\n\n", header), outer = TRUE)
}

---
title: "BayesSUR with random effects"
author: "Zhi Zhao"
output: rmarkdown::html_vignette

vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{BayesSUR with random effects}
  \usepackage[utf8]{inputenc}
---

```{css, echo=FALSE}
pre {
  overflow-y: auto;
}

pre[class] {
  max-height: 350px;
}
```

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
options(rmarkdown.html_vignette.check_title = FALSE)
```

The BayesSUR model has been extended to include mandatory variables by assigning Gaussian priors as random effects rather than spike-and-slab priors, named as **SSUR-MRF with random effects** in [Zhao et al. 2023](https://doi.org/10.1093/jrsssc/qlad102). 
The R code for the simulated data and real data analyses in [Zhao et al. 2023](https://doi.org/10.48550/arXiv.2101.05899) can be found at the GitHub repository [BayesSUR-RE](https://github.com/zhizuio/BayesSUR-RE). 

Here, we show some small examples to run the BayesSUR mdoel with random effects. 
To get started, load the package with

```{r, eval=TRUE}
library("BayesSUR")
```

## Simulate data

We design a network as the following figure (a) to construct a complex structure between $20$ response variables and $300$ predictors. 
It assumes that the responses are divided into six groups, and the first $120$ predictors are divided into nine groups.

![<font size="2">_Simulation scenarios: True relationships between response variables and predictors. (a) Network structure between $\mathbf Y$ and $\mathbf X$. (b) Spare latent indicator variable $\Gamma$ for the associations between $\mathbf Y$ and $\mathbf X$ in the SUR model. Black blocks indicate nonzero coefficients and white blocks indicate zero coefficients. (c) Additional structure in the residual covariance matrix between response variables not explained by $\mathbf X\mathbf B$. Black blocks indicate correlated residuals of the corresponding response variables and white blocks indicate uncorrelated residuals of the corresponding response variables._</font>](../man/figures/figure2.png){width=90%}

<br>

Load the simulation function `sim.ssur()` as follows.

```{r}
library("gRbase")
sim.ssur <- function(n, s, p, t0 = 0, seed = 123, mv = TRUE, 
                     t.df = Inf, random.intercept = 0, intercept = TRUE) {
  # set seed to fix coefficients
  set.seed(7193)
  sd_b <- 1
  mu_b <- 1
  b <- matrix(rnorm((p + ifelse(t0 == 0, 1, 0)) * s, mu_b, sd_b), p + ifelse(t0 == 0, 1, 0), s)

  # design groups and pathways of Gamma matrix
  gamma <- matrix(FALSE, p + ifelse(t0 == 0, 1, 0), s)
  if (t0 == 0) gamma[1, ] <- TRUE
  gamma[2:6 - ifelse(t0 == 0, 0, 1), 1:5] <- TRUE
  gamma[11:21 - ifelse(t0 == 0, 0, 1), 6:12] <- TRUE
  gamma[31:51 - ifelse(t0 == 0, 0, 1), 1:5] <- TRUE
  gamma[31:51 - ifelse(t0 == 0, 0, 1), 13:15] <- TRUE
  gamma[52:61 - ifelse(t0 == 0, 0, 1), 1:12] <- TRUE
  gamma[71:91 - ifelse(t0 == 0, 0, 1), 6:15] <- TRUE
  gamma[111:121 - ifelse(t0 == 0, 0, 1), 1:15] <- TRUE
  gamma[122 - ifelse(t0 == 0, 0, 1), 16:18] <- TRUE
  gamma[123 - ifelse(t0 == 0, 0, 1), 19] <- TRUE
  gamma[124 - ifelse(t0 == 0, 0, 1), 20] <- TRUE

  G_kron <- matrix(0, s * p, s * p)
  G_m <- bdiag(matrix(1, ncol = 5, nrow = 5), 
               matrix(1, ncol = 7, nrow = 7), 
               matrix(1, ncol = 8, nrow = 8))
  G_p <- bdiag(matrix(1, ncol = 5, nrow = 5), diag(3), 
               matrix(1, ncol = 11, nrow = 11), diag(9), 
               matrix(1, ncol = 21, nrow = 21), 
               matrix(1, ncol = 10, nrow = 10), diag(9), 
               matrix(1, ncol = 21, nrow = 21), diag(19), 
               matrix(1, ncol = 11, nrow = 11), diag(181))
  G_kron <- kronecker(G_m, G_p)

  combn11 <- combn(rep((1:5 - 1) * p, each = length(1:5)) + 
                     rep(1:5, times = length(1:5)), 2)
  combn12 <- combn(rep((1:5 - 1) * p, each = length(30:60)) + 
                     rep(30:60, times = length(1:5)), 2)
  combn13 <- combn(rep((1:5 - 1) * p, each = length(110:120)) + 
                     rep(110:120, times = length(1:5)), 2)
  combn21 <- combn(rep((6:12 - 1) * p, each = length(10:20)) + 
                     rep(10:20, times = length(6:12)), 2)
  combn22 <- combn(rep((6:12 - 1) * p, each = length(51:60)) + 
                     rep(51:60, times = length(6:12)), 2)
  combn23 <- combn(rep((6:12 - 1) * p, each = length(70:90)) + 
                     rep(70:90, times = length(6:12)), 2)
  combn24 <- combn(rep((6:12 - 1) * p, each = length(110:120)) + 
                     rep(110:120, times = length(6:12)), 2)
  combn31 <- combn(rep((13:15 - 1) * p, each = length(30:50)) + 
                     rep(30:50, times = length(13:15)), 2)
  combn32 <- combn(rep((13:15 - 1) * p, each = length(70:90)) + 
                     rep(70:90, times = length(13:15)), 2)
  combn33 <- combn(rep((13:15 - 1) * p, each = length(110:120)) + 
                     rep(110:120, times = length(13:15)), 2)
  combn4 <- combn(rep((16:18 - 1) * p, each = length(121)) + 
                    rep(121, times = length(16:18)), 2)
  combn5 <- matrix(rep((19 - 1) * p, each = length(122)) + 
                     rep(122, times = length(19)), nrow = 1, ncol = 2)
  combn6 <- matrix(rep((20 - 1) * p, each = length(123)) + 
                     rep(123, times = length(20)), nrow = 1, ncol = 2)

  combnAll <- rbind(t(combn11), t(combn12), t(combn13), 
                    t(combn21), t(combn22), t(combn23), t(combn24), 
                    t(combn31), t(combn32), t(combn33), 
                    t(combn4), combn5, combn6)

  set.seed(seed + 7284)
  sd_x <- 1
  x <- matrix(rnorm(n * p, 0, sd_x), n, p)

  if (t0 == 0 & intercept) x <- cbind(rep(1, n), x)
  if (!intercept) {
    gamma <- gamma[-1, ]
    b <- b[-1, ]
  }
  xb <- matrix(NA, n, s)
  if (mv) {
    for (i in 1:s) {
      if (sum(gamma[, i]) >= 1) {
        if (sum(gamma[, i]) == 1) {
          xb[, i] <- x[, gamma[, i]] * b[gamma[, i], i]
        } else {
          xb[, i] <- x[, gamma[, i]] %*% b[gamma[, i], i]
        }
      } else {
        xb[, i] <- sapply(1:s, function(i) rep(1, n) * b[1, i])
      }
    }
  } else {
    if (sum(gamma) >= 1) {
      xb <- x[, gamma] %*% b[gamma, ]
    } else {
      xb <- sapply(1:s, function(i) rep(1, n) * b[1, i])
    }
  }

  corr_param <- 0.9
  M <- matrix(corr_param, s, s)
  diag(M) <- rep(1, s)

  ## wanna make it decomposable
  Prime <- list(c(1:(s * .4), (s * .8):s), 
                c((s * .4):(s * .6)), 
                c((s * .65):(s * .75)), 
                c((s * .8):s))
  G <- matrix(0, s, s)
  for (i in 1:length(Prime)) {
    G[Prime[[i]], Prime[[i]]] <- 1
  }

  # check
  dimnames(G) <- list(1:s, 1:s)
  length(gRbase::mcsMAT(G - diag(s))) > 0

  var <- solve(BDgraph::rgwish(n = 1, adj = G, b = 3, D = M))

  # change seeds to add randomness on error
  set.seed(seed + 8493)
  sd_err <- 0.5
  if (is.infinite(t.df)) {
    err <- matrix(rnorm(n * s, 0, sd_err), n, s) %*% chol(as.matrix(var))
  } else {
    err <- matrix(rt(n * s, t.df), n, s) %*% chol(as.matrix(var))
  }

  if (t0 == 0) {
    b.re <- NA
    z <- NA
    y <- xb + err
    if (random.intercept != 0) {
      y <- y + matrix(rnorm(n * s, 0, sqrt(random.intercept)), n, s)
    }

    z <- sample(1:4, n, replace = T, prob = rep(1 / 4, 4))

    return(list(y = y, x = x, b = b, gamma = gamma, z = model.matrix(~ factor(z) + 0)[, ], 
                b.re = b.re, Gy = G, mrfG = combnAll))
  } else {
    # add random effects
    z <- t(rmultinom(n, size = 1, prob = c(.1, .2, .3, .4)))
    z <- sample(1:t0, n, replace = T, prob = rep(1 / t0, t0))
    set.seed(1683)
    b.re <- rnorm(t0, 0, 2)
    y <- matrix(b.re[z], nrow = n, ncol = s) + xb + err

    return(list(
      y = y, x = x, b = b, gamma = gamma, z = model.matrix(~ factor(z) + 0)[, ],
      b.re = b.re, Gy = G, mrfG = combnAll
    ))
  }
}
```

To simulate data with sample size $n=250$, responsible variables $s=20$ and covariates $p=300$, we can specify the corresponding parameters in the function `sim.ssur()` as follows.

```{r}
library("Matrix")
n <- 250
s <- 20
p <- 300
sim1 <- sim.ssur(n, s, p, seed = 1)
```

To simulate data from $4$ individual groups with group indicator variables following the defaul multinomial distribution $multinomial(0.1,0.2,0.3,0.4)$, we can simply add the argument `t0 = 4` in the function `sim.ssur()` as follows.

```{r}
t0 <- 4
sim2 <- sim.ssur(n, s, p, t0, seed = 1) # learning data
sim2.val <- sim.ssur(n, s, p, t0, seed=101) # validation data
```

## Run BayesSUR model with random effects

According to the guideline of prior specification in [Zhao et al. 2023](https://doi.org/10.1093/jrsssc/qlad102), we first set the following parameters `hyperpar` and then running the BayesSUR model with random effects via `betaPrior = "reGroup"` (default `betaPrior = "independent"` with spike-and-slab priors for all coefficients). 
**For illustration, we run a short MCMC** with `nIter = 300` and `burnin = 100`. 
Note that here the graph used for the Markov random field prior is the true graph from the returned object of the simulation `sim2$mrfG`.

```{r}
hyperpar <- list(mrf_d = -2, mrf_e = 1.6, a_w0 = 100, b_w0 = 500, a_w = 15, b_w = 60)
set.seed(1038)
fit2 <- BayesSUR(
  data = cbind(sim2$y, sim2$z, sim2$x),
  Y = 1:s, 
  X_0 = s + 1:t0, 
  X = s + t0 + 1:p,
  outFilePath = "sim2_mrf_re",
  hyperpar = hyperpar,
  gammaInit = "0",
  betaPrior = "reGroup",
  nIter = 300, burnin = 100,
  covariancePrior = "HIW",
  standardize = F,
  standardize.response = F,
  gammaPrior = "MRF",
  mrfG = sim2$mrfG,
  output_CPO = T
)
```
```
## BayesSUR -- Bayesian Seemingly Unrelated Regression Modelling
## Reading input files ... ... successfull!
## Clearing and initialising output files 
## Initialising the (SUR) MCMC Chain ...  ...  DONE!
## Drafting the output files with the start of the chain ... DONE! 
## 
## Starting 2 (parallel) chain(s) for 300 iterations:
## Temperature ladder updated, new temperature ratio : 1.1
##  MCMC ends.   --- Saving results and exiting
## Saved to :   sim2_mrf_re/data_SSUR_****_out.txt
## Final w0  : 4.9971
## Final w   : 2.29497
## Final tau : 0.293487    w/ proposal variance: 1.2229
## Final eta : 0.0471505
##   -- Average Omega : 0
## Final temperature ratio : 1.1
## 
## DONE, exiting!
```

Check some summarized information of the results:

```{r}
summary(fit2)
```
```
## Call:
##   BayesSUR(data = cbind(sim2$y, sim2$z, sim2$x), ...)
## 
## CPOs:
##         Min.      1st Qu.       Median      3rd Qu.         Max. 
## 0.0001896996 0.0242732651 0.0348570615 0.0465600279 0.1312571329 
## 
## Number of selected predictors (mPIP > 0.5): 19 of 20x300
## 
## Top 10 predictors on average mPIP across all responses:
##     X.74     X.69     X.77     X.82    X.114    X.116    X.122    X.157    X.265 
## 0.081840 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 0.049500 
##     X.16 
## 0.047015 
## 
## Top 10 responses on average mPIP across all predictors:
##         X.8        X.10         X.5        X.19        X.11         X.9         X.6 
## 0.012703000 0.010049000 0.007943333 0.006600000 0.005522333 0.004029667 0.003001667 
##        X.12         X.2         X.4 
## 0.002852333 0.002769667 0.002404667 
## 
## Expected log pointwise predictive density (elpd) estimates:
##   elpd.LOO = -16803.49,  elpd.WAIC = -16799.94
## 
## MCMC specification:
##   iterations = 300,  burn-in = 100,  chains = 2
##   gamma local move sampler: bandit
##   gamma initialisation: 0
## 
## Model specification:
##   covariance prior: HIW
##   gamma prior: MRF
## 
## Hyper-parameters:
##   a_w   b_w    nu a_tau b_tau a_eta b_eta mrf_d mrf_e  a_w0  b_w0 
##  15.0  60.0  22.0   0.1  10.0   0.1   1.0  -2.0   1.6 100.0 500.0
```

Show the estimates of regression coefficients, variable selection indicators and residual graph

```{r}
# show estimates
plot(fit2, estimator=c("beta","gamma","Gy"), type="heatmap", name.predictors = "auto")
```
![<font size="2">_Simulation result: Estimates of regression coefficients, variable selection indicators and residual graph._</font>](../man/figures/figureS.png){width=100%}

Compute the model performance with respect to **variable selection**

```{r}
# compute accuracy, sensitivity, specificity of variable selection
gamma <- getEstimator(fit2)
(accuracy <- sum(data.matrix(gamma > 0.5) == sim2$gamma) / prod(dim(gamma)))
```
```
## [1] 0.8725
```
```{r}
(sensitivity <- sum((data.matrix(gamma > 0.5) == 1) & (sim2$gamma == 1)) / sum(sim2$gamma == 1))
```
```
## [1] 0.01558442
```
```{r}
(specificity <- sum((data.matrix(gamma > 0.5) == 0) & (sim2$gamma == 0)) / sum(sim2$gamma == 0))
```
```
## [1] 0.9986616
```

Compute the model performance with respect to **response prediction**

```{r}
# compute RMSE and RMSPE for prediction performance
beta <- getEstimator(fit2, estimator = "beta", Pmax = .5, beta.type = "conditional")
(RMSE <- sqrt(sum((sim2$y - cbind(sim2$z, sim2$x) %*% beta)^2) / prod(dim(sim2$y))))
```
```
## [1] 8.723454
```
```{r}
(RMSPE <- sqrt(sum((sim2.val$y - cbind(sim2.val$z, sim2.val$x) %*% beta)^2) / prod(dim(sim2.val$y))))
```
```
## [1] 8.859939
```

Compute the model performance with respect to **coefficient bias**

```{r}
# compute bias of beta estimates
b <- sim2$b
b[sim2$gamma == 0] <- 0
(beta.l2 <- sqrt(sum((beta[-c(1:4), ] - b)^2) / prod(dim(b))))
```
```
## [1] 0.5039502
```

Compute the model performance with respect to **covariance selection**

```{r}
g.re <- getEstimator(fit2, estimator = "Gy")
(g.accuracy <- sum((g.re > 0.5) == sim2$Gy) / prod(dim(g.re)))
```
```
## [1] 0.585
```
```{r}
(g.sensitivity <- sum(((g.re > 0.5) == sim2$Gy)[sim2$Gy == 1]) / sum(sim2$Gy == 1))
```
```
## [1] 0.2475248
```
```{r}
(g.specificity <- sum(((g.re > 0.5) == sim2$Gy)[sim2$Gy == 0]) / sum(sim2$Gy == 0))
```
```
## [1] 0.9292929
```


## Referrence

> Zhi Zhao, Marco Banterle, Alex Lewin, Manuela Zucknick (2023).
> Multivariate Bayesian structured variable selection for pharmacogenomic studies.
> _Journal of the Royal Statistical Society: Series C (Applied Statistics)_, qlad102. DOI: [10.1093/jrsssc/qlad102](https://doi.org/10.1093/jrsssc/qlad102).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.BayesSUR.R
\name{print.BayesSUR}
\alias{print.BayesSUR}
\title{print method for class \code{BayesSUR}}
\usage{
\method{print}{BayesSUR}(x, Pmax = 0.5, ...)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{Pmax}{threshold that truncates the estimated coefficients based on 
thresholding the estimated latent indicator variable. Default is 0.5}

\item{...}{other arguments}
}
\value{
Return a short summary from an object of class \code{BayesSUR}, 
including the number of selected predictors with mPIP>\code{Pmax} and the 
expected log pointwise predictive density estimates (i.e., elpd.LOO and 
elpd.WAIC).
}
\description{
Print a short summary of a \code{BayesSUR} class object. It includes the 
argument matching information, number of selected predictors based on 
thresholding the posterior mean of the latent indicator variable at 0.5 
by default.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
)

## check output
# show the print information
print(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.BayesSUR.R
\name{coef.BayesSUR}
\alias{coef.BayesSUR}
\title{coef method for class \code{BayesSUR}}
\usage{
\method{coef}{BayesSUR}(object, beta.type = "marginal", Pmax = 0, ...)
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{beta.type}{type of output beta. Default is \code{marginal}, giving 
marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
estimation conditional on gamma=1.}

\item{Pmax}{If \code{Pmax=0.5} and \code{beta.type="conditional"}, it gives 
median probability model betas. Default is 0.}

\item{...}{other arguments}
}
\value{
Estimated coefficients are from an object of class \code{BayesSUR}. 
If the \code{BayesSUR} specified data standardization, the fitted values 
are base based on standardized data.
}
\description{
Extract the posterior mean of the coefficients of a \code{BayesSUR} class object
}
\examples{
data("exampleQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check prediction
beta.hat <- coef(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotNetwork.R
\name{plotNetwork}
\alias{plotNetwork}
\title{plot network representation of the associations between responses and predictors}
\usage{
plotNetwork(
  x,
  includeResponse = NULL,
  excludeResponse = NULL,
  includePredictor = NULL,
  excludePredictor = NULL,
  MatrixGamma = NULL,
  PmaxPredictor = 0.5,
  PmaxResponse = 0.5,
  nodesizePredictor = 2,
  nodesizeResponse = 15,
  no.isolates = FALSE,
  lineup = 1.2,
  gray.alpha = 0.6,
  edgewith.response = 5,
  edgewith.predictor = 2,
  edge.weight = FALSE,
  label.predictor = NULL,
  label.response = NULL,
  color.predictor = NULL,
  color.response = NULL,
  name.predictors = NULL,
  name.responses = NULL,
  color.edge.positive = "gray",
  color.edge.negative = "gray",
  vertex.frame.color = NA,
  layoutInCircle = FALSE,
  header = "",
  beta.type = "conditional",
  ...
)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{includeResponse}{A vector of the response names which are shown in the network}

\item{excludeResponse}{A vector of the response names which are not shown in the network}

\item{includePredictor}{A vector of the predictor names which are shown in the network}

\item{excludePredictor}{A vector of the predictor names which are not shown in the network}

\item{MatrixGamma}{A matrix or dataframe of the latent indicator variable.
Default is \code{NULL} and to extrate it from object of class inheriting
from an object of class \code{BayesSUR}}

\item{PmaxPredictor}{cutpoint for thresholding the estimated latent
indicator variable. Default is 0.5}

\item{PmaxResponse}{cutpoint for thresholding the learning structure matrix
of multiple response variables. Default is 0.5}

\item{nodesizePredictor}{node size of Predictors in the output graph.
Default is 15}

\item{nodesizeResponse}{node size of response variables in the output graph.
Default is 25}

\item{no.isolates}{remove isolated nodes from responses graph and full
graph, may get problem if there are also isolated Predictors}

\item{lineup}{A ratio of the heights between responses' area and predictors'}

\item{gray.alpha}{the opacity. The default is 0.6}

\item{edgewith.response}{the edge width between response nodes}

\item{edgewith.predictor}{the edge width between the predictor and response node}

\item{edge.weight}{draw weighted edges after thresholding at 0.5. The
default value \code{FALSE} is not to draw weighted edges}

\item{label.predictor}{A vector of the names of predictors}

\item{label.response}{A vector of the names of response variables}

\item{color.predictor}{color of the predictor nodes}

\item{color.response}{color of the response nodes}

\item{name.predictors}{A subtitle for the predictors}

\item{name.responses}{A subtitle for the responses}

\item{color.edge.positive}{color of the edges corresponding to estimated
positive regression coefficients}

\item{color.edge.negative}{color of the edges corresponding to estimated
negative regression coefficients}

\item{vertex.frame.color}{color of the frame of the vertices. If you don't
want vertices to have a frame, supply NA as the color name}

\item{layoutInCircle}{place vertices on a circle, in the order of their
vertex ids. The default is \code{FALSE}}

\item{header}{the main title}

\item{beta.type}{the type of output beta. Default is \code{marginal}, giving
marginal beta estimation. If \code{beta.type="conditional"}, it gives beta
estimation conditional on gamma=1}

\item{...}{other arguments}
}
\description{
Plot the network representation of the associations between responses and
predictors, based on the estimated gamma matrix and graph of responses
from a "BayesSUR" class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
# draw network representation of the associations between responses and covariates
plotNetwork(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotManhattan.R
\name{plotManhattan}
\alias{plotManhattan}
\title{plot Manhattan-like plots}
\usage{
plotManhattan(
  x,
  manhattan = c("mPIP", "numResponse"),
  x.loc = FALSE,
  axis.label = "auto",
  mark.responses = NULL,
  xlab1 = "Predictors",
  ylab1 = "mPIP",
  xlab2 = "Predictors",
  ylab2 = "No. of responses",
  threshold = 0.5,
  las = 0,
  cex.axis = 1,
  mark.pos = c(0, 0),
  mark.color = 2,
  mark.cex = 0.8,
  header = "",
  ...
)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{manhattan}{value(s) in \code{c('mPIP', 'numResponse')}. 
\code{manhattan='mPIP'} shows the Manhattan-like plot of the marginal 
posterior inclusion probabilities (mPIP). \code{manhattan='numResponse'} 
shows the Manhattan-like plot of the number of responses. The default is 
to show both figures.}

\item{x.loc}{a vector of features distance}

\item{axis.label}{a vector of predictor names which are shown in the 
Manhattan-like plot. The value \code{"NULL"} only showing the indices. The 
default \code{"auto"} show the predictor names from the original data.}

\item{mark.responses}{a vector of response names which are shown in the 
Manhattan-like plot for the mPIP}

\item{xlab1}{a title for the x axis of Manhattan-like plot for the mPIP}

\item{ylab1}{a title for the y axis of Manhattan-like plot for the mPIP}

\item{xlab2}{a title for the x axis of Manhattan-like plot for the numbers of responses}

\item{ylab2}{a title for the y axis of Manhattan-like plot for the numbers of responses}

\item{threshold}{threshold for showing number of response variables 
significantly associated with each feature}

\item{las}{graphical parameter of plot.default}

\item{cex.axis}{graphical parameter of plot.default}

\item{mark.pos}{the location of the marked text relative to the point}

\item{mark.color}{the color of the marked text. The default color is red.}

\item{mark.cex}{the fontsize of the marked text. The default fontsize is 0.8}

\item{header}{the main title}

\item{...}{other arguments}
}
\description{
Plot Manhattan-like plots for marginal posterior inclusion probabilities 
(mPIP) and numbers of responses of association for predictors of a 
\code{BayesSUR} class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
# show the Manhattan-like plots
plotManhattan(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGraph.R
\name{plotGraph}
\alias{plotGraph}
\title{plot graph for response variables}
\usage{
plotGraph(
  x,
  Pmax = 0.5,
  main = "Estimated graph of responses",
  edge.width = 2,
  edge.weight = FALSE,
  vertex.label = NULL,
  vertex.label.color = "black",
  vertex.size = 30,
  vertex.color = "dodgerblue",
  vertex.frame.color = NA,
  ...
)
}
\arguments{
\item{x}{either an object of class \code{BayesSUR} (default) or a symmetric 
numeric matrix representing an adjacency matrix for a given graph structure.
If x is an adjacency matrix, argument \code{main="Given graph of responses"} 
by default.}

\item{Pmax}{a value for thresholding the learning structure matrix of 
multiple response variables. Default is 0.5}

\item{main}{an overall title for the plot}

\item{edge.width}{edge width. Default is 2}

\item{edge.weight}{draw weighted edges after thresholding at 0.5. The 
default value \code{FALSE} is not to draw weighted edges}

\item{vertex.label}{character vector used to label the nodes}

\item{vertex.label.color}{label color. Default is \code{"black"}}

\item{vertex.size}{node size. Default is 30}

\item{vertex.color}{node color. Default is \code{"dodgerblue"}}

\item{vertex.frame.color}{node color. Default is \code{"NA"}}

\item{...}{other arguments}
}
\description{
Plot the estimated graph for multiple response variables from a 
\code{BayesSUR} class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
# show the graph relationship between responses
plotGraph(fit, estimator = "Gy")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elpd.R
\name{elpd}
\alias{elpd}
\title{expected log pointwise predictive density}
\usage{
elpd(object, method = "LOO")
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{method}{the name of the prediction accuracy index. Default is the 
\code{"LOO"} (Bayesian LOO estimate of out-of-sample predictive fit). The 
other index is the \code{"WAIC"} (widely applicable information criterion).
For the HRR models, both "\code{LOO}" and "\code{WAIC}" are computed based 
on the multivate t-distribution of the posterior predictive rather than 
approximation of importance sampling.}
}
\value{
Return the predictiion accuracy measure from an object of class 
\code{BayesSUR}. It is elpd.loo if the argumnet \code{method="LOO"} and 
elpd.WAIC if \code{method="WAIC"}.
}
\description{
Measure the prediction accuracy by the elpd (expected log pointwise 
predictive density). The out-of-sample predictive fit can either be 
estimated by Bayesian leave-one-out cross-validation (LOO) or by widely 
applicable information criterion (WAIC) (Vehtari et al. 2017).
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
)

## check output
# print prediction accuracy elpd (expected log pointwise predictive density)
# by the Bayesian LOO estimate of out-of-sample predictive fit
elpd(fit, method = "LOO")

}
\references{
Vehtari, A., Gelman, A., Gabry, J. (2017). \emph{Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC.} Statistics and Computing, 27(5): 1413–1432.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.BayesSUR.R
\name{summary.BayesSUR}
\alias{summary.BayesSUR}
\title{summary method for class \code{BayesSUR}}
\usage{
\method{summary}{BayesSUR}(object, Pmax = 0.5, ...)
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{Pmax}{threshold that truncates the estimated coefficients based on 
thresholding the estimated latent indicator variable. Default is 0.5}

\item{...}{other arguments}
}
\value{
Return a result summary from an object of class \code{BayesSUR}, 
including the CPOs, number of selected predictors with mPIP>\code{Pmax}, 
top 10 predictors on average mPIP across all responses, top 10 responses on 
average mPIP across all predictors, Expected log pointwise predictive 
density (elpd) estimates, MCMC specification, model specification (i.e., 
covariance prior and gamma prior) and hyper-parameters.
}
\description{
Summary method for class \code{BayesSUR}. It includes the argument matching 
information, Top predictors/responses on average mPIP across all 
responses/predictors, elpd estimates, MCMC specification, model 
specification and hyper-parameters. The summarized number of the selected 
variable corresponds to the posterior mean of the latent indicator variable 
thresholding at 0.5 by default.
}
\examples{
data(exampleEQTL, package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
)

## check output
# show the summary information
summary(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMCMCdiag.R
\name{plotMCMCdiag}
\alias{plotMCMCdiag}
\title{plot MCMC diagnostic plots}
\usage{
plotMCMCdiag(x, nbloc = 3, HIWg = NULL, header = "", ...)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{nbloc}{number of splits for the last half iterations after substracting 
burn-in length}

\item{HIWg}{diagnostic plot of the response graph. Default is \code{NULL}. 
\code{HIW="degree"} prints the diagnostic of the degrees of response nodes. 
\code{HIW="edges"} prints the diagnostic of every edge between two responses. 
\code{HIW="lik"} prints the diagnostic of the posterior likelihoods of the 
hyperparameters related to the response relationships}

\item{header}{the main title}

\item{...}{other arguments for the plots of the log-likelihood and model size}
}
\description{
Show trace plots and diagnostic density plots of a fitted model object of class \code{BayesSUR}.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
plotMCMCdiag(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleEQTL.R
\docType{data}
\name{exampleEQTL}
\alias{exampleEQTL}
\title{Simulated data set to mimic a small expression quantitative trait loci (eQTL) example}
\format{
An object of class \code{list} of length 4.
}
\usage{
exampleEQTL
}
\description{
Simulated data set to mimic a small expression quantitative trait loci 
(eQTL) example, with p=150 single nucleotide polymorphisms (SNPs) as 
explanatory variables, s=10 gene expression features as response variables 
and data for n=100 observations. Loading the data will load the associated 
blockList object needed to fit the model with BayesSUR(). The R code for 
generating the simulated data is given in the Examples paragraph.

#importFrom BDgraph rgwish
#importFrom gRbase mcsMAT
#importFrom scrime simulateSNPs
}
\examples{
# Load the eQTL sample dataset
data("exampleEQTL", package = "BayesSUR")
str(exampleEQTL)

\dontrun{
# ===============
# The code below is to show how to generate the dataset "exampleEQTL.rda" above
# ===============

requireNamespace("BDgraph", quietly = TRUE)
requireNamespace("gRbase", quietly = TRUE)
requireNamespace("scrime", quietly = TRUE)

########################### Problem Dimensions
n <- 100
p <- 150
s <- 10

############################ Select a set of n x p (SNPs) covariates

## The synthetic data in the paper use a subset of the real SNPs as covariates,
# but as the NFBC66 dataset is confidential we'll use scrime to sample similar data

x <- scrime::simulateSNPs(c(n, 10), p, c(3, 2), prop.explain = c(0.9, 0.95))$data[1:n, ]
x <- cbind(rep(1, n), x)

####################################################################

graph_pattern <- 2

snr <- 25

corr_param <- 0.9

### Create the underlying graph
if (graph_pattern == 1) {
  ### 1) Random but full
  G <- matrix(1, s, s)
  Prime <- list(c(1:s))
  Res <- Prime
  Sep <- list()
} else if (graph_pattern == 2) {
  ### 2) Block Diagonal structure
  Prime <- list(
    c(1:floor(s * 2 / 3)),
    c((floor(s * 2 / 3) + 1):(ceiling(s * 4 / 5) - 1)),
    c(ceiling(s * 4 / 5):s)
  )

  Res <- Prime
  Sep <- lapply(Res, function(x) which(x == -99))

  G <- matrix(0, s, s)
  for (i in Prime) {
    G[i, i] <- 1
  }
} else if (graph_pattern == 3) {
  ### 3) Decomposable model
  Prime <- list(
    c(1:floor(s * 5 / 12), ceiling(s * 9 / 10):s),
    c(floor(s * 2 / 9):(ceiling(s * 2 / 3) - 1)),
    c(ceiling(s * 2 / 3):(ceiling(s * 4 / 5) - 1)),
    c(ceiling(s * 4 / 5):s)
  )

  Sep <- list()
  H <- list()
  for (i in 2:length(Prime)) {
    H <- union(H, Prime[[i - 1]])
    Sep[[i - 1]] <- intersect(H, Prime[[i]])
  }

  Res <- list()
  Res[[1]] <- Prime[[1]]
  for (i in 2:length(Prime)) {
    Res[[i]] <- setdiff(Prime[[i]], Sep[[i - 1]])
  }

  G <- matrix(0, s, s)
  for (i in Prime) {
    G[i, i] <- 1
  }

  ## decomp check
  dimnames(G) <- list(1:s, 1:s)
  length(gRbase::mcsMAT(G - diag(s))) > 0
} else if (graph_pattern == 4) {
  ### 4) Non-decomposable model
  nblocks <- 5
  nElemPerBlock <- c(
    floor(s / 4), floor(s / 2) - 1 - floor(s / 4),
    ceiling(s * 2 / 3) - 1 - floor(s / 2), 7
  )
  nElemPerBlock <- c(nElemPerBlock, s - sum(nElemPerBlock))
  res <- 1:s
  blockIdx <- list()
  for (i in 1:nblocks) {
    # blockIdx[[i]] = sample(res,nElemPerBlock[i])
    blockIdx[[i]] <- res[1:nElemPerBlock[i]]
    res <- setdiff(res, blockIdx[[i]])
  }

  G <- matrix(0, s, s)
  ## add diagonal
  for (i in 1:nblocks) {
    G[blockIdx[[i]], blockIdx[[i]]] <- 1
  }
  ## add cycle
  G[blockIdx[[1]], blockIdx[[2]]] <- 1
  G[blockIdx[[2]], blockIdx[[1]]] <- 1
  G[blockIdx[[1]], blockIdx[[5]]] <- 1
  G[blockIdx[[5]], blockIdx[[1]]] <- 1
  G[blockIdx[[2]], blockIdx[[3]]] <- 1
  G[blockIdx[[3]], blockIdx[[2]]] <- 1
  G[blockIdx[[3]], blockIdx[[5]]] <- 1
  G[blockIdx[[5]], blockIdx[[3]]] <- 1

  ## decomp check
  dimnames(G) <- list(1:s, 1:s)
  length(gRbase::mcsMAT(G - diag(s))) > 0

  # Prime = blockIdx
  Res <- blockIdx ## this is not correct but not used in the non-decomp case
}

### Gamma Pattern
gamma <- matrix(0, p + 1, s)
gamma[1, ] <- 1


### 2) Extra Patterns

## outcomes (correlated in the decomp model) have some predictors in common
gamma[6:10, 6:9] <- 1

## outcomes (correlated in the decomp model) have some predictors in common
# gamma[16:20,14:15] = 1

## outcomes (sort-of correlated [pair-wise] in the decomp model)
# have predictors in common 6:15
gamma[26:30, 4:8] <- 1

## outcomes (NOT correlated in the decomp model) have predictors in common 16:17
gamma[36:40, c(3:5, 9:10)] <- 1

## these predictors are associated with ALL the outcomes
gamma[46:50, ] <- 1

combn11 <- combn(rep((6:9 - 1) * p, each = length(6:10 - 1)) + rep(6:10 - 1, 
                  times = length(6:9)), 2)
combn31 <- combn(rep((4:8 - 1) * p, each = length(26:30 - 1)) + rep(26:30 - 1, 
                  times = length(4:8)), 2)
combn32 <- combn(rep((4:8 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
                  times = length(4:8)), 2)
combn41 <- combn(rep((3:5 - 1) * p, each = length(36:40 - 1)) + rep(36:40 - 1, 
                  times = length(3:5)), 2)
combn42 <- combn(rep((3:5 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
                  times = length(3:5)), 2)
combn51 <- combn(rep((9:10 - 1) * p, each = length(36:40 - 1)) + rep(36:40 - 1, 
                  times = length(9:10)), 2)
combn52 <- combn(rep((9:10 - 1) * p, each = length(46:50 - 1)) + rep(46:50 - 1, 
                  times = length(9:10)), 2)

Gmrf <- rbind(t(combn11), t(combn31), t(combn32), t(combn41), t(combn42), t(combn51), t(combn52))

## get for every correlated bunch in the decomposable model,

if (graph_pattern < 4) {
  # a different set of predictors
  for (i in 1:length(Prime)) {
    gamma[6:10 + (i + 6) * 10, Prime[[i]]] <- 1
  } ## for each Prime component

  ## for every Residual instead
  for (i in 1:length(Res)) {
    gamma[6:10 + (i + 10) * 10, Res[[i]]] <- 1
  }
} else {
  for (i in 1:length(Prime)) {
    gamma[6:10 + (i + 4) * 10, Prime[[i]]] <- 1
  } ## for each Prime component

  ## for every Residual instead
  for (i in 1:length(Res)) {
    gamma[6:10 + (i + 9) * 10, Res[[i]]] <- 1
  }
}

#### Sample the betas
sd_b <- 1
b <- matrix(rnorm((p + 1) * s, 0, sd_b), p + 1, s)

xb <- matrix(NA, n, s)

for (i in 1:s) {
  if (sum(gamma[, i]) > 1) {
    xb[, i] <- x[, gamma[, i] == 1] \%*\% b[gamma[, i] == 1, i]
  } else {
    xb[, i] <- rep(1, n) * b[1, i]
  }
}

## Sample the variance
v_r <- mean(diag(var(xb))) / snr

nu <- s + 1

M <- matrix(corr_param, s, s)
diag(M) <- rep(1, s)

P <- BDgraph::rgwish(n = 1, adj = G, b = 3, D = v_r * M)

var <- solve(P)

factor <- 10
factor_min <- 0.01
factor_max <- 1000
count <- 0
maxit <- 10000

factor_prev <- 1

repeat{
  var <- var / factor * factor_prev

  ### Sample the errors and the Ys
  cVar <- chol(as.matrix(var))
  # err = matrix(rnorm(n*s),n,s) \%*\% cVar
  err <- matrix(rnorm(n * s, sd = 0.5), n, s) \%*\% cVar
  y <- xb + err

  ## Reparametrisation ( assuming PEO is 1:s )
  cVar <- t(cVar) # make it lower-tri
  S <- diag(diag(cVar))
  sigma <- S * S
  L <- cVar \%*\% solve(S)
  rho <- diag(s) - solve(L)

  ### S/N Ratio
  emp_snr <- mean(diag(var(xb) \%*\% solve(sigma)))
  emp_g_snr <- mean(diag(var((err) \%*\% t(rho)) \%*\% solve(sigma)))

  ##############

  if (abs(emp_snr - snr) < (snr / 10) | count > maxit) {
    break
  } else {
    if (emp_snr < snr) { # increase factor
      factor_min <- factor
    } else { # decrease factor
      factor_max <- factor
    }
    factor_prev <- factor
    factor <- (factor_min + factor_max) / 2
  }
  count <- count + 1
}

#################
colnames(y) <- paste("GEX", 1:ncol(y), sep = "")
colnames(G) <- colnames(y)
Gy <- G
gamma <- gamma[-1, ]
mrfG <- Gmrf[!duplicated(Gmrf), ]
data <- cbind(y, x[, -1]) # leave out the intercept because is coded inside already

exampleEQTL <- list(data = data, blockList = list(1:s, s + 1:p))

## Write data file to the user's directory by save()
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{BayesSUR_internal}
\alias{BayesSUR_internal}
\title{BayesSUR_internal}
\arguments{
\item{dataFile}{path to data file}

\item{outFilePath}{path to where the output is to be written}

\item{nIter}{number of iterations}

\item{nChains}{number of parallel chains to run

NOTE THAT THIS IS BASICALLY JUST A WRAPPER}
}
\description{
Run a SUR Bayesian sampler -- internal function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getEstimator.R
\name{getEstimator}
\alias{getEstimator}
\title{extract the posterior mean of parameters}
\usage{
getEstimator(object, estimator = "gamma", Pmax = 0, beta.type = "marginal")
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{estimator}{the name of one estimator. Default is the latent indicator 
estimator "\code{gamma}". Other options "\code{beta}", "\code{Gy}", 
"\code{CPO}" and "\code{logP}"
correspond the marginal (conditional) coefficient matrix if 
\code{beta.type="marginal"}(\code{"conditional"}), response graph and 
conditional predictive ordinate (CPO) respectively}

\item{Pmax}{threshold that truncate the estimator "\code{gamma}" or 
"\code{Gy}". Default is \code{0}. If \code{Pmax=0.5} and 
\code{beta.type="conditional"}, it gives median probability model betas}

\item{beta.type}{the type of output beta. Default is \code{marginal}, giving 
marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
estimation conditional on gamma=1}
}
\value{
Return the estimator from an object of class \code{BayesSUR}. It is 
a matrix if the length of argument \code{marginal} is greater than 1. 
Otherwise, it is a list
}
\description{
Extract the posterior mean of the parameters of a \code{BayesSUR} class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
# extract the posterior mean of the coefficients matrix
beta_hat <- getEstimator(fit, estimator = "beta")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEstimator.R
\name{plotEstimator}
\alias{plotEstimator}
\title{plot heatmap of estimators}
\usage{
plotEstimator(
  x,
  estimator = NULL,
  colorScale.gamma = grey((100:0)/100),
  colorScale.beta = c("blue", "white", "red"),
  legend.cex.axis = 1,
  name.responses = NA,
  name.predictors = NA,
  xlab = "",
  ylab = "",
  fig.tex = FALSE,
  output = "ParamEstimator",
  header = "",
  header.cex = 2,
  tick = FALSE,
  mgp = c(2.5, 1, 0),
  cex.main = 1.5,
  title.beta = NA,
  title.gamma = NA,
  title.Gy = NA,
  beta.type = "marginal",
  Pmax = 0,
  ...
)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{estimator}{print the heatmap of estimators. The value "beta" is for 
the estimated coefficients matrix, "gamma" for the latent indicator matrix 
and "Gy" for the graph of responses}

\item{colorScale.gamma}{value palette for gamma}

\item{colorScale.beta}{a vector of three colors for diverging color schemes}

\item{legend.cex.axis}{magnification of axis annotation relative to cex}

\item{name.responses}{a vector of the response names. The default is 
\code{NA} only to show the locations. The value \code{"auto"} show the 
response names from the orginal data.}

\item{name.predictors}{a vector of the predictor names. The default is 
\code{NA} only to show the locations. The value \code{"auto"} show the 
predictor names from the orginal data.}

\item{xlab}{a title for the x axis}

\item{ylab}{a title for the y axis}

\item{fig.tex}{print the figure through LaTex. Default is \code{FALSE}}

\item{output}{the file name of printed figure}

\item{header}{the main title}

\item{header.cex}{size of the main title for all estimators}

\item{tick}{a logical value specifying whether tickmarks and an axis line 
should be drawn. Default is \code{FALSE}}

\item{mgp}{the margin line (in mex units) for the axis title, axis labels 
and axis line}

\item{cex.main}{size of the title for each estimator}

\item{title.beta}{a title for the printed "beta"}

\item{title.gamma}{a title for the printed "gamma"}

\item{title.Gy}{a title for the printed "Gy"}

\item{beta.type}{the type of output beta. Default is \code{marginal}, giving 
marginal beta estimation. If \code{beta.type="conditional"}, it gives beta 
estimation conditional on gamma=1}

\item{Pmax}{threshold that truncate the estimator "\code{gamma}" or 
"\code{Gy}". Default is \code{0}. If \code{Pmax=0.5} and 
\code{beta.type="conditional"}, it gives median probability model betas}

\item{...}{other arguments}
}
\description{
Plot the posterior mean estimators from a \code{BayesSUR} class object, 
including the coefficients beta, latent indicator variable gamma and 
graph of responses.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
# Plot the estimators from the fitted object
plotEstimator(fit, estimator = c("beta", "gamma", "Gy"))

\dontrun{
## Set up temporary work directory for saving a pdf figure
# td <- tempdir()
# oldwd <- getwd()
# setwd(td)

## Produce authentic math formulas in the graph
# plotEstimator(fit, estimator = c("beta", "gamma", "Gy"), fig.tex = TRUE)
# system(paste(getOption("pdfviewer"), "ParamEstimator.pdf"))
# setwd(oldwd)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.BayesSUR.R
\name{predict.BayesSUR}
\alias{predict.BayesSUR}
\title{predict method for class \code{BayesSUR}}
\usage{
\method{predict}{BayesSUR}(object, newx, type = "response", beta.type = "marginal", Pmax = 0, ...)
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{newx}{Matrix of new values for x at which predictions are to be made}

\item{type}{Type of prediction required. \code{type="response"} gives the 
fitted responses; \code{type="coefficients"} returns the estimated 
coefficients depending on the arguments \code{beta.type} and \code{Pmax}.
\code{type="nonzero"} returns a list of the indices of the nonzero 
coefficients corresponding to the estimated latent indicator variable 
thresholding at \code{Pmax}}

\item{beta.type}{the type of estimated coefficients beta for prediction. 
Default is \code{marginal}, giving marginal beta estimation. If 
\code{beta.type="conditional"}, it gives conditional beta estimation}

\item{Pmax}{If \code{type="nonzero"}, it is a threshold for the estimated 
latent indicator variable. If \code{type="coefficients"}, 
\code{beta.type="conditional"} and \code{Pmax=0.5}, it gives median 
probability model betas. Default is 0}

\item{...}{other arguments}
}
\value{
Predicted values extracted from an object of class \code{BayesSUR}. 
If the \code{BayesSUR} specified data standardization, the fitted values 
are base based on standardized data.
}
\description{
Predict responses corresponding to the posterior mean of the coefficients, 
return posterior mean of coefficients or indices of nonzero coefficients of 
a \code{BayesSUR} class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 20, burnin = 10, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check prediction
predict.val <- predict(fit, newx = exampleEQTL[["blockList"]][[2]])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCPO.R
\name{plotCPO}
\alias{plotCPO}
\title{plot conditional predictive ordinate}
\usage{
plotCPO(
  x,
  outlier.mark = TRUE,
  outlier.thresh = 0.01,
  scale.CPO = TRUE,
  x.loc = FALSE,
  axis.label = NULL,
  las = 0,
  cex.axis = 1,
  mark.pos = c(0, -0.01),
  mark.color = 2,
  mark.cex = 0.8,
  xlab = "Observations",
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{outlier.mark}{mark the outliers with the response names. 
The default is \code{FALSE}}

\item{outlier.thresh}{threshold for the CPOs. The default is 0.01.}

\item{scale.CPO}{scaled CPOs which is divided by their maximum. 
The default is \code{TRUE}}

\item{x.loc}{a vector of features distance}

\item{axis.label}{a vector of predictor names which are shown in CPO plot. 
The default is \code{NULL} only showing the indices. The value \code{"auto"} 
show the predictor names from the original data.}

\item{las}{graphical parameter of plot.default}

\item{cex.axis}{graphical parameter of plot.default}

\item{mark.pos}{location of the marked text relative to the point}

\item{mark.color}{color of the marked text. The default color is red}

\item{mark.cex}{font size of the marked text. The default font size is 0.8}

\item{xlab}{a title for the x axis}

\item{ylab}{a title for the y axis}

\item{...}{other arguments}
}
\description{
Plot the conditional predictive ordinate (CPO) for each individual of a 
fitted model generated by \code{BayesSUR} which is a \code{BayesSUR} object. 
CPO is a handy posterior predictive check because it may be used to identify 
outliers, influential observations, and for hypothesis testing across 
different non-nested models (Gelfand 1996).
}
\details{
The default threshold for the CPOs to detect the outliers is 0.01 
by Congdon (2005). It can be tuned by the argument \code{outlier.thresh}.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
)

## check output
# plot the conditional predictive ordinate (CPO)
plotCPO(fit)

}
\references{
Statisticat, LLC (2013). \emph{Bayesian Inference.} Farmington, CT: Statisticat, LLC.

Gelfand A. (1996). \emph{Model Determination Using Sampling Based Methods}. In Gilks W., Richardson S., Spiegelhalter D. (eds.), Markov Chain Monte Carlo in Practice, pp. 145–161. Chapman & Hall, Boca Raton, FL.

Congdon P. (2005). \emph{Bayesian Models for Categorical Data}. John Wiley & Sons, West Sussex, England.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesSUR.R
\name{BayesSUR}
\alias{BayesSUR}
\alias{BayesSUR-package}
\title{Fitting BayesSUR models}
\usage{
BayesSUR(
  data = NULL,
  Y,
  X,
  X_0 = NULL,
  covariancePrior = "HIW",
  gammaPrior = "hotspot",
  betaPrior = "independent",
  nIter = 10000,
  burnin = 5000,
  nChains = 2,
  outFilePath = "",
  gammaSampler = "bandit",
  gammaInit = "R",
  mrfG = NULL,
  standardize = TRUE,
  standardize.response = TRUE,
  maxThreads = 1,
  tick = 1000,
  output_gamma = TRUE,
  output_beta = TRUE,
  output_Gy = TRUE,
  output_sigmaRho = TRUE,
  output_pi = TRUE,
  output_tail = TRUE,
  output_model_size = TRUE,
  output_model_visit = FALSE,
  output_CPO = FALSE,
  output_Y = TRUE,
  output_X = TRUE,
  hyperpar = list(),
  tmpFolder = "tmp/"
)
}
\arguments{
\item{data}{a numeric matrix with variables on the columns and observations 
on the rows, if arguments \code{Y} and \code{X} (and possibly \code{X_0}) 
are vectors. Can be \code{NULL} if arguments \code{Y} and \code{X} (and 
possibly \code{X_0}) are numeric matrices}

\item{Y, X}{vectors of indices (with respect to the data matrix) for the 
outcomes (\code{Y}) and the predictors to select (\code{X}) respectively; 
if the \code{data} argument is \code{NULL}, these needs to be numeric 
matrices containing the data instead, with variables on the columns and 
observations on the rows}

\item{X_0}{vectors of indices (with respect to the data matrix) for the 
fixed predictors that are not selected, i.e. always included in the model; 
if the data argument is not provided, this needs to be a numeric matrix 
containing the data instead, with variables on the columns and observations 
on the rows}

\item{covariancePrior}{string indicating the prior for the covariance $C$; 
it has to be either \code{HIW} for the hyper-inverse-Wishar (which will 
result in a sparse covariance matrix), \code{IW} for the inverse-Wishart 
prior (dense covariance) or \code{IG} for independent inverse-Gamma on all 
the diagonal elements and 0 otherwise. See the details for the model 
specification}

\item{gammaPrior}{string indicating the gamma prior to use, either 
\code{hotspot} (default) for the Hotspot prior of Bottolo (2011), \code{MRF} 
for the Markov Random Field prior or \code{hierarchical} for a simpler 
hierarchical prior. See the details for the model specification}

\item{betaPrior}{string indicating the prior for regression coefficients; it 
has to be either \code{independent} for independent spike-and-slab priors 
(only slab part for \code{X_0} if specified), or \code{reGroup} for weakly 
normal priors for mandatory variables (random effects) and spike-and-slab 
priors for other variables of Zhao (2023)}

\item{nIter}{number of iterations for the MCMC procedure. Default 10000}

\item{burnin}{number of iterations to discard at the start of the chain. 
Default is 5000}

\item{nChains}{number of parallel tempered chains to run (default 2). The 
temperature is adapted during the burnin phase}

\item{outFilePath}{path to where the output files are to be written}

\item{gammaSampler}{string indicating the type of sampler for gamma, either 
\code{bandit} for the Thompson sampling inspired samper or \code{MC3} for 
the usual MC^3 sampler.  See Russo et al.(2018) or Madigan and York (1995) 
for details}

\item{gammaInit}{gamma initialisation to either all-zeros (\code{0}), all 
ones (\code{1}), MLE-informed (\code{MLE}) or (default) randomly (\code{R})}

\item{mrfG}{either a matrix or a path to the file containing (the edge list 
of) the G matrix for the MRF prior on gamma (if necessary)}

\item{standardize}{logical flag for X variable standardization. Default is 
\code{standardize=TRUE}. Coefficients are returned on the standardized scale}

\item{standardize.response}{logical flag for Y standardization. Default is 
\code{standardize.response=TRUE}}

\item{maxThreads}{maximum threads used for parallelization. Default is 1. 
Reproducibility of results with \code{set.seed()} is only guaranteed if 
\code{maxThreads=1}}

\item{tick}{an integer used for printing the iteration index and some updated 
parameters every tick-th iteration. Default is 1000}

\item{output_gamma}{allow (\code{TRUE}) or suppress (\code{FALSE}) the 
output for  gamma. See the return value below for more information}

\item{output_beta}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for beta. See the return value below for more information}

\item{output_Gy}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for Gy. See the return value below for more information}

\item{output_sigmaRho}{allow (\code{TRUE}) or suppress (\code{FALSE}) the 
output for sigmaRho. See the return value below for more information}

\item{output_pi}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for pi. See the return value below for more information}

\item{output_tail}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for tail (hotspot tail probability). See the return value below for more 
information}

\item{output_model_size}{allow (\code{TRUE}) or suppress (\code{FALSE}) the 
output for model_size. See the return value below for more information}

\item{output_model_visit}{allow (\code{TRUE}) or suppress (\code{FALSE}) the 
output for all visited models over the MCMC iterations. Default is 
\code{FALSE}. See the return value below for more information}

\item{output_CPO}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for (scaled) conditional predictive ordinates (\code{*_CPO_out.txt}),
CPO with joint posterior predictive of the response variables 
(\code{*_CPOsumy_out.txt}) and widely applicable information criterion 
(\code{*_WAIC_out.txt}). See the return value below for more information}

\item{output_Y}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for responses dataset Y}

\item{output_X}{allow (\code{TRUE}) or suppress (\code{FALSE}) the output 
for predictors dataset X}

\item{hyperpar}{a list of named hypeparameters to use instead of the default 
values. Valid names are mrf_d, mrf_e, a_sigma, b_sigma, a_tau, b_tau, nu, 
a_eta, b_eta, a_o, b_o, a_pi, b_pi, a_w and b_w. Their default values are 
a_w=2, b_w=5, a_omega=2, b_omega=1, a_o=2, b_o=p-2, a_pi=2, b_pi=1, nu=s+2, 
a_tau=0.1, b_tau=10, a_eta=0.1, b_eta=1, a_sigma=1, b_sigma=1, mrf_d=-3 and 
mrf_e=0.03. See the vignette for more information}

\item{tmpFolder}{the path to a temporary folder where intermediate data 
files are stored (will be erased at the end of the chain). It is specified 
relative to \code{outFilePath}}
}
\value{
An object of class \code{BayesSUR} is saved as 
\code{obj_BayesSUR.RData} in the output file, including the following 
components:
\itemize{
\item status - the running status
\item input - a list of all input parameters by the user
\item output - a list of the all output filenames:
\itemize{
\item "\code{*_logP_out.txt}" - contains each row for the \eqn{1000t}-th iteration's log-likelihoods of parameters, i.e., Tau, Eta, JunctionTree, SigmaRho, O, Pi, Gamma, W, Beta and data conditional log-likelihood depending on the models.
\item "\code{*_gamma_out.txt}" - posterior mean of the latent indicator matrix.
\item "\code{*_pi_out.txt}" - posterior mean of the predictor effects (prospensity) by decomposing the probability of the latent indicator.
\item "\code{*_hotspot_tail_p_out.txt}" - posterior mean of the hotspot tail probability. Only available for the hotspot prior on the gamma.
\item "\code{*_beta_out.txt}" - posterior mean of the coefficients matrix.
\item "\code{*_Gy_out.txt}" - posterior mean of the response graph. Only available for the HIW prior on the covariance.
\item "\code{*_sigmaRho_out.txt}" - posterior mean of the transformed parameters. Not available for the IG prior on the covariance.
\item "\code{*_model_size_out.txt}" - contains each row for the\eqn{1000t}-th iteration's model sizes of the multiple response variables.
\item "\code{*_model_visit_gy_out.txt}" - contains each row for the nonzero indices of the vectorized estimated graph matrix for each iteration.
\item "\code{*_model_visit_gamma_out.txt}" - contains each row for the nonzero indices of the vectorized estimated gamma matrix for each iteration.
\item "\code{*_CPO_out.txt}" - the (scaled) conditional predictive ordinates (CPO).
\item "\code{*_CPOsumy_out.txt}" - the (scaled) conditional predictive ordinates (CPO) with joint posterior predictive of the response variables.
\item "\code{*_WAIC_out.txt}" - the widely applicable information criterion (WAIC).
\item "\code{*_Y.txt}" - responses dataset.
\item "\code{*_X.txt}" - predictors dataset.
\item "\code{*_X0.txt}" - fixed predictors dataset.
}
\item call - the matched call.
}
}
\description{
Main function of the package. Fits a range of models introduced in the 
package vignette \code{BayesSUR.pdf}. Returns an object of S3 class 
\code{BayesSUR}. There are three options for the prior on the residual 
covariance matrix (i.e., independent inverse-Gamma, inverse-Wishart and 
hyper-inverse Wishart) and three options for the prior on the latent 
indicator variable (i.e., independent Bernoulli, hotspot and Markov random 
field). So there are nine models in total. See details for their combinations.
}
\details{
The arguments \code{covariancePrior} and \code{gammaPrior} specify 
the model HRR, dSUR or SSUR with different gamma prior. Let 
\eqn{\gamma_{jk}} be latent indicator variable of each coefficient and 
\eqn{C} be covariance matrix of response variables. The nine models 
specified through the arguments \code{covariancePrior} and 
\code{gammaPrior} are as follows.
\tabular{cccc}{
                \tab \eqn{\gamma_{jk}}~Bernoulli \tab \eqn{\gamma_{jk}}~hotspot \tab \eqn{\gamma}~MRF \cr
  \eqn{C}~indep \tab HRR-B                       \tab HRR-H                     \tab HRR-M           \cr
  \eqn{C}~IW   \tab dSUR-B                      \tab dSUR-H                    \tab dSUR-M          \cr
  \eqn{C}~HIW   \tab SSUR-B                      \tab SSUR-H                    \tab SSUR-M
}
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)
set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 5, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/", output_CPO = TRUE
)

## check output
# show the summary information
summary(fit)

# show the estimated beta, gamma and graph of responses Gy
plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap")

\dontrun{
## Set up temporary work directory for saving a pdf figure
# td <- tempdir()
# oldwd <- getwd()
# setwd(td)

## Produce authentic math formulas in the graph
# plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap", fig.tex = TRUE)
# system(paste(getOption("pdfviewer"), "ParamEstimator.pdf"))
# setwd(oldwd)
}

}
\references{
Russo D, Van Roy B, Kazerouni A, Osband I, Wen Z (2018). \emph{A tutorial on Thompson sampling.} Foundations and Trends in Machine Learning, 11: 1-96.

Madigan D, York J (1995). \emph{Bayesian graphical models for discrete data.} International Statistical Review, 63: 215–232.

Bottolo L, Banterle M, Richardson S, Ala-Korpela M, Jarvelin MR, Lewin A (2020). \emph{A computationally efficient Bayesian seemingly unrelated regressions model for high-dimensional quantitative trait loci discovery.} Journal of Royal Statistical Society: Series C, 70: 886-908.

Zhao Z, Banterle M, Bottolo L, Richardson S, Lewin A, Zucknick M (2021). \emph{BayesSUR: An R package for high-dimensional multivariate Bayesian variable and covariance selection in linear regression.} Journal of Statistical Software, 100: 1–32.

Zhao Z, Banterle M, Lewin A, Zucknick M (2023). \emph{Multivariate Bayesian structured variable selection for pharmacogenomic studies.} Journal of the Royal Statistical Society: Series C (Applied Statistics), qlad102.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleGDSC.R
\docType{data}
\name{exampleGDSC}
\alias{exampleGDSC}
\title{Preprocessed data set to mimic a small pharmacogenomic example}
\format{
An object of class \code{list} of length 3.
}
\usage{
exampleGDSC
}
\description{
Preprocessed data set to mimic a small pharmacogenetic example from the 
Genomics of Drug Sensitivity in Cancer (GDSC) database, with p=850 gene 
features as explanatory variables, s=7 drugs sensitivity data as response 
variables and data for n=498 cell lines. Gene features include p1=343 gene 
expression features (GEX), p2=426 by copy number variations (CNV) and p3=68 
mutated genes (MUT). Loading the data will load the associated blockList 
(and mrfG) objects needed to fit the model with BayesSUR(). The R code for 
generating the simulated data is given in the Examples paragraph.

#importFrom plyr mapvalues
#importFrom data.table like
}
\examples{
# Load the GDSC sample dataset
data("exampleGDSC", package = "BayesSUR")
str(exampleGDSC)

\dontrun{
# ===============
# This code below is to do preprocessing of GDSC data and obtain the complete dataset
# "exampleGDSC.rda" above. The user needs load the datasets from
# https://cellmodelpassports.sanger.ac.uk/downloads.
# But downloading and transforming the three used datasets below to *.csv files first.
# ===============

requireNamespace("plyr", quietly = TRUE)
requireNamespace("data.table", quietly = TRUE)


features <- data.frame(read.csv("/gdsc_en_input_w5.csv", head = T))
names.fea <- strsplit(rownames(features), "")
features <- t(features)
p <- c(13321, 13747 - 13321, 13818 - 13747)
Cell.Line <- rownames(features)
features <- data.frame(Cell.Line, features)

ic50_00 <- data.frame(read.csv("gdsc_drug_sensitivity_fitted_data_w5.csv", head = T))
ic50_0 <- ic50_00[, c(1, 4, 7)]
drug.id <- data.frame(read.csv("gdsc_tissue_output_w5.csv", head = T))[, c(1, 3)]
drug.id2 <- drug.id[!duplicated(drug.id$drug.id), ]
# delete drug.id=1066 since ID1066 and ID156 both correspond drug AZD6482,
# and no ID1066 in the "suppl.Data1" by Garnett et al. (2012)
drug.id2 <- drug.id2[drug.id2$drug.id != 1066, ]
drug.id2$drug.name <- as.character(drug.id2$drug.name)
drug.id2$drug.name <- substr(drug.id2$drug.name, 1, nchar(drug.id2$drug.name) - 6)
drug.id2$drug.name <- gsub(" ", "-", drug.id2$drug.name)

ic50 <- ic50_0
# mapping the drug_id to drug names in drug sensitivity data set
ic50$drug_id <- plyr::mapvalues(ic50$drug_id, from = drug.id2[, 2], to = drug.id2[, 1])
colnames(ic50) <- c("Cell.Line", "compound", "IC50")

# transform drug sensitivity overall cell lines to a data matrix
y0 <- reshape(ic50, v.names = "IC50", timevar = "compound", 
              idvar = "Cell.Line", direction = "wide")
y0$Cell.Line <- gsub("-", ".", y0$Cell.Line)

# ===============
# select nonmissing pharmacological data
# ===============
y00 <- y0
m0 <- dim(y0)[2] - 1
eps <- 0.05
# r1.na is better to be not smaller than r2.na
r1.na <- 0.3
r2.na <- 0.2
k <- 1
while (sum(is.na(y0[, 2:(1 + m0)])) > 0) {
  r1.na <- r1.na - eps / k
  r2.na <- r1.na - eps / k
  k <- k + 1
  ## select drugs with <30\% (decreasing with k) missing data overall cell lines
  na.y <- apply(y0[, 2:(1 + m0)], 2, function(xx) sum(is.na(xx)) / length(xx))
  while (sum(na.y < r1.na) < m0) {
    y0 <- y0[, -c(1 + which(na.y >= r1.na))]
    m0 <- sum(na.y < r1.na)
    na.y <- apply(y0[, 2:(1 + m0)], 2, function(xx) sum(is.na(xx)) / length(xx))
  }

  ## select cell lines with treatment of at least 80\% (increasing with k) drugs
  na.y0 <- apply(y0[, 2:(1 + m0)], 1, function(xx) sum(is.na(xx)) / length(xx))
  while (sum(na.y0 < r2.na) < (dim(y0)[1])) {
    y0 <- y0[na.y0 < r2.na, ]
    na.y0 <- apply(y0[, 2:(1 + m0)], 1, function(xx) sum(is.na(xx)) / length(xx))
  }
  num.na <- sum(is.na(y0[, 2:(1 + m0)]))
  message("#{NA}=", num.na, "\n", "r1.na =", r1.na, ", r2.na =", r2.na, "\n")
}

# ===============
# combine drug sensitivity, tissues and molecular features
# ===============
yx <- merge(y0, features, by = "Cell.Line")
names.cell.line <- yx$Cell.Line
names.drug <- colnames(yx)[2:(dim(y0)[2])]
names.drug <- substr(names.drug, 6, nchar(names.drug))
# numbers of gene expression features, copy number festures and muatation features
p <- c(13321, 13747 - 13321, 13818 - 13747)
num.nonpen <- 13
yx <- data.matrix(yx[, -1])
y <- yx[, 1:(dim(y0)[2] - 1)]
x <- cbind(yx[, dim(y0)[2] - 1 + sum(p) + 1:num.nonpen], yx[, dim(y0)[2] - 1 + 1:sum(p)])

# delete genes with only one mutated cell line
x <- x[, 
  -c(num.nonpen + p[1] + p[2] + which(colSums(x[, num.nonpen + p[1] + p[2] + 1:p[3]]) <= 1))]
p[3] <- ncol(x) - num.nonpen - p[1] - p[2]

GDSC <- list(
  y = y, x = x, p = p, num.nonpen = num.nonpen, names.cell.line = names.cell.line,
  names.drug = names.drug
)


## ================
## ================
## select a small set of drugs
## ================
## ================

name_drugs <- c(
  "Methotrexate", "RDEA119", "PD-0325901", "CI-1040", "AZD6244", "Nilotinib",
  "Axitinib"
)

# extract the drugs' pharmacological profiling and tissue dummy
YX0 <- cbind(GDSC$y[, colnames(GDSC$y) \%in\% paste("IC50.", name_drugs, sep = "")]
[, c(1, 3, 6, 4, 7, 2, 5)], GDSC$x[, 1:GDSC$num.nonpen])
colnames(YX0) <- c(name_drugs, colnames(GDSC$x)[1:GDSC$num.nonpen])
# extract the genetic information of CNV & MUT
X23 <- GDSC$x[, GDSC$num.nonpen + GDSC$p[1] + 1:(p[2] + p[3])]
colnames(X23)[1:p[2]] <- paste(substr(
  colnames(X23)[1:p[2]], 1,
  nchar(colnames(X23)[1:p[2]]) - 3
), ".CNV", sep = "")

# locate all genes with CNV or MUT information
name_genes_duplicate <- c(
  substr(colnames(X23)[1:p[2]], 1, nchar(colnames(X23)[1:p[2]]) - 4),
  substr(colnames(X23)[p[2] + 1:p[3]], 1, nchar(colnames(X23)[p[2] + 1:p[3]]) - 4)
)
name_genes <- name_genes_duplicate[!duplicated(name_genes_duplicate)]

# select the GEX which have the common genes with CNV or MUT
X1 <- 
  GDSC$x[, GDSC$num.nonpen + which(colnames(GDSC$x)[GDSC$num.nonpen + 1:p[1]] \%in\% name_genes)]

p[1] <- ncol(X1)
X1 <- log(X1)

# summary the data information
exampleGDSC <- list(data = cbind(YX0, X1, X23))
exampleGDSC$blockList <- list(
  1:length(name_drugs), length(name_drugs) + 1:GDSC$num.nonpen,
  ncol(YX0) + 1:sum(p)
)

# ========================
# construct the G matrix: edge potentials in the MRF prior
# ========================

# edges between drugs: Group1 ("RDEA119","17-AAG","PD-0325901","CI-1040" and "AZD6244")
# indexed as (2:5)
# http://software.broadinstitute.org/gsea/msigdb/cards/KEGG_MAPK_SIGNALING_PATHWAY
pathway_genes <- read.table("MAPK_pathway.txt")[[1]]
Idx_Pathway1 <- which(c(colnames(X1), name_genes_duplicate) \%in\% pathway_genes)
Gmrf_Group1Pathway1 <- t(combn(rep(Idx_Pathway1, each = length(2:5)) +
  rep((2:5 - 1) * sum(p), times = length(Idx_Pathway1)), 2))

# edges between drugs: Group2 ("Nilotinib","Axitinib") indexed as (6:7)
# delete gene ABL2
Idx_Pathway2 <- which(c(colnames(X1), name_genes_duplicate) \%like\% "BCR" |
  c(colnames(X1), name_genes_duplicate) \%like\% "ABL")[-c(3, 5)]
Gmrf_Group2Pathway2 <- t(combn(rep(Idx_Pathway2, each = length(6:7)) +
  rep((6:7 - 1) * sum(p), times = length(Idx_Pathway2)), 2))

# edges between the common gene in different data sources
Gmrf_CommonGene <- NULL
list_CommonGene <- list(0)
k <- 1
for (i in 1:length(name_genes)) {
  Idx_CommonGene <- which(c(colnames(X1), name_genes_duplicate) == name_genes[i])
  if (length(Idx_CommonGene) > 1) {
    Gmrf_CommonGene <- rbind(Gmrf_CommonGene, 
    t(combn(rep(Idx_CommonGene, each = length(name_drugs))
    + rep((1:length(name_drugs) - 1) * sum(p), times = length(Idx_CommonGene)), 2)))
    k <- k + 1
  }
}
Gmrf_duplicate <- rbind(Gmrf_Group1Pathway1, Gmrf_Group2Pathway2, Gmrf_CommonGene)
Gmrf <- Gmrf_duplicate[!duplicated(Gmrf_duplicate), ]
exampleGDSC$mrfG <- Gmrf

# create the target gene names of the two groups of drugs
targetGenes1 <- matrix(Idx_Pathway1, nrow = 1)
colnames(targetGenes1) <- colnames(exampleGDSC$data)[seq_along(targetGene$group1)]
targetGenes2 <- matrix(Idx_Pathway2, nrow = 1)
colnames(targetGenes2) <- colnames(exampleGDSC$data)[seq_along(targetGene$group2)]

targetGene <- list(group1 = targetGenes1, group2 = targetGenes2)

## Write data file exampleGDSC.rda to the user's directory by save()
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted.BayesSUR.R
\name{fitted.BayesSUR}
\alias{fitted.BayesSUR}
\title{get fitted responses}
\usage{
\method{fitted}{BayesSUR}(object, Pmax = 0, beta.type = "marginal", ...)
}
\arguments{
\item{object}{an object of class \code{BayesSUR}}

\item{Pmax}{valid if \code{beta.type="conditional"}. If 
\code{beta.type="conditional"} and \code{Pmax=0.5}, it gives median 
probability model betas. Default is 0}

\item{beta.type}{type of estimated beta for the fitted model. Default is 
\code{marginal}, giving marginal beta estimation. If 
\code{beta.type="conditional"}, it gives beta estimation conditional 
on gamma=1}

\item{...}{other arguments}
}
\value{
Fitted values extracted from an object of class \code{BayesSUR}. If 
the \code{BayesSUR} specified data standardization, the fitted values are 
base based on standardized data.
}
\description{
Return the fitted response values that correspond to the posterior mean 
estimates from a \code{BayesSUR} class object.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 10, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check fitted values
fitted.val <- fitted(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.BayesSUR.R
\name{plot.BayesSUR}
\alias{plot.BayesSUR}
\title{create a selection of plots}
\usage{
\method{plot}{BayesSUR}(x, estimator = NULL, type = NULL, ...)
}
\arguments{
\item{x}{an object of class \code{BayesSUR}}

\item{estimator}{It is in \code{c(NULL, 'beta', 'gamma', 'Gy', 'logP', 'CPO')} 
and works by combining with argument \code{type}.
\itemize{
  \item If \code{estimator} is in \code{c("beta", "gamma", "Gy")} and 
  argument \code{type="heatmap"}, it prints heatmaps of the specified 
  estimator in \code{estimator} by a call to to function 
  \code{plotEstimator()} for more other arguments.
  \item If \code{estimator="Gy"} and argument \code{type="graph"}, it prints 
  a structure graph of \code{"Gy"} by a call to function \code{plotGraph()} 
  for more other arguments.
  \item If \code{estimator=c("gamma", "Gy")} and argument 
  \code{type="network"}, it prints the estimated network between the 
  response variables and predictors with nonzero coefficients by a call to 
  function \code{plotMCMCdiag()} for more other arguments.
  \item If \code{estimator=NULL} (default) and \code{type=NULL} (default), 
  it interactively prints the plots of estimators (i.e., beta, gamma 
  and (or) Gy), response graph Gy, network, Manhattan and MCMC diagnostics.
}}

\item{type}{It is one of \code{NULL}, \code{"heatmap"}, \code{"graph"}, 
\code{"network"}, \code{"Manhattan"} and \code{"diagnostics"}, and works by 
combining with argument \code{estimator}.
\itemize{
  \item If \code{type="Manhattan"} and argument \code{estimator="gamma"}, 
  it prints Manhattan-like plots for marginal posterior inclusion 
  probabilities (mPIP) and numbers of associated response variables for 
  individual predictors by a call to function \code{plotManhattan()} for 
  more other arguments.
  \item If \code{type="diagnostics"} and argument \code{estimator="logP"} 
  it shows trace plots and diagnostic density plots of a fitted model by a 
  call to function \code{plotMCMCdiag()} for more other arguments.
  \item If \code{type="diagnostics"} and argument \code{estimator="CPO"}, 
  it shows the conditional predictive ordinate (CPO) for each individual of 
  a fitted model by a call to function \code{plotCPO()} for more other arguments.
}}

\item{...}{other arguments, see functions \code{plotEstimator()}, 
\code{plotGraph()}, \code{plotNetwork()}, \code{plotManhattan()}, 
\code{plotMCMCdiag()} or \code{plotCPO()}}
}
\description{
plot method for class \code{BayesSUR}. This is the main plot function to be 
called by the user. This function calls one or several of the following 
functions: \code{plotEstimator()}, \code{plotGraph()}, \code{plotMCMCdiag()}, 
\code{plotManhattan()}, \code{plotNetwork()}, \code{plotCPO()}.
}
\examples{
data("exampleEQTL", package = "BayesSUR")
hyperpar <- list(a_w = 2, b_w = 5)

set.seed(9173)
fit <- BayesSUR(
  Y = exampleEQTL[["blockList"]][[1]],
  X = exampleEQTL[["blockList"]][[2]],
  data = exampleEQTL[["data"]], outFilePath = tempdir(),
  nIter = 2, burnin = 0, nChains = 1, gammaPrior = "hotspot",
  hyperpar = hyperpar, tmpFolder = "tmp/"
)

## check output
\dontrun{
## Show the interactive plots. Note that it needs at least 2000*(nbloc+1) iterations
## for the diagnostic plots where nbloc=3 by default
# plot(fit)
}

## plot heatmaps of the estimated beta, gamma and Gy
plot(fit, estimator = c("beta", "gamma", "Gy"), type = "heatmap")

## plot estimated graph of responses Gy
plot(fit, estimator = "Gy", type = "graph")

## plot network between response variables and associated predictors
plot(fit, estimator = c("gamma", "Gy"), type = "network")

## print Manhattan-like plots
plot(fit, estimator = "gamma", type = "Manhattan")

## print MCMC diagnostic plots
#plot(fit, estimator = "logP", type = "diagnostics")

}
